"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const PromiseUtils_1 = require("./PromiseUtils");
const Enums_1 = require("./consts/Enums");
const TypedEventEmitter_1 = __importDefault(require("./consts/TypedEventEmitter"));
class BaseClient extends TypedEventEmitter_1.default {
    constructor(logger, symbol) {
        super();
        this.logger = logger;
        this.symbol = symbol;
        this.RECONNECT_INTERVAL = 5000;
        this.status = Enums_1.ClientStatus.Disconnected;
        this.setClientStatus = (status) => {
            if (this.status === status) {
                return;
            }
            this.status = status;
            (status === Enums_1.ClientStatus.Connected ? this.logger.info : this.logger.error)(`${this.getName()} status changed: ${status}`);
            this.emit('status.changed', status);
        };
        this.raceCall = (promise, raceHandler, raceTimeout) => {
            return (0, PromiseUtils_1.racePromise)(promise, (reject) => {
                this.setClientStatus(Enums_1.ClientStatus.Disconnected);
                raceHandler(reject);
            }, raceTimeout);
        };
        this.clearReconnectTimer = () => {
            if (this.reconnectionTimer) {
                clearInterval(this.reconnectionTimer);
                this.reconnectionTimer = undefined;
            }
        };
        this.getName = () => `${this.serviceName()}-${this.symbol}`;
    }
    isConnected() {
        return this.status === Enums_1.ClientStatus.Connected;
    }
    isDisconnected() {
        return this.status === Enums_1.ClientStatus.Disconnected;
    }
    isOutOfSync() {
        return this.status === Enums_1.ClientStatus.OutOfSync;
    }
}
exports.default = BaseClient;
//# sourceMappingURL=BaseClient.js.map