/// <reference types="node" />
import { Secp256k1ZKP } from '@vulpemventures/secp256k1-zkp';
import { BIP32Interface } from 'bip32';
import { Network, Transaction, TxOutput } from 'bitcoinjs-lib';
import { ClaimDetails, Musig, RefundDetails, Types } from 'boltz-core';
import { LiquidClaimDetails, LiquidRefundDetails } from 'boltz-core/dist/lib/liquid';
import { ECPairInterface } from 'ecpair';
import { Transaction as LiquidTransaction, TxOutput as LiquidTxOutput } from 'liquidjs-lib';
import { Network as LiquidNetwork } from 'liquidjs-lib/src/networks';
import { IChainClient } from './chain/ChainClient';
import { CurrencyType } from './consts/Enums';
import Swap from './db/models/Swap';
import SwapOutputType from './swap/SwapOutputType';
import Wallet from './wallet/Wallet';
import { Currency } from './wallet/WalletManager';
type UnblindedOutput = Omit<LiquidTxOutput, 'value'> & {
    value: number;
    isLbtc: boolean;
};
export declare let zkp: Secp256k1ZKP;
export declare const setup: () => Promise<void>;
export declare const parseTransaction: (type: CurrencyType, rawTx: string | Buffer) => Transaction | LiquidTransaction;
export declare const fromOutputScript: (type: CurrencyType, outputScript: Buffer, network: Network | LiquidNetwork) => string;
export declare const toOutputScript: (type: CurrencyType, toDecode: string, network: Network | LiquidNetwork) => Buffer;
export declare const unblindOutput: (wallet: Wallet, output: LiquidTxOutput, blindingKey?: Buffer) => UnblindedOutput;
export declare const getOutputValue: (wallet: Wallet, output: TxOutput | LiquidTxOutput) => number;
export declare const constructClaimDetails: (swapOutputType: SwapOutputType, wallet: Wallet, swap: Swap, transaction: Transaction | LiquidTransaction, preimage: Buffer, cooperative?: boolean) => ClaimDetails | LiquidClaimDetails;
export declare const constructClaimTransaction: (wallet: Wallet, claimDetails: ClaimDetails[] | LiquidClaimDetails[], destinationAddress: string, feePerVbyte: number) => LiquidTransaction | Transaction;
export declare const constructRefundTransaction: (wallet: Wallet, refundDetails: RefundDetails[] | LiquidRefundDetails[], destinationAddress: string, timeoutBlockHeight: number, feePerVbyte: number) => LiquidTransaction | Transaction;
export declare const calculateTransactionFee: (chainClient: IChainClient, transaction: Transaction | LiquidTransaction) => Promise<number>;
export declare const createMusig: (ourKeys: ECPairInterface | BIP32Interface, theirPublicKey: Buffer) => Musig;
export declare const tweakMusig: (type: CurrencyType, musig: Musig, swapTree: Types.SwapTree) => Buffer;
export declare const hashForWitnessV1: (currency: Currency, tx: Transaction | LiquidTransaction, index: number, leafHash?: Buffer, hashType?: number) => Promise<Buffer>;
export { ClaimDetails, RefundDetails, LiquidClaimDetails, LiquidRefundDetails, UnblindedOutput, };
