"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hashForWitnessV1 = exports.tweakMusig = exports.createMusig = exports.calculateTransactionFee = exports.constructRefundTransaction = exports.constructClaimTransaction = exports.constructClaimDetails = exports.getOutputValue = exports.unblindOutput = exports.toOutputScript = exports.fromOutputScript = exports.parseTransaction = exports.setup = exports.zkp = void 0;
const secp256k1_zkp_1 = __importDefault(require("@vulpemventures/secp256k1-zkp"));
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const boltz_core_1 = require("boltz-core");
const liquid_1 = require("boltz-core/dist/lib/liquid");
const crypto_1 = require("crypto");
const liquidjs_lib_1 = require("liquidjs-lib");
const ecc = __importStar(require("tiny-secp256k1"));
const ECPairHelper_1 = require("./ECPairHelper");
const Utils_1 = require("./Utils");
const Enums_1 = require("./consts/Enums");
const LiquidTypes_1 = require("./consts/LiquidTypes");
let confi;
const setup = async () => {
    (0, boltz_core_1.init)(ecc);
    (0, bitcoinjs_lib_1.initEccLib)(ecc);
    exports.zkp = await (0, secp256k1_zkp_1.default)();
    confi = new liquidjs_lib_1.confidential.Confidential(exports.zkp);
    (0, liquid_1.init)(exports.zkp);
};
exports.setup = setup;
const parseTransaction = (type, rawTx) => {
    if (rawTx instanceof Buffer) {
        return isBitcoin(type)
            ? bitcoinjs_lib_1.Transaction.fromBuffer(rawTx)
            : liquidjs_lib_1.Transaction.fromBuffer(rawTx);
    }
    else {
        return isBitcoin(type)
            ? bitcoinjs_lib_1.Transaction.fromHex(rawTx)
            : liquidjs_lib_1.Transaction.fromHex(rawTx);
    }
};
exports.parseTransaction = parseTransaction;
const fromOutputScript = (type, outputScript, network) => {
    return isBitcoin(type)
        ? bitcoinjs_lib_1.address.fromOutputScript(outputScript, network)
        : liquidjs_lib_1.address.fromOutputScript(outputScript, network);
};
exports.fromOutputScript = fromOutputScript;
const toOutputScript = (type, toDecode, network) => {
    return isBitcoin(type)
        ? bitcoinjs_lib_1.address.toOutputScript(toDecode, network)
        : liquidjs_lib_1.address.toOutputScript(toDecode, network);
};
exports.toOutputScript = toOutputScript;
const unblindOutput = (wallet, output, blindingKey) => {
    let value = 0;
    let asset = output.asset;
    if (output.rangeProof?.length !== 0) {
        if (blindingKey !== undefined) {
            const unblinded = confi.unblindOutputWithKey(output, blindingKey);
            value = Number(unblinded.value);
            asset = unblinded.asset;
        }
    }
    else {
        value = liquidjs_lib_1.confidential.confidentialValueToSatoshi(output.value);
        asset = output.asset;
        // Remove the un-confidential prefix
        asset = asset.slice(1, asset.length);
    }
    asset = (0, Utils_1.reverseBuffer)(asset);
    return {
        ...output,
        value: value,
        asset: asset,
        isLbtc: asset.equals((0, Utils_1.getHexBuffer)(wallet.network.assetHash)),
    };
};
exports.unblindOutput = unblindOutput;
const getOutputValue = (wallet, output) => {
    if (isBitcoin(wallet.type)) {
        return output.value;
    }
    const unblinded = (0, exports.unblindOutput)(wallet, output, wallet.deriveBlindingKeyFromScript(output.script)
        .privateKey);
    return unblinded.isLbtc ? unblinded.value : 0;
};
exports.getOutputValue = getOutputValue;
const constructClaimDetails = (swapOutputType, wallet, swap, transaction, preimage, cooperative = false) => {
    // Compatibility mode with database schema version 0 in which this column didn't exist
    if (swap.lockupTransactionVout === undefined) {
        swap.lockupTransactionVout = (0, boltz_core_1.detectSwap)((0, Utils_1.getHexBuffer)(swap.redeemScript), transaction).vout;
    }
    const output = transaction.outs[swap.lockupTransactionVout];
    const claimDetails = {
        ...output,
        preimage,
        txHash: transaction.getHash(),
        vout: swap.lockupTransactionVout,
        keys: wallet.getKeysByIndex(swap.keyIndex),
    };
    switch (swap.version) {
        case Enums_1.SwapVersion.Taproot: {
            claimDetails.type = boltz_core_1.OutputType.Taproot;
            claimDetails.cooperative = cooperative;
            claimDetails.swapTree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(swap.redeemScript);
            claimDetails.internalKey = (0, exports.createMusig)(claimDetails.keys, (0, Utils_1.getHexBuffer)(swap.refundPublicKey)).getAggregatedPublicKey();
            break;
        }
        default: {
            claimDetails.type = swapOutputType.get(wallet.type);
            claimDetails.redeemScript = (0, Utils_1.getHexBuffer)(swap.redeemScript);
            break;
        }
    }
    return claimDetails;
};
exports.constructClaimDetails = constructClaimDetails;
const constructClaimTransaction = (wallet, claimDetails, destinationAddress, feePerVbyte) => {
    if (isBitcoin(wallet.type)) {
        return (0, boltz_core_1.targetFee)(feePerVbyte, (fee) => (0, boltz_core_1.constructClaimTransaction)(claimDetails, wallet.decodeAddress(destinationAddress), fee, true));
    }
    const liquidDetails = populateBlindingKeys(wallet, claimDetails);
    const decodedAddress = liquidjs_lib_1.address.fromConfidential(destinationAddress);
    return (0, boltz_core_1.targetFee)(feePerVbyte, (fee) => (0, liquid_1.constructClaimTransaction)(liquidDetails, decodedAddress.scriptPubKey, fee, true, wallet.network, decodedAddress.blindingKey));
};
exports.constructClaimTransaction = constructClaimTransaction;
const constructRefundTransaction = (wallet, refundDetails, destinationAddress, timeoutBlockHeight, feePerVbyte) => {
    if (isBitcoin(wallet.type)) {
        return (0, boltz_core_1.targetFee)(feePerVbyte, (fee) => (0, boltz_core_1.constructRefundTransaction)(refundDetails, wallet.decodeAddress(destinationAddress), timeoutBlockHeight, fee, true));
    }
    const liquidDetails = populateBlindingKeys(wallet, refundDetails);
    const decodedAddress = liquidjs_lib_1.address.fromConfidential(destinationAddress);
    return (0, boltz_core_1.targetFee)(feePerVbyte, (fee) => (0, liquid_1.constructRefundTransaction)(liquidDetails, decodedAddress.scriptPubKey, timeoutBlockHeight, fee, true, wallet.network, decodedAddress.blindingKey));
};
exports.constructRefundTransaction = constructRefundTransaction;
const calculateTransactionFee = async (chainClient, transaction) => {
    return chainClient.symbol !== LiquidTypes_1.liquidSymbol
        ? (0, Utils_1.calculateUtxoTransactionFee)(chainClient, transaction)
        : (0, Utils_1.calculateLiquidTransactionFee)(transaction);
};
exports.calculateTransactionFee = calculateTransactionFee;
const createMusig = (ourKeys, theirPublicKey) => new boltz_core_1.Musig(exports.zkp, ECPairHelper_1.ECPair.fromPrivateKey(ourKeys.privateKey), (0, crypto_1.randomBytes)(32), [
    ourKeys.publicKey,
    theirPublicKey,
]);
exports.createMusig = createMusig;
const tweakMusig = (type, musig, swapTree) => (isBitcoin(type) ? boltz_core_1.TaprootUtils : liquid_1.TaprootUtils).tweakMusig(musig, swapTree.tree);
exports.tweakMusig = tweakMusig;
const hashForWitnessV1 = async (currency, tx, index, leafHash, hashType = bitcoinjs_lib_1.Transaction.SIGHASH_DEFAULT) => {
    const inputs = await Promise.all(tx.ins.map(async (input) => {
        const inTx = (0, exports.parseTransaction)(currency.type, await currency.chainClient.getRawTransaction((0, Utils_1.getHexString)((0, Utils_1.reverseBuffer)(input.hash))));
        return inTx.outs[input.index];
    }));
    if (isBitcoin(currency.type)) {
        return tx.hashForWitnessV1(index, inputs.map((input) => input.script), inputs.map((input) => input.value), hashType, leafHash);
    }
    return tx.hashForWitnessV1(index, inputs.map((input) => input.script), inputs.map((input) => ({
        asset: input.asset,
        value: input.value,
    })), hashType, currency.network.genesisBlockHash, leafHash);
};
exports.hashForWitnessV1 = hashForWitnessV1;
const isBitcoin = (type) => type === Enums_1.CurrencyType.BitcoinLike;
const populateBlindingKeys = (wallet, utxos) => {
    for (const utxo of utxos) {
        utxo.blindingPrivateKey = wallet.deriveBlindingKeyFromScript(utxo.script).privateKey;
    }
    return utxos;
};
//# sourceMappingURL=Core.js.map