"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerExitHandler = void 0;
const PromiseUtils_1 = require("./PromiseUtils");
const exitHandlerTimeout = 5000;
const parseExitArg = (arg) => {
    if (!isNaN(+arg)) {
        return +arg;
    }
    switch (arg) {
        case 'SIGINT':
            return 0;
        default:
            return 1;
    }
};
const exitHandler = async (handler, arg) => {
    try {
        await (0, PromiseUtils_1.racePromise)(handler(), (reject) => reject('timeout'), exitHandlerTimeout);
    }
    catch (e) {
        console.error('Exit handler threw', e);
    }
    // eslint-disable-next-line no-process-exit
    process.exit(parseExitArg(arg));
};
const registerExitHandler = (handler) => {
    [
        'beforeExit',
        'SIGHUP',
        'SIGINT',
        'SIGQUIT',
        'SIGILL',
        'SIGTRAP',
        'SIGABRT',
        'SIGBUS',
        'SIGFPE',
        'SIGUSR1',
        'SIGSEGV',
        'SIGUSR2',
        'SIGTERM',
    ].forEach((evt) => process.on(evt, (arg) => exitHandler(handler, arg)));
};
exports.registerExitHandler = registerExitHandler;
//# sourceMappingURL=ExitHandler.js.map