"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const prom_client_1 = require("prom-client");
const Utils_1 = require("./Utils");
const StatsRepository_1 = __importStar(require("./db/repositories/StatsRepository"));
class Prometheus {
    constructor(logger, config, pairs) {
        this.logger = logger;
        this.config = config;
        this.pairs = new Set();
        this.start = async () => {
            if ([this.nodeRegistry, this.swapRegistry].some((registry) => registry === undefined)) {
                return;
            }
            const respondMetrics = (registry) => {
                return async (_, res) => {
                    res.setHeader('Content-Type', registry.contentType);
                    res.send(await registry.metrics());
                };
            };
            const app = (0, express_1.default)();
            app.route('/metrics').get(respondMetrics(this.nodeRegistry));
            app.route('/swapMetrics').get(respondMetrics(this.swapRegistry));
            new Promise((resolve) => {
                app.listen(this.config.port, this.config.host, () => {
                    this.logger.info(`Metrics server listening on: ${this.config.host}:${this.config.port}`);
                    resolve();
                });
            });
        };
        this.registerMetrics = () => {
            (0, prom_client_1.collectDefaultMetrics)({
                register: this.nodeRegistry,
            });
            const defaults = Array.from(this.pairs.values()).flatMap((pair) => Object.values(StatsRepository_1.SwapType).map((type) => ({
                type,
                pair,
            })));
            const setDefaults = (gauge, defaultValue) => {
                defaults.forEach((defaultLabels) => {
                    gauge.set(defaultLabels, defaultValue);
                });
            };
            this.swapRegistry.registerMetric(new prom_client_1.Gauge({
                name: `${Prometheus.metric_prefix}swap_counts`,
                labelNames: ['pair', 'type', 'status'],
                help: 'number of swaps',
                collect: async function () {
                    const counts = await StatsRepository_1.default.getSwapCounts();
                    counts.forEach((count) => this.set({ pair: count.pair, type: count.type, status: count.status }, count.count));
                },
            }));
            this.swapRegistry.registerMetric(new prom_client_1.Gauge({
                name: `${Prometheus.metric_prefix}swap_volume`,
                labelNames: ['pair', 'type'],
                help: 'volume of swaps',
                collect: async function () {
                    const volumes = await StatsRepository_1.default.getVolumePerPairType();
                    volumes.forEach((volume) => this.set({ pair: volume.pair, type: volume.type }, volume.volume));
                },
            }));
            this.swapRegistry.registerMetric(new prom_client_1.Gauge({
                name: `${Prometheus.metric_prefix}swap_pending_counts`,
                labelNames: ['pair', 'type'],
                help: 'count of pending swaps',
                collect: async function () {
                    const counts = await StatsRepository_1.default.getPendingSwapsCounts();
                    setDefaults(this, 0);
                    counts.forEach((count) => this.set({ pair: count.pair, type: count.type }, count.count));
                },
            }));
            this.swapRegistry.registerMetric(new prom_client_1.Gauge({
                name: `${Prometheus.metric_prefix}swap_locked_funds`,
                labelNames: ['pair', 'type'],
                help: 'coins locked in pending swaps',
                collect: async function () {
                    const lockedFunds = await StatsRepository_1.default.getLockedFunds();
                    setDefaults(this, 0);
                    lockedFunds.forEach((locked) => this.set({ pair: locked.pair, type: 'reverse' }, locked.locked));
                },
            }));
        };
        pairs.forEach((pair) => this.pairs.add((0, Utils_1.getPairId)(pair)));
        if (this.config === undefined ||
            Object.values(this.config).some((value) => value === undefined)) {
            this.logger.warn('Disabling metrics server because of missing configuration');
            return;
        }
        this.nodeRegistry = new prom_client_1.Registry();
        this.swapRegistry = new prom_client_1.Registry();
        this.registerMetrics();
    }
}
Prometheus.metric_prefix = 'boltz_';
exports.default = Prometheus;
//# sourceMappingURL=Prometheus.js.map