"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.allSettledFirst = exports.allSettled = exports.racePromise = void 0;
const racePromise = async (promise, raceHandler, raceTimeout) => {
    let timeout = undefined;
    const timeoutPromise = new Promise((_, reject) => {
        timeout = setTimeout(() => raceHandler(reject), raceTimeout);
    });
    try {
        const res = await Promise.race([
            promise instanceof Promise ? promise : promise(),
            timeoutPromise,
        ]);
        if (timeout !== undefined) {
            clearTimeout(timeout);
        }
        return res;
    }
    catch (e) {
        if (timeout !== undefined) {
            clearTimeout(timeout);
        }
        throw e;
    }
};
exports.racePromise = racePromise;
const allSettled = async (promises) => {
    const settled = await Promise.allSettled(promises);
    const results = settled
        .filter((res) => res.status === 'fulfilled')
        .map((res) => res.value);
    if (results.length === 0) {
        throw settled[0].reason;
    }
    return results;
};
exports.allSettled = allSettled;
const allSettledFirst = async (promises) => (await (0, exports.allSettled)(promises))[0];
exports.allSettledFirst = allSettledFirst;
//# sourceMappingURL=PromiseUtils.js.map