"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ChainClient_1 = __importDefault(require("./chain/ChainClient"));
const LndClient_1 = __importDefault(require("./lightning/LndClient"));
const ClnClient_1 = __importDefault(require("./lightning/cln/ClnClient"));
const MpayClient_1 = __importDefault(require("./lightning/cln/MpayClient"));
class VersionSplit {
    constructor(version) {
        this.eq = (cmp) => {
            return (this.parts.length === cmp.parts.length &&
                this.parts.every((elem, i) => elem === cmp.parts[i]));
        };
        this.gt = (cmp) => {
            for (let i = 0; i < Math.max(this.parts.length, cmp.parts.length); i++) {
                const v1 = VersionSplit.getVersionPart(this.parts, i);
                const v2 = VersionSplit.getVersionPart(cmp.parts, i);
                if (v1 > v2) {
                    return true;
                }
                else if (v2 > v1) {
                    return false;
                }
            }
            return false;
        };
        const split = version.split('.');
        this.parts = split.map(Number);
    }
}
VersionSplit.getVersionPart = (arr, index) => {
    return arr.length > index ? arr[index] : 0;
};
class Comperator {
}
Comperator.versionInBounds = (version, limits) => {
    if (typeof version === 'number') {
        return (version <= limits.maximal &&
            version >= limits.minimal);
    }
    const [versionSplit, minSplit, maxSplit] = [
        version,
        limits.minimal,
        limits.maximal,
    ].map((ver) => new VersionSplit(ver));
    return (!minSplit.gt(versionSplit) &&
        (maxSplit.gt(versionSplit) || maxSplit.eq(versionSplit)));
};
class VersionCheck {
}
VersionCheck.versionLimits = {
    [ChainClient_1.default.serviceName]: {
        minimal: 220000,
        maximal: 270000,
    },
    [ClnClient_1.default.serviceName]: {
        minimal: '23.05',
        maximal: '24.02.2',
    },
    [ClnClient_1.default.serviceNameHold]: {
        minimal: '0.0.3',
        maximal: '0.0.5',
    },
    [MpayClient_1.default.serviceName]: {
        minimal: '0.1.0',
        maximal: '0.1.1',
    },
    [LndClient_1.default.serviceName]: {
        minimal: '0.16.0',
        maximal: '0.17.5',
    },
};
VersionCheck.checkChainClientVersion = (symbol, version) => {
    if (!Comperator.versionInBounds(version, VersionCheck.versionLimits[ChainClient_1.default.serviceName])) {
        throw VersionCheck.unsupportedVersionError(`${symbol} Core`, version, VersionCheck.versionLimits[ChainClient_1.default.serviceName]);
    }
};
VersionCheck.checkLightningVersion = (serviceName, symbol, version) => {
    let limits;
    let sanitizedVersion = version;
    switch (serviceName) {
        case LndClient_1.default.serviceName:
            sanitizedVersion = version.split('-')[0];
            limits = VersionCheck.versionLimits[LndClient_1.default.serviceName];
            break;
        case ClnClient_1.default.serviceName:
            if (version.startsWith('v')) {
                sanitizedVersion = sanitizedVersion.slice(1);
            }
            if (version.endsWith(ClnClient_1.default.moddedVersionSuffix)) {
                sanitizedVersion = sanitizedVersion.slice(0, -ClnClient_1.default.moddedVersionSuffix.length);
            }
            limits = VersionCheck.versionLimits[ClnClient_1.default.serviceName];
            break;
        case ClnClient_1.default.serviceNameHold:
            limits = VersionCheck.versionLimits[ClnClient_1.default.serviceNameHold];
            break;
        case MpayClient_1.default.serviceName:
            limits = VersionCheck.versionLimits[MpayClient_1.default.serviceName];
            break;
        default:
            throw `unsupported lightning client ${serviceName}`;
    }
    if (!Comperator.versionInBounds(sanitizedVersion, limits)) {
        throw VersionCheck.unsupportedVersionError(`${symbol} ${serviceName}`, version, limits);
    }
};
VersionCheck.unsupportedVersionError = (service, actual, limits) => {
    return `unsupported ${service} version: ${actual}; min version ${limits.minimal}; max version ${limits.maximal}`;
};
exports.default = VersionCheck;
//# sourceMappingURL=VersionCheck.js.map