"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const Utils_1 = require("../Utils");
const ReferralRepository_1 = __importDefault(require("../db/repositories/ReferralRepository"));
class Bouncer {
}
_a = Bouncer;
Bouncer.timestampDeltaTolerance = 60;
Bouncer.errorUnauthorized = 'unauthorized';
Bouncer.validateRequestAuthentication = async (req) => {
    const ts = _a.checkTimestamp(_a.getRequestHeader(req, 'TS'));
    const referral = await _a.fetchApiCredentials(_a.getRequestHeader(req, 'API-KEY'));
    _a.verifyHmac(_a.getRequestHeader(req, 'API-HMAC'), referral, ts, req.method, req.originalUrl, 
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    req.rawBody);
    return referral;
};
Bouncer.verifyHmac = (providedHmac, referral, ts, method, path, body) => {
    let hmac = (0, crypto_1.createHmac)('sha256', referral.apiSecret).update(`${ts}${method}${path}`);
    if (method === 'POST') {
        hmac = hmac.update(body);
    }
    const hexHmac = hmac.digest('hex');
    if (providedHmac !== hexHmac) {
        throw _a.errorUnauthorized;
    }
};
Bouncer.checkTimestamp = (providedTsRaw) => {
    const providedTs = parseInt(providedTsRaw);
    if (isNaN(providedTs)) {
        throw 'TS header not a number';
    }
    const now = (0, Utils_1.getUnixTime)();
    if (providedTs > now + _a.timestampDeltaTolerance ||
        providedTs < now - _a.timestampDeltaTolerance) {
        throw `TS header deviates from server time by more than ${_a.timestampDeltaTolerance} seconds`;
    }
    return providedTs;
};
Bouncer.fetchApiCredentials = async (apiKey) => {
    const referral = await ReferralRepository_1.default.getReferralByApiKey(apiKey);
    if (referral === null) {
        throw _a.errorUnauthorized;
    }
    return referral;
};
Bouncer.getRequestHeader = (req, name) => {
    const header = req.get(name);
    if (header === undefined) {
        throw `${name} header not set`;
    }
    return header;
};
exports.default = Bouncer;
//# sourceMappingURL=Bouncer.js.map