"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const Utils_1 = require("../../../Utils");
const Utils_2 = require("../../Utils");
const RouterBase_1 = __importDefault(require("./RouterBase"));
class InfoRouter extends RouterBase_1.default {
    constructor(logger, service) {
        super(logger, '');
        this.service = service;
        this.getRouter = () => {
            /**
             * @openapi
             * tags:
             *   name: Info
             *   description: General information
             */
            const router = (0, express_1.Router)();
            /**
             * @openapi
             * /version:
             *   get:
             *     description: Version of the backend
             *     tags: [Info]
             *     responses:
             *       '200':
             *         content:
             *           application/json:
             *             schema:
             *               type: object
             *               properties:
             *                 version:
             *                   type: string
             *                   required: true
             */
            router.get('/version', this.handleError(this.getVersion));
            /**
             * @openapi
             * /infos:
             *   get:
             *     description: Information about the configuration of the backend
             *     tags: [Info]
             *     responses:
             *       '200':
             *         content:
             *           application/json:
             *             schema:
             *               type: array
             *               items:
             *                 type: string
             */
            router.get('/infos', this.handleError(this.getInfos));
            /**
             * @openapi
             * /warnings:
             *   get:
             *     description: Warnings about the configuration of the backend
             *     tags: [Info]
             *     responses:
             *       '200':
             *         content:
             *           application/json:
             *             schema:
             *               type: array
             *               items:
             *                 type: string
             */
            router.get('/warnings', this.handleError(this.getWarnings));
            return router;
        };
        this.getVersion = (_req, res) => (0, Utils_2.successResponse)(res, {
            version: (0, Utils_1.getVersion)(),
        });
        this.getInfos = (_req, res) => (0, Utils_2.successResponse)(res, this.service.getInfos());
        this.getWarnings = (_req, res) => (0, Utils_2.successResponse)(res, this.service.getWarnings());
    }
}
exports.default = InfoRouter;
//# sourceMappingURL=InfoRouter.js.map