"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AddressType = void 0;
const BaseClient_1 = __importDefault(require("../BaseClient"));
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const ChainTipRepository_1 = __importDefault(require("../db/repositories/ChainTipRepository"));
const MempoolSpace_1 = __importDefault(require("./MempoolSpace"));
const RpcClient_1 = __importDefault(require("./RpcClient"));
const ZmqClient_1 = __importStar(require("./ZmqClient"));
var AddressType;
(function (AddressType) {
    AddressType["Legacy"] = "legacy";
    AddressType["P2shegwit"] = "p2sh-segwit";
    AddressType["Bech32"] = "bech32";
    AddressType["Taproot"] = "bech32m";
})(AddressType || (exports.AddressType = AddressType = {}));
class ChainClient extends BaseClient_1.default {
    constructor(logger, config, symbol) {
        super(logger, symbol);
        this.config = config;
        this.currencyType = Enums_1.CurrencyType.BitcoinLike;
        this.feeFloor = 2;
        this.serviceName = () => {
            return ChainClient.serviceName;
        };
        this.connect = async () => {
            let zmqNotifications = [];
            if (this.config.zmqpubrawtx !== undefined &&
                (this.config.zmqpubrawblock !== undefined ||
                    this.config.zmqpubhashblock !== undefined)) {
                this.logger.debug(`Using configured ZMQ endpoints for ${this.symbol} client`);
                if (this.config.zmqpubrawtx) {
                    zmqNotifications.push({
                        type: ZmqClient_1.filters.rawTx,
                        address: this.config.zmqpubrawtx,
                    });
                }
                if (this.config.zmqpubrawblock) {
                    zmqNotifications.push({
                        type: ZmqClient_1.filters.rawBlock,
                        address: this.config.zmqpubrawblock,
                    });
                }
                if (this.config.zmqpubhashblock) {
                    zmqNotifications.push({
                        type: ZmqClient_1.filters.hashBlock,
                        address: this.config.zmqpubhashblock,
                    });
                }
            }
            else {
                this.logger.debug(`ZMQ endpoints for ${this.symbol} client not configured; fetching from RPC`);
                // Dogecoin Core and Zcash don't support the 'getzmqnotifications' method *yet*
                // Therefore, the hosts and ports for these chains have to be configured manually
                try {
                    zmqNotifications = await this.getZmqNotifications();
                }
                catch (error) {
                    if (error.message === 'Method not found') {
                        this.logger.warn(`${this.symbol} client does not support fetching ZMQ endpoints. Please set them in the config`);
                    }
                    throw error;
                }
            }
            await this.zmqClient.init(this.currencyType, zmqNotifications);
            await this.listenToZmq();
            if (this.mempoolSpace) {
                await this.mempoolSpace.init();
            }
        };
        this.disconnect = () => {
            this.clearReconnectTimer();
            this.zmqClient.close();
            this.setClientStatus(Enums_1.ClientStatus.Disconnected);
        };
        this.getNetworkInfo = () => {
            return this.client.request('getnetworkinfo');
        };
        this.getBlockchainInfo = async () => {
            const blockchainInfo = await this.client.request('getblockchaininfo');
            return {
                ...blockchainInfo,
                scannedBlocks: this.zmqClient.blockHeight,
            };
        };
        this.getBlock = (hash) => {
            return this.client.request('getblock', [hash]);
        };
        this.getBlockVerbose = (hash) => {
            return this.client.request('getblock', [hash, 2]);
        };
        this.getBlockhash = (height) => {
            return this.client.request('getblockhash', [height]);
        };
        /**
         * Add an output to the list of relevant ones
         */
        this.addOutputFilter = (outputScript) => {
            this.zmqClient.relevantOutputs.add((0, Utils_1.getHexString)(outputScript));
        };
        /**
         * Removes an output from the list of relevant ones
         */
        this.removeOutputFilter = (outputScript) => {
            this.zmqClient.relevantOutputs.delete((0, Utils_1.getHexString)(outputScript));
        };
        /**
         * Adds an input hash to the list of relevant ones
         */
        this.addInputFilter = (inputHash) => {
            this.zmqClient.relevantInputs.add((0, Utils_1.getHexString)(inputHash));
        };
        /**
         * Removes an input hash from the list of relevant ones
         */
        this.removeInputFilter = (inputHash) => {
            this.zmqClient.relevantInputs.delete((0, Utils_1.getHexString)(inputHash));
        };
        this.rescanChain = async (startHeight) => {
            await this.zmqClient.rescanChain(startHeight);
        };
        this.sendRawTransaction = (rawTransaction) => {
            return this.client.request('sendrawtransaction', [rawTransaction]);
        };
        this.getRawTransaction = async (id) => {
            try {
                return await this.client.request('getrawtransaction', [id]);
            }
            catch (error) {
                throw this.formatGetTransactionError(id, error);
            }
        };
        this.getRawTransactionVerbose = async (id) => {
            try {
                return await this.client.request('getrawtransaction', [
                    id,
                    1,
                ]);
            }
            catch (error) {
                throw this.formatGetTransactionError(id, error);
            }
        };
        this.getRawMempool = async () => {
            return this.client.request('getrawmempool');
        };
        this.estimateFee = async (confTarget = 2) => {
            return this.estimateFeeWithFloor(confTarget);
        };
        this.getWalletInfo = async () => {
            return this.client.request('getwalletinfo');
        };
        this.sendToAddress = (address, amount, satPerVbyte, subtractFeeFromAmount = false) => {
            return this.client.request('sendtoaddress', [
                address,
                amount / ChainClient.decimals,
                undefined,
                undefined,
                subtractFeeFromAmount,
                false,
                undefined,
                undefined,
                undefined,
                satPerVbyte,
            ]);
        };
        this.listUnspent = (minimalConfirmations = 0) => {
            return this.client.request('listunspent', [
                minimalConfirmations,
            ]);
        };
        this.generate = async (blocks) => {
            return this.client.request('generatetoaddress', [
                blocks,
                await this.getNewAddress(),
            ]);
        };
        this.getNewAddress = (type = AddressType.Bech32) => {
            return this.client.request('getnewaddress', [undefined, type]);
        };
        this.estimateFeeWithFloor = async (confTarget) => {
            const mempoolFee = this.mempoolSpace?.latestFee();
            const estimation = mempoolFee !== undefined
                ? mempoolFee
                : await this.estimateFeeChainClient(confTarget);
            return Math.max(estimation, this.feeFloor);
        };
        this.estimateFeeChainClient = async (confTarget = 1) => {
            try {
                const response = await this.client.request('estimatesmartfee', [
                    confTarget,
                ]);
                if (response.feerate) {
                    const feePerKb = response.feerate * ChainClient.decimals;
                    return Math.max(Math.round(feePerKb / 1000), 2);
                }
                return this.feeFloor;
            }
            catch (error) {
                if (error.message === 'Method not found') {
                    // TODO: use estimatefee for outdated node versions
                    this.logger.warn(`'estimatesmartfee' method not found on ${this.symbol} chain`);
                    return 2;
                }
                throw error;
            }
        };
        this.getZmqNotifications = () => {
            return this.client.request('getzmqnotifications');
        };
        this.listenToZmq = async () => {
            const { scannedBlocks } = await this.getBlockchainInfo();
            const chainTip = await ChainTipRepository_1.default.findOrCreateTip(this.symbol, scannedBlocks);
            this.zmqClient.on('block', async (height) => {
                this.logger.silly(`Got new ${this.symbol} block: ${height}`);
                await ChainTipRepository_1.default.updateTip(chainTip, height);
                this.emit('block', height);
            });
            this.zmqClient.on('transaction', ({ transaction, confirmed }) => {
                this.emit('transaction', { transaction, confirmed });
            });
        };
        this.formatGetTransactionError = (id, error) => {
            const formattedError = (0, Utils_1.formatError)(error);
            if (formattedError.includes('No such mempool or blockchain transaction')) {
                return `${formattedError} ID: ${id}`;
            }
            return error;
        };
        this.client = new RpcClient_1.default(this.config);
        this.zmqClient = new ZmqClient_1.default(symbol, logger, this);
        if (this.config.mempoolSpace && this.config.mempoolSpace !== '') {
            this.mempoolSpace = new MempoolSpace_1.default(this.logger, this.symbol, this.config.mempoolSpace);
        }
    }
}
ChainClient.serviceName = 'Core';
ChainClient.decimals = 100000000;
exports.default = ChainClient;
//# sourceMappingURL=ChainClient.js.map