import { Transaction } from 'liquidjs-lib';
import { ChainConfig } from '../Config';
import Logger from '../Logger';
import { AddressInfo, LiquidBalances } from '../consts/LiquidTypes';
import ChainClient, { AddressType, IChainClient } from './ChainClient';
declare enum LiquidAddressType {
    Blech32 = "blech32"
}
interface IElementsClient extends IChainClient<Transaction> {
    getAddressInfo(address: string): Promise<AddressInfo>;
    getBalances(): Promise<LiquidBalances>;
    getNewAddress(type?: AddressType | LiquidAddressType): Promise<string>;
    dumpBlindingKey(address: string): Promise<string>;
}
declare class ElementsClient extends ChainClient<Transaction> implements IElementsClient {
    readonly isLowball: boolean;
    static readonly symbol = "L-BTC";
    constructor(logger: Logger, config: ChainConfig, isLowball?: boolean);
    serviceName: () => string;
    getBalances: () => Promise<LiquidBalances>;
    getNewAddress: (type?: AddressType | LiquidAddressType) => Promise<string>;
    sendToAddress: (address: string, amount: number, satPerVbyte?: number, subtractFeeFromAmount?: boolean) => Promise<string>;
    estimateFee: () => Promise<number>;
    getAddressInfo: (address: string) => Promise<AddressInfo>;
    dumpBlindingKey: (address: string) => Promise<string>;
}
export default ElementsClient;
export { LiquidAddressType, IElementsClient };
