"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiquidAddressType = void 0;
const Enums_1 = require("../consts/Enums");
const LiquidTypes_1 = require("../consts/LiquidTypes");
const ChainClient_1 = __importDefault(require("./ChainClient"));
var LiquidAddressType;
(function (LiquidAddressType) {
    LiquidAddressType["Blech32"] = "blech32";
})(LiquidAddressType || (exports.LiquidAddressType = LiquidAddressType = {}));
class ElementsClient extends ChainClient_1.default {
    constructor(logger, config, isLowball = false) {
        super(logger, config, ElementsClient.symbol);
        this.isLowball = isLowball;
        this.serviceName = () => {
            return 'Elements';
        };
        this.getBalances = async () => {
            const res = await this.client.request('getbalances');
            for (const balanceType of Object.values(res.mine)) {
                for (const [key, value] of Object.entries(balanceType)) {
                    balanceType[key] = value * ChainClient_1.default.decimals;
                }
            }
            return res;
        };
        this.getNewAddress = (type = LiquidAddressType.Blech32) => {
            return this.client.request('getnewaddress', [undefined, type]);
        };
        this.sendToAddress = (address, amount, satPerVbyte, subtractFeeFromAmount = false) => {
            return this.client.request('sendtoaddress', [
                address,
                amount / ChainClient_1.default.decimals,
                undefined,
                undefined,
                subtractFeeFromAmount,
                false,
                undefined,
                undefined,
                undefined,
                undefined,
                false,
                satPerVbyte,
            ]);
        };
        this.estimateFee = async () => {
            return this.feeFloor;
        };
        this.getAddressInfo = (address) => {
            return this.client.request('getaddressinfo', [address]);
        };
        this.dumpBlindingKey = (address) => {
            return this.client.request('dumpblindingkey', [address]);
        };
        this.currencyType = Enums_1.CurrencyType.Liquid;
        this.feeFloor = isLowball ? 0.01 : 0.11;
    }
}
ElementsClient.symbol = LiquidTypes_1.liquidSymbol;
exports.default = ElementsClient;
//# sourceMappingURL=ElementsClient.js.map