/// <reference types="node" />
import { Transaction } from 'liquidjs-lib';
import BaseClient from '../BaseClient';
import { LiquidChainConfig } from '../Config';
import Logger from '../Logger';
import { CurrencyType } from '../consts/Enums';
import { UnspentUtxo } from '../consts/Types';
import { AddressType, ChainClientEvents } from './ChainClient';
import { IElementsClient, LiquidAddressType } from './ElementsClient';
declare class ElementsWrapper extends BaseClient<ChainClientEvents<Transaction>> implements IElementsClient {
    readonly currencyType = CurrencyType.Liquid;
    private readonly clients;
    constructor(logger: Logger, config: LiquidChainConfig);
    serviceName: () => string;
    connect: () => Promise<void>;
    disconnect: () => void;
    rescanChain: (startHeight: number) => Promise<void>;
    addInputFilter: (inputHash: Buffer) => void;
    addOutputFilter: (outputScript: Buffer) => void;
    removeOutputFilter: (outputScript: Buffer) => void;
    removeInputFilter: (inputHash: Buffer) => void;
    getBlockchainInfo: () => Promise<{
        scannedBlocks: number;
        chain: string;
        blocks: number;
        headers: number;
        bestblockhash: string;
        difficulty: number;
        mediantime: number;
        verificationprogress: number;
        initialblockdownload: string;
        chainwork: string;
        size_on_disk: number;
        pruned: boolean;
    } & {
        lowball?: {
            scannedBlocks: number;
            chain: string;
            blocks: number;
            headers: number;
            bestblockhash: string;
            difficulty: number;
            mediantime: number;
            verificationprogress: number;
            initialblockdownload: string;
            chainwork: string;
            size_on_disk: number;
            pruned: boolean;
        } | undefined;
    }>;
    getNetworkInfo: () => Promise<import("../consts/Types").NetworkInfo & {
        lowball?: import("../consts/Types").NetworkInfo | undefined;
    }>;
    sendRawTransaction: (transactionHex: string, isSwapRelated?: boolean) => Promise<string>;
    getRawTransaction: (transactionId: string) => Promise<string>;
    getRawTransactionVerbose: (transactionId: string) => Promise<import("../consts/Types").RawTransaction>;
    estimateFee: () => Promise<number>;
    listUnspent: (minimalConfirmations?: number) => Promise<UnspentUtxo[]>;
    getAddressInfo: (address: string) => Promise<import("../consts/LiquidTypes").AddressInfo>;
    getBalances: () => Promise<import("../consts/LiquidTypes").LiquidBalances>;
    getNewAddress: (type?: AddressType | LiquidAddressType) => Promise<string>;
    dumpBlindingKey: (address: string) => Promise<string>;
    sendToAddress: (address: string, amount: number, satPerVbyte?: number, subtractFeeFromAmount?: boolean) => Promise<string>;
    private walletClient;
    private publicClient;
    private lowballClient;
    private annotateLowballInfo;
}
export default ElementsWrapper;
