"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const http_1 = __importDefault(require("http"));
const Errors_1 = __importDefault(require("./Errors"));
class RpcClient {
    constructor(config) {
        this.options = {};
        this.request = (method, params) => {
            return new Promise((resolve, reject) => {
                const serializedRequest = JSON.stringify({
                    method,
                    params,
                });
                const request = http_1.default.request(this.options, (response) => {
                    let buffer = '';
                    response.on('data', (chunk) => {
                        buffer += chunk.toString();
                    });
                    response.on('end', () => {
                        if (response.statusCode === 401) {
                            reject('401 unauthorized');
                            return;
                        }
                        if (response.statusCode === 403) {
                            reject('403 forbidden');
                            return;
                        }
                        const parsedResponse = JSON.parse(buffer);
                        if (parsedResponse.error) {
                            reject(parsedResponse.error);
                        }
                        resolve(parsedResponse.result);
                    });
                });
                request.on('error', (error) => {
                    reject(error);
                });
                request.setHeader('Content-Length', serializedRequest.length);
                request.setHeader('Authorization', `Basic ${this.auth}`);
                request.write(serializedRequest);
                request.end();
            });
        };
        this.options = {
            host: config.host,
            port: config.port,
            path: '/',
            method: 'POST',
        };
        // If a cookie is configured, it will be preferred
        if (config.cookie && config.cookie !== '') {
            if (!(0, fs_1.existsSync)(config.cookie)) {
                throw Errors_1.default.INVALID_COOKIE_FILE(config.cookie);
            }
            const cookieFile = (0, fs_1.readFileSync)(config.cookie, 'utf-8').trim();
            this.auth = Buffer.from(cookieFile).toString('base64');
        }
        else if (config.user && config.password) {
            this.auth = Buffer.from(`${config.user}:${config.password}`).toString('base64');
        }
        else {
            throw Errors_1.default.NO_AUTHENTICATION();
        }
    }
}
exports.default = RpcClient;
//# sourceMappingURL=RpcClient.js.map