import { Transaction } from 'bitcoinjs-lib';
import { Transaction as LiquidTransaction } from 'liquidjs-lib';
import Logger from '../Logger';
import { CurrencyType } from '../consts/Enums';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import ChainClient from './ChainClient';
type ZmqNotification = {
    type: string;
    address: string;
};
declare const filters: {
    rawTx: string;
    rawBlock: string;
    hashBlock: string;
};
export type SomeTransaction = Transaction | LiquidTransaction;
declare class ZmqClient<T extends SomeTransaction> extends TypedEventEmitter<{
    block: number;
    transaction: {
        transaction: T;
        confirmed: boolean;
    };
}> {
    private symbol;
    private logger;
    private chainClient;
    utxos: Set<string>;
    relevantInputs: Set<string>;
    relevantOutputs: Set<string>;
    blockHeight: number;
    private currencyType;
    private bestBlockHash;
    private rawBlockAddress?;
    private hashBlockAddress?;
    private sockets;
    private compatibilityRescan;
    private blockHandleLock;
    private static readonly connectTimeout;
    constructor(symbol: string, logger: Logger, chainClient: ChainClient);
    init: (currencyType: CurrencyType, notifications: ZmqNotification[]) => Promise<void>;
    close: () => void;
    rescanChain: (startHeight: number) => Promise<void>;
    private initRawTransaction;
    private initRawBlock;
    private initHashBlock;
    private isRelevantTransaction;
    private newChainTip;
    private logReorganize;
    private logOrphanBlock;
    private createSocket;
}
export default ZmqClient;
export { ZmqNotification, filters };
