"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.musigFromExtractedKey = exports.printError = exports.printResponse = exports.currencyTypeFromNetwork = exports.parseNetwork = exports.parseBlindingKey = exports.getWalletStub = exports.prepareTx = exports.callback = exports.loadBoltzClient = void 0;
const grpc_js_1 = require("@grpc/grpc-js");
const boltz_core_1 = require("boltz-core");
const liquid_1 = require("boltz-core/dist/lib/liquid");
const console = __importStar(require("console"));
const crypto_1 = require("crypto");
const Core_1 = require("../Core");
const ECPairHelper_1 = require("../ECPairHelper");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const boltzrpc_grpc_pb_1 = require("../proto/boltzrpc_grpc_pb");
const loadBoltzClient = (argv) => {
    return new boltzrpc_grpc_pb_1.BoltzClient(`${argv.rpc.host}:${argv.rpc.port}`, grpc_js_1.credentials.createInsecure());
};
exports.loadBoltzClient = loadBoltzClient;
const callback = (formatter) => {
    return (error, response) => {
        if (error) {
            (0, exports.printError)(error);
            // eslint-disable-next-line no-process-exit
            process.exit(1);
        }
        else {
            const responseObj = response.toObject();
            if (Object.keys(responseObj).length === 0) {
                console.log('success');
            }
            else {
                (0, exports.printResponse)(formatter !== undefined ? formatter(response) : responseObj);
            }
        }
    };
};
exports.callback = callback;
const prepareTx = async (argv, keyExtractionFunc) => {
    await (0, Core_1.setup)();
    const type = (0, exports.currencyTypeFromNetwork)(argv.network);
    const network = (0, exports.parseNetwork)(argv.network);
    const transaction = (0, Core_1.parseTransaction)(type, argv.rawTransaction);
    const blindingKey = (0, exports.parseBlindingKey)(type, argv.blindingKey);
    const res = {
        type,
        blindingKey,
        transaction,
        walletStub: (0, exports.getWalletStub)(type, network, argv.destinationAddress, argv.blindingKey),
        destinationAddress: argv.destinationAddress,
        keys: ECPairHelper_1.ECPair.fromPrivateKey((0, Utils_1.getHexBuffer)(argv.privateKey)),
    };
    // If the redeem script can be parsed as JSON, it is a swap tree
    try {
        const tree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(argv.redeemScript.replaceAll('\\"', '"'));
        const { musig, swapOutput } = (0, exports.musigFromExtractedKey)(type, res.keys, keyExtractionFunc(tree), tree, transaction);
        res.swapOutput = {
            ...swapOutput,
            swapTree: tree,
            internalKey: musig.getAggregatedPublicKey(),
        };
    }
    catch (e) {
        res.redeemScript = (0, Utils_1.getHexBuffer)(argv.redeemScript);
        res.swapOutput = (0, boltz_core_1.detectSwap)(res.redeemScript, transaction);
    }
    return res;
};
exports.prepareTx = prepareTx;
const getWalletStub = (type, network, destinationAddress, blindingKey) => ({
    type,
    network,
    deriveBlindingKeyFromScript: () => ({
        privateKey: (0, exports.parseBlindingKey)(type, blindingKey),
    }),
    decodeAddress: () => (0, Core_1.toOutputScript)(type, destinationAddress, network),
});
exports.getWalletStub = getWalletStub;
const parseBlindingKey = (type, blindingKey) => type === Enums_1.CurrencyType.Liquid ? (0, Utils_1.getHexBuffer)(blindingKey) : undefined;
exports.parseBlindingKey = parseBlindingKey;
const parseNetwork = (network) => (0, exports.currencyTypeFromNetwork)(network) === Enums_1.CurrencyType.BitcoinLike
    ? boltz_core_1.Networks[network]
    : liquid_1.Networks[network];
exports.parseNetwork = parseNetwork;
const currencyTypeFromNetwork = (network) => network.includes('liquid') ? Enums_1.CurrencyType.Liquid : Enums_1.CurrencyType.BitcoinLike;
exports.currencyTypeFromNetwork = currencyTypeFromNetwork;
const printResponse = (response) => {
    console.log((0, Utils_1.stringify)(response));
};
exports.printResponse = printResponse;
const printError = (error) => {
    console.error(`${error.name}: ${error.message}`);
};
exports.printError = printError;
const musigFromExtractedKey = (type, ourKeys, theirPublicKey, tree, lockupTx) => {
    for (const tieBreaker of ['02', '03']) {
        const compressedKey = Buffer.concat([
            (0, Utils_1.getHexBuffer)(tieBreaker),
            theirPublicKey,
        ]);
        const musig = new boltz_core_1.Musig(Core_1.zkp, ourKeys, (0, crypto_1.randomBytes)(32), [
            compressedKey,
            ourKeys.publicKey,
        ]);
        const tweakedKey = (0, Core_1.tweakMusig)(type, musig, tree);
        const swapOutput = (0, boltz_core_1.detectSwap)(tweakedKey, lockupTx);
        if (swapOutput !== undefined) {
            return {
                musig,
                tweakedKey,
                swapOutput,
                theirPublicKey: compressedKey,
            };
        }
    }
    throw 'could not find swap output';
};
exports.musigFromExtractedKey = musigFromExtractedKey;
//# sourceMappingURL=Command.js.map