"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.finalizeCooperativeTransaction = exports.prepareCooperativeTransaction = exports.setupCooperativeTransaction = void 0;
const boltz_core_1 = require("boltz-core");
const liquid_1 = require("boltz-core/dist/lib/liquid");
const Core_1 = require("../Core");
const ECPairHelper_1 = require("../ECPairHelper");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const Command_1 = require("./Command");
const setupCooperativeTransaction = async (argv, keyExtractionFunc, lockupTx) => {
    await (0, Core_1.setup)();
    const network = (0, Command_1.parseNetwork)(argv.network);
    const currencyType = (0, Command_1.currencyTypeFromNetwork)(argv.network);
    const swapTree = boltz_core_1.SwapTreeSerializer.deserializeSwapTree(argv.swapTree);
    const keys = ECPairHelper_1.ECPair.fromPrivateKey((0, Utils_1.getHexBuffer)(argv.privateKey));
    const { musig, tweakedKey, theirPublicKey } = (0, Command_1.musigFromExtractedKey)(currencyType, keys, keyExtractionFunc(swapTree), swapTree, lockupTx);
    return {
        keys,
        musig,
        network,
        tweakedKey,
        currencyType,
        theirPublicKey,
    };
};
exports.setupCooperativeTransaction = setupCooperativeTransaction;
const prepareCooperativeTransaction = (argv, network, currencyType, keys, tweakedKey, lockupTx) => {
    const swapOutput = (0, boltz_core_1.detectSwap)(tweakedKey, lockupTx);
    if (swapOutput === undefined) {
        throw 'could not find swap output';
    }
    const details = {
        ...swapOutput,
        keys,
        txHash: lockupTx.getHash(),
        type: boltz_core_1.OutputType.Taproot,
        cooperative: true,
    };
    const tx = (0, Core_1.constructClaimTransaction)((0, Command_1.getWalletStub)(currencyType, network, argv.destinationAddress, argv.blindingKey), [details], argv.destinationAddress, argv.feePerVbyte);
    return {
        details,
        tx: tx,
    };
};
exports.prepareCooperativeTransaction = prepareCooperativeTransaction;
const finalizeCooperativeTransaction = (tx, musig, network, currencyType, otherPublicKey, details, partialSig) => {
    musig.aggregateNonces([[otherPublicKey, (0, Utils_1.getHexBuffer)(partialSig.pubNonce)]]);
    let hash;
    if (currencyType === Enums_1.CurrencyType.BitcoinLike) {
        hash = boltz_core_1.TaprootUtils.hashForWitnessV1([details], tx, 0);
    }
    else {
        hash = liquid_1.TaprootUtils.hashForWitnessV1(network, [details], tx, 0);
    }
    musig.initializeSession(hash);
    musig.signPartial();
    musig.addPartial(otherPublicKey, (0, Utils_1.getHexBuffer)(partialSig.partialSignature));
    tx.setWitness(0, [musig.aggregatePartials()]);
    return tx;
};
exports.finalizeCooperativeTransaction = finalizeCooperativeTransaction;
//# sourceMappingURL=TaprootHelper.js.map