"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const boltz_core_1 = require("boltz-core");
const Core_1 = require("../../Core");
const Utils_1 = require("../../Utils");
const BoltzApiClient_1 = __importDefault(require("../BoltzApiClient"));
const BuilderComponents_1 = __importDefault(require("../BuilderComponents"));
const Command_1 = require("../Command");
const TaprootHelper_1 = require("../TaprootHelper");
exports.command = 'claim-cooperative <network> <preimage> <privateKey> <swapId> <swapTree> <destinationAddress> [feePerVbyte] [blindingKey]';
exports.describe = 'claims a Taproot Reverse Submarine Swap cooperatively';
exports.builder = {
    network: BuilderComponents_1.default.network,
    preimage: BuilderComponents_1.default.preimage,
    privateKey: BuilderComponents_1.default.privateKey,
    swapId: BuilderComponents_1.default.swapId,
    swapTree: BuilderComponents_1.default.swapTree,
    destinationAddress: BuilderComponents_1.default.destinationAddress,
    feePerVbyte: BuilderComponents_1.default.feePerVbyte,
    blindingKey: BuilderComponents_1.default.blindingKey,
};
const handler = async (argv) => {
    const boltzClient = new BoltzApiClient_1.default();
    const swapStatus = await boltzClient.getStatus(argv.swapId);
    if (swapStatus.transaction === undefined) {
        throw 'no transaction in swap status';
    }
    const network = (0, Command_1.parseNetwork)(argv.network);
    const currencyType = (0, Command_1.currencyTypeFromNetwork)(argv.network);
    const lockupTx = (0, Core_1.parseTransaction)(currencyType, swapStatus.transaction.hex);
    const { keys, tweakedKey, theirPublicKey, musig } = await (0, TaprootHelper_1.setupCooperativeTransaction)(argv, boltz_core_1.extractRefundPublicKeyFromReverseSwapTree, lockupTx);
    const { details, tx } = (0, TaprootHelper_1.prepareCooperativeTransaction)(argv, network, currencyType, keys, tweakedKey, lockupTx);
    const partialSig = await boltzClient.getReverseClaimPartialSignature(argv.swapId, argv.preimage, tx.toHex(), 0, (0, Utils_1.getHexString)(Buffer.from(musig.getPublicNonce())));
    console.log((0, Utils_1.stringify)({
        claimTransaction: (0, TaprootHelper_1.finalizeCooperativeTransaction)(tx, musig, network, currencyType, theirPublicKey, details, partialSig).toHex(),
    }));
};
exports.handler = handler;
//# sourceMappingURL=CooperativeClaim.js.map