"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const axios_1 = __importDefault(require("axios"));
const crypto_1 = require("crypto");
const Utils_1 = require("../../Utils");
exports.command = 'queryreferrals <key> <secret>';
exports.describe = 'queries referrals with API key and secret';
exports.builder = {
    rest: {
        hidden: true,
    },
    'rest.host': {
        type: 'string',
        default: '127.0.0.1',
        describe: 'Boltz REST API host',
    },
    'rest.port': {
        type: 'number',
        default: '9001',
        describe: 'Boltz REST API port',
    },
    key: {
        type: 'string',
        describe: 'API key',
    },
    secret: {
        type: 'string',
        describe: 'API secret',
    },
};
const handler = async (argv) => {
    try {
        const [idRes, fees, stats] = await Promise.all([
            sendAuthenticatedRequest(argv, '/v2/referral'),
            sendAuthenticatedRequest(argv, '/v2/referral/fees'),
            sendAuthenticatedRequest(argv, '/v2/referral/stats'),
        ]);
        console.log((0, Utils_1.stringify)({ id: idRes.data.id, fees: fees.data, stats: stats.data }));
    }
    catch (error) {
        if (error.message && error.response) {
            console.error(`${error.message}: ${(0, Utils_1.stringify)(error.response.data)}`);
        }
        else {
            console.error(error);
        }
    }
};
exports.handler = handler;
const sendAuthenticatedRequest = (argv, path) => {
    const ts = (0, Utils_1.getUnixTime)();
    const hmac = (0, crypto_1.createHmac)('sha256', argv.secret)
        .update(`${ts}GET${path}`)
        .digest('hex');
    return axios_1.default.get(`http://${argv.rest.host}:${argv.rest.port}${path}`, {
        headers: {
            TS: ts.toString(),
            'API-KEY': argv.key,
            'API-HMAC': hmac,
        },
    });
};
//# sourceMappingURL=QueryReferrals.js.map