"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const boltzrpc_pb_1 = require("../../proto/boltzrpc_pb");
const BuilderComponents_1 = __importDefault(require("../BuilderComponents"));
const Command_1 = require("../Command");
exports.command = 'sendcoins <symbol> <address> <amount> [fee] [send_all]';
exports.describe = 'sends coins to a specified address';
exports.builder = {
    symbol: BuilderComponents_1.default.symbol,
    address: {
        describe: 'address to which the funds should be sent',
        type: 'string',
    },
    amount: {
        describe: 'amount that should be sent',
        type: 'number',
    },
    fee: {
        describe: 'sat/vbyte or gas price in gwei that should be paid as fee',
        type: 'number',
        default: 2,
    },
    send_all: {
        describe: 'ignores the amount and sends the whole balance of the wallet',
        type: 'boolean',
    },
};
const handler = (argv) => {
    const request = new boltzrpc_pb_1.SendCoinsRequest();
    request.setSymbol(argv.symbol);
    request.setAddress(argv.address);
    request.setAmount(argv.amount);
    request.setFee(argv.fee);
    request.setSendAll(argv.send_all);
    (0, Command_1.loadBoltzClient)(argv).sendCoins(request, (0, Command_1.callback)());
};
exports.handler = handler;
//# sourceMappingURL=SendCoins.js.map