"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const Utils_1 = require("../../Utils");
const boltzrpc_pb_1 = require("../../proto/boltzrpc_pb");
const Command_1 = require("../Command");
exports.command = 'unblindoutputs [id] [hex]';
exports.describe = 'unblinds the outputs of a transaction';
exports.builder = {
    id: {
        describe: 'id of the transaction to unblind',
        type: 'string',
    },
    hex: {
        describe: 'raw hex of the transaction to unblind',
        type: 'string',
    },
};
const handler = (argv) => {
    const request = new boltzrpc_pb_1.UnblindOutputsRequest();
    const isSet = (value) => value !== undefined && typeof value === 'string' && value !== '';
    const idSet = isSet(argv.id);
    const hexSet = isSet(argv.hex);
    if (!idSet && !hexSet) {
        console.error('Either id or hex must be set');
        return;
    }
    if (idSet) {
        request.setId(argv.id);
    }
    else if (hexSet) {
        request.setHex(argv.hex);
    }
    (0, Command_1.loadBoltzClient)(argv).unblindOutputs(request, (0, Command_1.callback)((res) => {
        return {
            outputList: res.getOutputsList().map((out) => {
                const res = {
                    ...out.toObject(),
                };
                for (const name of ['asset', 'script', 'nonce']) {
                    res[name] = (0, Utils_1.getHexString)(Buffer.from(out[`get${(0, Utils_1.capitalizeFirstLetter)(name)}_asU8`]()));
                }
                for (const name of ['rangeProof', 'surjectionProof']) {
                    res['isBlinded'] = res[name] !== undefined && res[name] !== '';
                    delete res[name];
                }
                return res;
            }),
        };
    }));
};
exports.handler = handler;
//# sourceMappingURL=UnblindOutputs.js.map