"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const Utils_1 = require("../../../Utils");
const ContractUtils_1 = require("../../../wallet/ethereum/ContractUtils");
const BuilderComponents_1 = __importDefault(require("../../BuilderComponents"));
const EthereumUtils_1 = require("../EthereumUtils");
exports.command = 'claim <preimage> [token]';
exports.describe = 'claims Ether or a ERC20 token from the corresponding swap contract';
exports.builder = {
    preimage: {
        describe: 'preimage with which the funds have been locked',
        type: 'string',
    },
    token: BuilderComponents_1.default.token,
    queryStartHeightDelta: BuilderComponents_1.default.queryStartHeightDelta,
};
const handler = async (argv) => {
    const signer = await (0, EthereumUtils_1.connectEthereum)(argv.provider);
    const { etherSwap, erc20Swap } = (0, EthereumUtils_1.getContracts)(argv.chain, signer);
    const preimage = (0, Utils_1.getHexBuffer)(argv.preimage);
    let transaction;
    if (argv.token) {
        const erc20SwapValues = await (0, ContractUtils_1.queryERC20SwapValues)(erc20Swap, bitcoinjs_lib_1.crypto.sha256(preimage), await (0, EthereumUtils_1.getLogsQueryStartHeight)(signer.provider, argv.queryStartHeightDelta));
        transaction = await erc20Swap['claim(bytes32,uint256,address,address,uint256)'](preimage, erc20SwapValues.amount, erc20SwapValues.tokenAddress, erc20SwapValues.refundAddress, erc20SwapValues.timelock);
    }
    else {
        const etherSwapValues = await (0, ContractUtils_1.queryEtherSwapValues)(etherSwap, bitcoinjs_lib_1.crypto.sha256(preimage), await (0, EthereumUtils_1.getLogsQueryStartHeight)(signer.provider, argv.queryStartHeightDelta));
        transaction = await etherSwap['claim(bytes32,uint256,address,uint256)'](preimage, etherSwapValues.amount, etherSwapValues.refundAddress, etherSwapValues.timelock);
    }
    console.log(`Claimed ${argv.token ? 'ERC20 token' : 'Ether'} in: ${transaction.hash}`);
};
exports.handler = handler;
//# sourceMappingURL=Claim.js.map