"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const Utils_1 = require("../../../Utils");
const Consts_1 = require("../../../consts/Consts");
const BuilderComponents_1 = __importDefault(require("../../BuilderComponents"));
const EthereumUtils_1 = require("../EthereumUtils");
exports.command = 'lock <preimageHash> <amount> <timelock> [token]';
exports.describe = 'locks Ether or a ERC20 token in the corresponding swap contract';
exports.builder = {
    preimageHash: {
        describe: 'preimage hash with which the funds should be locked',
        type: 'string',
    },
    amount: {
        describe: 'amount of tokens that should be locked up',
        type: 'number',
    },
    timelock: {
        describe: 'timelock delta in blocks',
        type: 'number',
    },
    token: BuilderComponents_1.default.token,
};
const handler = async (argv) => {
    const signer = await (0, EthereumUtils_1.connectEthereum)(argv.provider);
    const { etherSwap, erc20Swap, token } = (0, EthereumUtils_1.getContracts)(argv.chain, signer);
    const preimageHash = (0, Utils_1.getHexBuffer)(argv.preimageHash);
    const amount = BigInt(argv.amount) * Consts_1.etherDecimals;
    const boltzAddress = await (0, EthereumUtils_1.getBoltzAddress)();
    if (boltzAddress === undefined) {
        console.log('Could not lock coins because the address of Boltz could not be queried');
        return;
    }
    let transaction;
    if (argv.token) {
        await token.approve(await erc20Swap.getAddress(), amount);
        transaction = await erc20Swap.lock(preimageHash, amount, await token.getAddress(), boltzAddress, argv.timelock);
    }
    else {
        transaction = await etherSwap.lock(preimageHash, boltzAddress, argv.timelock, {
            value: amount,
        });
    }
    console.log(`Sent ${argv.token ? 'ERC20 token' : 'Ether'} in: ${transaction.hash}`);
};
exports.handler = handler;
//# sourceMappingURL=Lock.js.map