"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const Utils_1 = require("../../../Utils");
const ContractUtils_1 = require("../../../wallet/ethereum/ContractUtils");
const BuilderComponents_1 = __importDefault(require("../../BuilderComponents"));
const EthereumUtils_1 = require("../EthereumUtils");
exports.command = 'refund <preimageHash> [token]';
exports.describe = 'refunds Ether or a ERC20 token from the corresponding swap contract';
exports.builder = {
    preimageHash: {
        describe: 'preimage hash with which the funds have been locked',
        type: 'string',
    },
    token: BuilderComponents_1.default.token,
};
const handler = async (argv) => {
    const signer = await (0, EthereumUtils_1.connectEthereum)(argv.provider);
    const { etherSwap, erc20Swap } = (0, EthereumUtils_1.getContracts)(argv.chain, signer);
    const preimageHash = (0, Utils_1.getHexBuffer)(argv.preimageHash);
    let transaction;
    if (argv.token) {
        const erc20SwapValues = await (0, ContractUtils_1.queryERC20SwapValues)(erc20Swap, preimageHash, await (0, EthereumUtils_1.getLogsQueryStartHeight)(signer.provider, argv.queryStartHeightDelta));
        transaction = await erc20Swap.refund(preimageHash, erc20SwapValues.amount, erc20SwapValues.tokenAddress, erc20SwapValues.claimAddress, erc20SwapValues.timelock);
    }
    else {
        const etherSwapValues = await (0, ContractUtils_1.queryEtherSwapValues)(etherSwap, preimageHash, await (0, EthereumUtils_1.getLogsQueryStartHeight)(signer.provider, argv.queryStartHeightDelta));
        transaction = await etherSwap.refund(preimageHash, etherSwapValues.amount, etherSwapValues.claimAddress, etherSwapValues.timelock);
    }
    console.log(`Refunded ${argv.token ? 'ERC20 token' : 'Ether'} in: ${transaction.hash}`);
};
exports.handler = handler;
//# sourceMappingURL=Refund.js.map