"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.command = void 0;
const Consts_1 = require("../../../consts/Consts");
const BuilderComponents_1 = __importDefault(require("../../BuilderComponents"));
const EthereumUtils_1 = require("../EthereumUtils");
exports.command = 'send <amount> [destination] [token]';
exports.describe = 'sends Ether or a ERC20 token to the destined address';
exports.builder = {
    amount: {
        describe: 'amount of tokens that should be sent',
        type: 'number',
    },
    destination: {
        describe: 'address to which the coins should be sent',
        type: 'string',
    },
    token: BuilderComponents_1.default.token,
};
const handler = async (argv) => {
    const signer = await (0, EthereumUtils_1.connectEthereum)(argv.provider);
    const { token } = (0, EthereumUtils_1.getContracts)(argv.chain, signer);
    const amount = BigInt(argv.amount) * Consts_1.etherDecimals;
    let transactionHash;
    const destination = argv.destination || (await (0, EthereumUtils_1.getBoltzAddress)());
    if (destination === undefined) {
        console.log('Did not send coins because no onchain address was specified');
        return;
    }
    if (argv.token) {
        const transaction = await token.transfer(destination, amount);
        await transaction.wait(1);
        transactionHash = transaction.hash;
    }
    else {
        const transaction = await signer.sendTransaction({
            value: amount,
            to: destination,
        });
        await transaction.wait(1);
        transactionHash = transaction.hash;
    }
    console.log(`Sent ${argv.token ? 'ERC20 token' : 'Ether'} in: ${transactionHash}`);
};
exports.handler = handler;
//# sourceMappingURL=Send.js.map