"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const DenominationConverter_1 = require("../DenominationConverter");
const StatsRepository_1 = __importDefault(require("../db/repositories/StatsRepository"));
const Utils_1 = require("./Utils");
class Stats {
}
_a = Stats;
Stats.totalString = 'total';
Stats.generate = async (minYear, minMonth, referral) => {
    const [volumes, tradeCounts, failureRates] = await Promise.all([
        StatsRepository_1.default.getVolume(minYear, minMonth, referral),
        StatsRepository_1.default.getTradeCounts(minYear, minMonth, referral),
        StatsRepository_1.default.getFailureRates(minYear, minMonth, referral),
    ]);
    const stats = {};
    const getMonthObj = ({ year, month }) => {
        return (0, Utils_1.getNestedObject)((0, Utils_1.getNestedObject)(stats, year), month);
    };
    volumes.forEach((volume) => {
        const obj = (0, Utils_1.getNestedObject)(getMonthObj(volume), 'volume');
        obj[volume.pair || _a.totalString] = (0, DenominationConverter_1.satoshisToPaddedCoins)(volume.sum);
    });
    tradeCounts.forEach((counts) => {
        const obj = (0, Utils_1.getNestedObject)(getMonthObj(counts), 'trades');
        obj[counts.pair || _a.totalString] = counts.count;
    });
    failureRates.forEach((fails) => {
        const obj = (0, Utils_1.getNestedObject)(getMonthObj(fails), 'failureRates');
        obj[fails.isReverse ? 'reverseSwaps' : 'swaps'] = fails.failureRate;
    });
    return stats;
};
exports.default = Stats;
//# sourceMappingURL=Stats.js.map