import Sequelize from 'sequelize';
import { PostgresConfig } from '../Config';
import Logger from '../Logger';
import { Currency } from '../wallet/WalletManager';
declare enum DatabaseType {
    'SQLite' = 0,
    'PostgreSQL' = 1
}
declare class Database {
    private readonly logger;
    private readonly sqlitePath;
    static readonly memoryDatabase = ":memory:";
    static type: DatabaseType;
    static sequelize: Sequelize.Sequelize;
    private migration;
    /**
     * @param logger logger that should be used
     * @param sqlitePath the file path to the SQLite database; if ':memory:' the database will be stored in the memory
     * @param postgresConfig configuration of connection to a PostgreSQL database; takes precedence over SQLite if set
     */
    constructor(logger: Logger, sqlitePath: string, postgresConfig?: PostgresConfig);
    init: () => Promise<void>;
    migrate: (currencies: Map<string, Currency>) => Promise<void>;
    close: () => Promise<void>;
    private loadModels;
}
export default Database;
export { DatabaseType };
