"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const boltz_core_1 = require("boltz-core");
const sequelize_1 = require("sequelize");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const ChannelCreation_1 = __importDefault(require("./models/ChannelCreation"));
const DatabaseVersion_1 = __importDefault(require("./models/DatabaseVersion"));
const Referral_1 = __importDefault(require("./models/Referral"));
const ReverseSwap_1 = __importStar(require("./models/ReverseSwap"));
const Swap_1 = __importDefault(require("./models/Swap"));
const DatabaseVersionRepository_1 = __importDefault(require("./repositories/DatabaseVersionRepository"));
// TODO: integration tests for actual migrations
class Migration {
    constructor(logger, sequelize) {
        this.logger = logger;
        this.sequelize = sequelize;
        this.migrate = async (currencies) => {
            await DatabaseVersion_1.default.sync();
            const versionRow = await DatabaseVersionRepository_1.default.getVersion();
            // When no version row is found, just insert the latest version into the database
            if (!versionRow) {
                this.logger.verbose('No schema version found in database');
                this.logger.debug(`Inserting latest schema version ${Migration.latestSchemaVersion} in database`);
                await DatabaseVersionRepository_1.default.createVersion(Migration.latestSchemaVersion);
                return;
            }
            if (versionRow.version === Migration.latestSchemaVersion) {
                this.logger.verbose(`Database has latest schema version ${Migration.latestSchemaVersion}`);
                return;
            }
            this.logOutdatedVersion(versionRow.version);
            switch (versionRow.version) {
                // The migration from schema version 1 to 2 adds support for Ether and ERC20 tokens
                // Which means that we can safely assume that all Swaps that are in the database
                // already were on a Bitcoin like chain
                case 1: {
                    // Sanity check the chain clients
                    for (const currency of currencies.values()) {
                        try {
                            if (currency.chainClient) {
                                this.logger.debug(`Sanity checking ${currency.symbol} chain client for migration`);
                                await currency.chainClient.getBlockchainInfo();
                                this.logger.debug(`${currency.symbol} chain client is ready for migration`);
                            }
                        }
                        catch (error) {
                            throw `could not connect to to chain client of ${currency.symbol}: ${(0, Utils_1.formatError)(error)}`;
                        }
                    }
                    this.logUpdatingTable('swaps');
                    // Add the missing columns to make querying via the model possible
                    await this.sequelize.query('ALTER TABLE swaps ADD failureReason VARCHAR(255)');
                    await this.sequelize.query('ALTER TABLE swaps ADD lockupTransactionVout VARCHAR(255)');
                    const allSwaps = await Swap_1.default.findAll();
                    const allChannelCreations = await ChannelCreation_1.default.findAll();
                    // To drop the "swaps" table, we also need to drop "channelCreations" because it has a foreign constraint
                    await this.dropTable('channelCreations');
                    await this.dropTable('swaps');
                    await Swap_1.default.sync();
                    await ChannelCreation_1.default.sync();
                    for (const swap of allSwaps) {
                        let lockupTransactionVout = null;
                        if (swap.lockupTransactionId) {
                            const { base, quote } = (0, Utils_1.splitPairId)(swap.pair);
                            const chainCurrency = (0, Utils_1.getChainCurrency)(base, quote, swap.orderSide, false);
                            const chainClient = currencies.get(chainCurrency).chainClient;
                            const lockupTransaction = bitcoinjs_lib_1.Transaction.fromHex(await chainClient.getRawTransaction(swap.lockupTransactionId));
                            lockupTransactionVout = (0, boltz_core_1.detectSwap)((0, Utils_1.getHexBuffer)(swap.redeemScript), lockupTransaction).vout;
                        }
                        await Swap_1.default.create({
                            ...this.getModelDataValues(swap),
                            lockupTransactionVout,
                        });
                    }
                    for (const channelCreation of allChannelCreations) {
                        await ChannelCreation_1.default.create({
                            ...this.getModelDataValues(channelCreation),
                        });
                    }
                    this.logUpdatingTable('reverseSwaps');
                    // Add the missing columns to make querying via the model possible
                    await this.sequelize.query('ALTER TABLE reverseSwaps ADD claimAddress VARCHAR(255)');
                    await this.sequelize.query('ALTER TABLE reverseSwaps ADD preimageHash VARCHAR(255)');
                    await this.sequelize.query('ALTER TABLE reverseSwaps ADD failureReason VARCHAR(255)');
                    const allReverseSwaps = await ReverseSwap_1.default.findAll();
                    await this.dropTable('reverseSwaps');
                    await ReverseSwap_1.default.sync();
                    for (const reverseSwap of allReverseSwaps) {
                        // The "claimAddress" does not have to be set because it is only needed for Swaps on the Ethereum chain and
                        // databases of the schema version 1 do not support Ethereum Swaps
                        await ReverseSwap_1.default.create({
                            ...this.getModelDataValues(reverseSwap),
                            preimageHash: (0, Utils_1.decodeInvoice)(reverseSwap.invoice).paymentHash,
                        });
                    }
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                }
                // Database schema version 3 adds support for the prepay miner fee on the Ethereum chain
                case 2:
                    this.logUpdatingTable('reverseSwaps');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'minerFeeOnchainAmount', {
                        type: new sequelize_1.DataTypes.INTEGER(),
                        allowNull: true,
                    });
                    // Because adding unique columns is not possible with SQLite, that property is omitted here
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'minerFeeInvoicePreimage', {
                        type: new sequelize_1.DataTypes.STRING(64),
                        allowNull: true,
                    });
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                // Schema version 4 adds referrals for fee sharing
                case 3:
                    this.logUpdatingTable('swaps');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('swaps', 'referral', {
                        type: new sequelize_1.DataTypes.STRING(255),
                        allowNull: true,
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .addIndex('swaps', ['referral'], {
                        unique: false,
                        fields: ['referral'],
                    });
                    this.logUpdatingTable('reverseSwaps');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'referral', {
                        type: new sequelize_1.DataTypes.STRING(255),
                        allowNull: true,
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .addIndex('reverseSwaps', ['referral'], {
                        unique: false,
                        fields: ['referral'],
                    });
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                // Schema version 5 adds API keys to referrals
                case 4: {
                    this.logUpdatingTable('referrals');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('referrals', 'apiKey', {
                        type: new sequelize_1.DataTypes.STRING(255),
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('referrals', 'apiSecret', {
                        type: new sequelize_1.DataTypes.STRING(255),
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .addIndex('referrals', ['apiKey']);
                    const referrals = await Referral_1.default.findAll();
                    for (const referral of referrals) {
                        await referral.update({
                            apiKey: (0, Utils_1.createApiCredential)(),
                            apiSecret: (0, Utils_1.createApiCredential)(),
                        });
                    }
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('referrals', 'apiKey', {
                        unique: true,
                        allowNull: false,
                        type: new sequelize_1.DataTypes.STRING(255),
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('referrals', 'apiSecret', {
                        unique: true,
                        allowNull: false,
                        type: new sequelize_1.DataTypes.STRING(255),
                    });
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                }
                case 5: {
                    this.logUpdatingTable('swaps');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('swaps', 'invoiceAmount', {
                        allowNull: true,
                        type: new sequelize_1.DataTypes.INTEGER(),
                    });
                    await this.logProgress('swaps', 100, await Swap_1.default.findAll({
                        attributes: ['id', 'invoice'],
                        where: {
                            invoice: {
                                [sequelize_1.Op.not]: null,
                            },
                        },
                    }), async (swap) => {
                        await swap.update({
                            invoiceAmount: (0, Utils_1.decodeInvoice)(swap.invoice).satoshis,
                        });
                    });
                    this.logUpdatingTable('reverseSwaps');
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'invoiceAmount', {
                        type: new sequelize_1.DataTypes.INTEGER(),
                    });
                    await this.logProgress('reverseSwaps', 100, await ReverseSwap_1.default.findAll({
                        attributes: ['id', 'invoice'],
                    }), async (reverseSwap) => {
                        await reverseSwap.update({
                            invoiceAmount: (0, Utils_1.decodeInvoice)(reverseSwap.invoice).satoshis,
                        });
                    });
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('reverseSwaps', 'invoiceAmount', {
                        allowNull: false,
                        type: new sequelize_1.DataTypes.INTEGER(),
                    });
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                }
                case 6: {
                    this.logUpdatingTable('reverseSwaps');
                    const attrs = {
                        type: new sequelize_1.DataTypes.INTEGER(),
                        allowNull: true,
                        validate: {
                            isIn: [
                                Object.values(ReverseSwap_1.NodeType).filter((val) => typeof val === 'number'),
                            ],
                        },
                    };
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'node', attrs);
                    await this.sequelize
                        .getQueryInterface()
                        .bulkUpdate('reverseSwaps', { node: ReverseSwap_1.NodeType.LND }, {});
                    attrs.allowNull = false;
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('reverseSwaps', 'node', attrs);
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                }
                case 7: {
                    this.logUpdatingTable('swaps');
                    const attrs = {
                        type: new sequelize_1.DataTypes.INTEGER(),
                        allowNull: true,
                        validate: {
                            isIn: [
                                Object.values(Enums_1.SwapVersion).filter((val) => typeof val === 'number'),
                            ],
                        },
                    };
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('swaps', 'version', attrs);
                    await this.sequelize
                        .getQueryInterface()
                        .bulkUpdate('swaps', { version: Enums_1.SwapVersion.Legacy }, {});
                    attrs.allowNull = false;
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('swaps', 'version', attrs);
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('swaps', 'refundPublicKey', {
                        type: new sequelize_1.DataTypes.STRING(),
                        allowNull: true,
                    });
                    this.logUpdatingTable('reverseSwaps');
                    attrs.allowNull = true;
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'version', attrs);
                    await this.sequelize
                        .getQueryInterface()
                        .bulkUpdate('reverseSwaps', { version: Enums_1.SwapVersion.Legacy }, {});
                    attrs.allowNull = false;
                    await this.sequelize
                        .getQueryInterface()
                        .changeColumn('reverseSwaps', 'version', attrs);
                    await this.sequelize
                        .getQueryInterface()
                        .addColumn('reverseSwaps', 'claimPublicKey', {
                        type: new sequelize_1.DataTypes.STRING(),
                        allowNull: true,
                    });
                    await this.finishMigration(versionRow.version, currencies);
                    break;
                }
                default:
                    throw `found unexpected database version ${versionRow.version}`;
            }
        };
        this.dropTable = async (table) => {
            await this.sequelize.query(`DROP TABLE ${table}`);
        };
        this.getModelDataValues = (model) => {
            return model['dataValues'];
        };
        this.logUpdatingTable = (table) => {
            this.logger.verbose(`Updating database table ${table}`);
        };
        this.finishMigration = async (updatedFromVersion, currencies) => {
            const currentVersion = updatedFromVersion + 1;
            this.logger.info(`Finished database migration to schema version ${currentVersion}`);
            await DatabaseVersionRepository_1.default.updateVersion(currentVersion);
            // Run the migration again if the current schema version is not the latest one
            if (currentVersion !== Migration.latestSchemaVersion) {
                await this.migrate(currencies);
            }
        };
        this.logOutdatedVersion = (version) => {
            this.logger.warn(`Found database with outdated schema version ${version}`);
            this.logger.info(`Starting migration to database schema version ${version + 1}`);
        };
        this.logProgress = async (name, logIncrement, entries, cb) => {
            this.logger.debug(`Migrating ${entries.length} ${name}`);
            for (const [index, entry] of entries.entries()) {
                await cb(entry);
                if (index !== 0 && index % logIncrement === 0) {
                    this.logger.debug(`Migrated ${index}/${entries.length} ${name}`);
                }
            }
        };
    }
}
Migration.latestSchemaVersion = 8;
exports.default = Migration;
//# sourceMappingURL=Migration.js.map