"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeType = void 0;
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const Pair_1 = __importDefault(require("./Pair"));
var NodeType;
(function (NodeType) {
    NodeType[NodeType["LND"] = 0] = "LND";
    NodeType[NodeType["CLN"] = 1] = "CLN";
})(NodeType || (exports.NodeType = NodeType = {}));
class ReverseSwap extends sequelize_1.Model {
}
ReverseSwap.load = (sequelize) => {
    ReverseSwap.init({
        id: {
            type: new sequelize_1.DataTypes.STRING(255),
            primaryKey: true,
            allowNull: false,
        },
        version: {
            type: new sequelize_1.DataTypes.INTEGER(),
            allowNull: false,
            validate: {
                isIn: [
                    Object.values(Enums_1.SwapVersion).filter((val) => typeof val === 'number'),
                ],
            },
        },
        lockupAddress: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        keyIndex: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: true },
        claimPublicKey: { type: new sequelize_1.DataTypes.STRING(), allowNull: true },
        redeemScript: { type: new sequelize_1.DataTypes.TEXT(), allowNull: true },
        claimAddress: { type: new sequelize_1.DataTypes.STRING(255), allowNull: true },
        fee: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: false },
        referral: { type: new sequelize_1.DataTypes.STRING(255), allowNull: true },
        minerFee: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        pair: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        orderSide: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: false },
        status: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        failureReason: { type: new sequelize_1.DataTypes.STRING(255), allowNull: true },
        timeoutBlockHeight: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: false },
        node: {
            type: new sequelize_1.DataTypes.INTEGER(),
            allowNull: false,
            validate: {
                isIn: [
                    Object.values(NodeType).filter((val) => typeof val === 'number'),
                ],
            },
        },
        invoice: {
            type: new sequelize_1.DataTypes.TEXT(),
            allowNull: false,
            unique: true,
        },
        invoiceAmount: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: false },
        minerFeeInvoice: {
            type: new sequelize_1.DataTypes.TEXT(),
            allowNull: true,
            unique: true,
        },
        minerFeeInvoicePreimage: {
            type: new sequelize_1.DataTypes.STRING(64),
            allowNull: true,
            unique: true,
        },
        minerFeeOnchainAmount: {
            type: new sequelize_1.DataTypes.BIGINT(),
            allowNull: true,
        },
        preimageHash: {
            type: new sequelize_1.DataTypes.STRING(64),
            allowNull: false,
            unique: true,
        },
        preimage: { type: new sequelize_1.DataTypes.STRING(64), allowNull: true },
        onchainAmount: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: false },
        transactionId: { type: new sequelize_1.DataTypes.STRING(255), allowNull: true },
        transactionVout: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: true },
    }, {
        sequelize,
        tableName: 'reverseSwaps',
        indexes: [
            {
                unique: true,
                fields: ['id'],
            },
            {
                unique: false,
                fields: ['status'],
            },
            {
                unique: true,
                fields: ['preimageHash'],
            },
            {
                unique: true,
                fields: ['invoice'],
            },
            {
                unique: true,
                fields: ['minerFeeInvoice'],
            },
            {
                unique: false,
                fields: ['referral'],
            },
            {
                unique: false,
                fields: ['lockupAddress'],
            },
            {
                unique: false,
                fields: ['transactionId'],
            },
        ],
    });
    ReverseSwap.belongsTo(Pair_1.default, {
        foreignKey: 'pair',
    });
};
exports.default = ReverseSwap;
//# sourceMappingURL=ReverseSwap.js.map