"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const Pair_1 = __importDefault(require("./Pair"));
class Swap extends sequelize_1.Model {
}
Swap.load = (sequelize) => {
    Swap.init({
        id: {
            type: new sequelize_1.DataTypes.STRING(255),
            primaryKey: true,
            allowNull: false,
        },
        version: {
            type: new sequelize_1.DataTypes.INTEGER(),
            allowNull: false,
            validate: {
                isIn: [
                    Object.values(Enums_1.SwapVersion).filter((val) => typeof val === 'number'),
                ],
            },
        },
        keyIndex: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: true },
        refundPublicKey: { type: new sequelize_1.DataTypes.STRING(), allowNull: true },
        redeemScript: { type: new sequelize_1.DataTypes.TEXT(), allowNull: true },
        fee: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        referral: { type: new sequelize_1.DataTypes.STRING(255), allowNull: true },
        routingFee: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: true },
        minerFee: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        pair: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        orderSide: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: false },
        status: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        failureReason: { type: new sequelize_1.DataTypes.TEXT(), allowNull: true },
        preimageHash: {
            type: new sequelize_1.DataTypes.STRING(64),
            allowNull: false,
            unique: true,
        },
        invoice: {
            type: new sequelize_1.DataTypes.TEXT(),
            allowNull: true,
            unique: true,
        },
        invoiceAmount: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        acceptZeroConf: { type: sequelize_1.DataTypes.BOOLEAN, allowNull: true },
        timeoutBlockHeight: { type: new sequelize_1.DataTypes.INTEGER(), allowNull: false },
        rate: { type: new sequelize_1.DataTypes.REAL(), allowNull: true },
        expectedAmount: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        onchainAmount: { type: new sequelize_1.DataTypes.BIGINT(), allowNull: true },
        lockupAddress: { type: new sequelize_1.DataTypes.STRING(255), allowNull: false },
        lockupTransactionId: {
            type: new sequelize_1.DataTypes.STRING(255),
            allowNull: true,
        },
        lockupTransactionVout: {
            type: new sequelize_1.DataTypes.INTEGER(),
            allowNull: true,
        },
    }, {
        sequelize,
        tableName: 'swaps',
        indexes: [
            {
                unique: true,
                fields: ['id'],
            },
            {
                unique: false,
                fields: ['status'],
            },
            {
                unique: true,
                fields: ['preimageHash'],
            },
            {
                unique: true,
                fields: ['invoice'],
            },
            {
                unique: false,
                fields: ['referral'],
            },
            {
                unique: false,
                fields: ['lockupAddress'],
            },
            {
                unique: false,
                fields: ['lockupTransactionId'],
            },
        ],
    });
    Swap.belongsTo(Pair_1.default, {
        foreignKey: 'pair',
    });
};
exports.default = Swap;
//# sourceMappingURL=Swap.js.map