"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const Database_1 = __importStar(require("../Database"));
const Referral_1 = __importDefault(require("../models/Referral"));
class ReferralRepository {
}
_a = ReferralRepository;
ReferralRepository.referralSumQuery = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT pair, status, fee, referral, "createdAt" FROM swaps
    UNION ALL
    SELECT pair, status, fee, referral, "createdAt" FROM "reverseSwaps"
)
SELECT
    EXTRACT(YEAR from data."createdAt") AS year,
    EXTRACT(MONTH from data."createdAt") AS month,
    pair,
    data.referral as referral,
    (SUM(data.fee * r."feeShare") / 100)::BIGINT AS sum
FROM data
    INNER JOIN referrals r ON data.referral = r.id
WHERE data.status IN (?)
GROUP BY year, month, pair, data.referral
ORDER BY year, month;
    `,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT pair, status, fee, referral, createdAt FROM swaps
    UNION ALL
    SELECT pair, status, fee, referral, createdAt FROM reverseSwaps
)
SELECT
    CAST(STRFTIME('%Y', data.createdAt) AS INT) AS year,
    CAST(STRFTIME('%m', data.createdAt) AS INT) AS month,
    pair,
    referral,
    SUM(data.fee * referrals.feeShare) / 100 AS sum
FROM data
    INNER JOIN referrals ON data.referral = referrals.id
WHERE data.status IN (?)
GROUP BY year, month, pair, referral
ORDER BY year, month;
`,
};
ReferralRepository.addReferral = (referral) => {
    return Referral_1.default.create(referral);
};
ReferralRepository.getReferrals = () => {
    return Referral_1.default.findAll();
};
ReferralRepository.getReferralByApiKey = (apiKey) => {
    return Referral_1.default.findOne({
        where: {
            apiKey,
        },
    });
};
ReferralRepository.getReferralByRoutingNode = (routingNode) => {
    return Referral_1.default.findOne({
        where: {
            routingNode,
        },
    });
};
ReferralRepository.getReferralSum = async (referralKey) => {
    const res = await Database_1.default.sequelize.query({
        query: _a.referralSumQuery[Database_1.default.type],
        values: [Enums_1.SuccessSwapUpdateEvents],
    }, {
        type: sequelize_1.QueryTypes.SELECT,
    });
    if (referralKey) {
        return res.filter((row) => row.referral === referralKey);
    }
    return res;
};
exports.default = ReferralRepository;
//# sourceMappingURL=ReferralRepository.js.map