"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const ReverseSwap_1 = __importDefault(require("../models/ReverseSwap"));
class ReverseSwapRepository {
}
ReverseSwapRepository.getReverseSwaps = (options) => {
    return ReverseSwap_1.default.findAll({
        where: options,
    });
};
ReverseSwapRepository.getReverseSwapsExpirable = (height) => {
    return ReverseSwap_1.default.findAll({
        where: {
            status: {
                [sequelize_1.Op.notIn]: [
                    Enums_1.SwapUpdateEvent.SwapExpired,
                    Enums_1.SwapUpdateEvent.TransactionFailed,
                    Enums_1.SwapUpdateEvent.TransactionRefunded,
                    Enums_1.SwapUpdateEvent.InvoiceSettled,
                ],
            },
            timeoutBlockHeight: {
                [sequelize_1.Op.lte]: height,
            },
        },
    });
};
ReverseSwapRepository.getReverseSwap = (options) => {
    return ReverseSwap_1.default.findOne({
        where: options,
    });
};
ReverseSwapRepository.addReverseSwap = (reverseSwap) => {
    return ReverseSwap_1.default.create(reverseSwap);
};
ReverseSwapRepository.setReverseSwapStatus = (reverseSwap, status, failureReason) => {
    return reverseSwap.update({
        status,
        failureReason,
    });
};
ReverseSwapRepository.setLockupTransaction = (reverseSwap, transactionId, minerFee, vout) => {
    return reverseSwap.update({
        minerFee,
        transactionId,
        transactionVout: vout,
        status: Enums_1.SwapUpdateEvent.TransactionMempool,
    });
};
ReverseSwapRepository.setPreimage = (reverseSwap, preimage) => reverseSwap.update({ preimage });
ReverseSwapRepository.setInvoiceSettled = (reverseSwap, preimage) => {
    return reverseSwap.update({
        preimage,
        status: Enums_1.SwapUpdateEvent.InvoiceSettled,
    });
};
ReverseSwapRepository.setTransactionRefunded = (reverseSwap, minerFee, failureReason) => {
    return reverseSwap.update({
        failureReason,
        minerFee: reverseSwap.minerFee + minerFee,
        status: Enums_1.SwapUpdateEvent.TransactionRefunded,
    });
};
ReverseSwapRepository.dropTable = () => {
    return ReverseSwap_1.default.drop();
};
exports.default = ReverseSwapRepository;
//# sourceMappingURL=ReverseSwapRepository.js.map