type StatsDate = {
    year: number;
    month: number;
};
type Volume = StatsDate & {
    pair?: string;
    sum: number;
};
type TradeCount = StatsDate & {
    pair?: string;
    count: number;
};
type FailureRate = StatsDate & {
    isReverse: boolean;
    failureRate: number;
};
declare enum SwapType {
    Swap = "swap",
    Reverse = "reverse"
}
type BaseMetric = {
    pair: string;
    type: SwapType;
};
type SwapCount = BaseMetric & {
    status: 'success' | 'failure' | 'timeout';
    count: number;
};
type VolumePerPairType = BaseMetric & {
    volume: number;
};
type PendingSwaps = BaseMetric & {
    count: number;
};
type LockedFunds = {
    pair: string;
    locked: number;
};
declare class StatsRepository {
    private static readonly queryVolume;
    private static readonly queryTradeCounts;
    private static readonly queryFailureRates;
    private static readonly querySwapCounts;
    private static readonly queryVolumePerPairType;
    private static readonly queryPendingSwapsCounts;
    private static readonly queryLockedFunds;
    static getVolume: (minYear?: number, minMonth?: number, referral?: string | null) => Promise<Volume[]>;
    static getTradeCounts: (minYear?: number, minMonth?: number, referral?: string | null) => Promise<TradeCount[]>;
    static getFailureRates: (minYear?: number, minMonth?: number, referral?: string | null) => Promise<FailureRate[]>;
    static getSwapCounts: () => Promise<SwapCount[]>;
    static getVolumePerPairType: () => Promise<VolumePerPairType[]>;
    static getPendingSwapsCounts: () => Promise<PendingSwaps[]>;
    static getLockedFunds: () => Promise<LockedFunds[]>;
    private static query;
}
export default StatsRepository;
export { SwapType, StatsDate };
