"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwapType = void 0;
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const Database_1 = __importStar(require("../Database"));
var SwapType;
(function (SwapType) {
    SwapType["Swap"] = "swap";
    SwapType["Reverse"] = "reverse";
})(SwapType || (exports.SwapType = SwapType = {}));
class StatsRepository {
}
StatsRepository.queryVolume = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT
        pair,
        status,
        referral,
        CASE WHEN "orderSide" = 1
            THEN "invoiceAmount"
            ELSE "onchainAmount"
        END AS amount,
        "createdAt"
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT
        pair,
        status,
        referral,
        CASE WHEN "orderSide" = 1
            THEN "onchainAmount"
            ELSE "invoiceAmount"
        END AS amount,
        "createdAt"
    FROM "reverseSwaps"
    WHERE status = ?
)
SELECT
    EXTRACT(YEAR FROM "createdAt") AS year,
    EXTRACT(MONTH FROM "createdAt") AS month,
    pair,
    SUM(amount) AS sum
FROM data
WHERE
    CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END
    AND ((
      EXTRACT(YEAR FROM "createdAt") >= ? AND
      EXTRACT(MONTH FROM "createdAt") >= ?
    ) OR EXTRACT(YEAR from "createdAt") > ?)
GROUP BY GROUPING SETS (
    (year, month),
    (year, month, pair)
)
ORDER BY year, month, pair NULLS FIRST;    
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT
        pair,
        status,
        referral,
        CASE WHEN orderSide THEN invoiceAmount ELSE onchainAmount END AS amount,
        createdAt
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT
        pair,
        status,
        referral,
        CASE WHEN orderSide THEN onchainAmount ELSE invoiceAmount END AS amount,
        createdAt
    FROM reverseSwaps
    WHERE status = ?
), groupedSwaps AS (
    SELECT
        CAST(STRFTIME('%Y', createdAt) AS INT) AS year,
        CAST(STRFTIME('%m', createdAt) AS INT) AS month,
        pair,
        SUM(amount) AS sum
    FROM data
    WHERE CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END
    GROUP BY year, month, pair
    ORDER BY year, month, pair
), groupedTotals AS (
    SELECT * FROM groupedSwaps
    UNION ALL
    SELECT year, month, NULL AS pair, SUM(sum) AS sum
    FROM groupedSwaps
    GROUP BY year, month
)
SELECT * FROM groupedTotals
WHERE (year >= ? AND month >= ?) OR year > ?
ORDER BY year, month, pair;
`,
};
StatsRepository.queryTradeCounts = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT pair, status, referral, "createdAt"
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT pair, status, referral, "createdAt"
    FROM "reverseSwaps"
    WHERE status = ?
)
SELECT
    EXTRACT(YEAR FROM "createdAt") AS year,
    EXTRACT(MONTH FROM "createdAt") AS month,
    pair,
    COUNT(*) AS count
FROM data
WHERE
    CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END
    AND ((
        EXTRACT(YEAR FROM "createdAt") >= ? AND
        EXTRACT(MONTH FROM "createdAt") >= ?
    ) OR EXTRACT(YEAR FROM "createdAt") > ?)
GROUP BY GROUPING SETS (
    (year, month),
    (pair, year, month)
)
ORDER BY year, month, pair NULLS FIRST;
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT pair, status, referral, createdAt
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT pair, status, referral, createdAt
    FROM reverseSwaps
    WHERE status = ?
), groupedSwaps AS (
    SELECT
        CAST(STRFTIME('%Y', createdAt) AS INT) AS year,
        CAST(STRFTIME('%m', createdAt) AS INT) AS month,
        pair,
        COUNT(*) AS count
    FROM data
    WHERE CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END
    GROUP BY pair, year, month
    ORDER BY year, month
), groupedTotals AS (
    SELECT * FROM groupedSwaps
    UNION ALL
    SELECT year, month, NULL AS pair, SUM(count) AS count
    FROM groupedSwaps
    GROUP BY year, month
)
SELECT * FROM groupedTotals
WHERE (year >= ? AND month >= ?) OR year > ?
ORDER BY year, month, pair;
`,
};
StatsRepository.queryFailureRates = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT pair, false AS "isReverse", status, referral, "createdAt" FROM swaps
    UNION ALL
    SELECT pair, true as "isReverse", status, referral, "createdAt" FROM "reverseSwaps"
)
SELECT
    EXTRACT(YEAR FROM "createdAt") AS year,
    EXTRACT(MONTH FROM "createdAt") AS month,
    "isReverse",
    COUNT(*) FILTER (
        WHERE status IN (?)
    ) / CAST(COUNT(*) AS REAL) AS "failureRate"
FROM data
WHERE
    CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END
    AND ((
        EXTRACT(YEAR FROM "createdAt") >= ? AND
        EXTRACT(MONTH FROM "createdAt") >= ?
    ) OR EXTRACT(YEAR FROM "createdAt") > ?)
GROUP BY year, month, "isReverse"
ORDER BY year, month, "isReverse";
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT pair, false AS isReverse, status, referral, createdAt FROM swaps
    UNION ALL
    SELECT pair, true as isReverse, status, referral, createdAt FROM reverseSwaps
)
SELECT
    CAST(STRFTIME('%Y', createdAt) AS INT) AS year,
    CAST(STRFTIME('%m', createdAt) AS INT) AS month,
    pair,
    isReverse,
    COUNT(*) FILTER (
        WHERE status IN (?)
    ) / CAST(COUNT(*) AS REAL) AS failureRate
FROM data
WHERE
    CASE WHEN ? IS NOT NULL THEN referral = ? ELSE TRUE END AND
    ((year >= ? AND month >= ?) OR year > ?)
GROUP BY year, month, isReverse
ORDER BY year, month, isReverse;
`,
};
StatsRepository.querySwapCounts = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type,
        CASE
            WHEN status = ? THEN 'success'
            WHEN status = ? THEN 'failure'
            ELSE 'timeout'
        END AS status
    FROM swaps
    UNION ALL
    SELECT
        pair,
        'reverse' AS type,
        CASE
            WHEN status = ? THEN 'success'
            WHEN status IN (?) THEN 'failure'
            ELSE 'timeout'
        END AS status
    FROM "reverseSwaps"
)
SELECT
    pair,
    type,
    status,
    COUNT(*) AS count
FROM data
GROUP BY pair, type, status;
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type,
        CASE
            WHEN status == ? THEN 'success'
            WHEN status == ? THEN 'failure'
            ELSE 'timeout'
        END AS status
    FROM swaps
    UNION ALL
    SELECT
        pair,
        'reverse' AS type,
        CASE
            WHEN status == ? THEN 'success'
            WHEN status IN (?) THEN 'failure'
            ELSE 'timeout'
        END AS status
    FROM reverseSwaps
)
SELECT
    pair,
    type,
    status,
    COUNT(*) AS count
FROM data
GROUP BY pair, type, status;
`,
};
StatsRepository.queryVolumePerPairType = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type,
        CASE WHEN "orderSide" = 1
            THEN "invoiceAmount"
            ELSE "onchainAmount"
        END AS amount
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT
        pair,
        'reverse' AS type,
        CASE WHEN "orderSide" = 1
            THEN "onchainAmount"
            ELSE "invoiceAmount"
        END AS amount
    FROM "reverseSwaps"
    WHERE status = ?
)
SELECT pair, type, SUM(amount)::BIGINT AS volume
FROM data
GROUP BY pair, type;
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type,
        CASE WHEN orderSide THEN invoiceAmount ELSE onchainAmount END AS amount
    FROM swaps
    WHERE status = ?
    UNION ALL
    SELECT
        pair,
        'reverse' AS type,
        CASE WHEN orderSide THEN onchainAmount ELSE invoiceAmount END AS amount
    FROM reverseSwaps
    WHERE status = ?
)
SELECT pair, type, SUM(amount) AS volume
FROM data
GROUP BY pair, type;
`,
};
StatsRepository.queryPendingSwapsCounts = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type
    FROM swaps
    WHERE status NOT IN (?)
    UNION ALL
    SELECT
        pair,
        'reverse' AS type
    FROM "reverseSwaps"
    WHERE status NOT IN (?)
)
SELECT
    pair,
    type,
    COUNT(*) as count
FROM data
GROUP BY pair, type;
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
WITH data AS (
    SELECT
        pair,
        'swap' AS type
    FROM swaps
    WHERE status NOT IN (?)
    UNION ALL
    SELECT
        pair,
        'reverse' AS type
    FROM reverseSwaps
    WHERE status NOT IN (?)
)
SELECT
    pair,
    type,
    COUNT(*) as count
FROM data
GROUP BY pair, type;
`,
};
StatsRepository.queryLockedFunds = {
    // language=PostgreSQL
    [Database_1.DatabaseType.PostgreSQL]: `
SELECT
    pair,
    SUM(
        CASE WHEN "orderSide" = 1
            THEN "onchainAmount"
            ELSE "invoiceAmount"
        END
    )::BIGINT AS locked
FROM "reverseSwaps"
WHERE status IN (?)
GROUP BY pair;
`,
    // language=SQLite
    [Database_1.DatabaseType.SQLite]: `
SELECT
    pair,
    SUM(amount) AS locked
FROM (
    SELECT
        pair,
        CASE WHEN orderSide THEN onchainAmount ELSE invoiceAmount END AS amount
    FROM reverseSwaps
    WHERE status IN (?)
)
GROUP BY pair;
`,
};
StatsRepository.getVolume = (minYear = 0, minMonth = 0, referral = null) => {
    return StatsRepository.query({
        query: StatsRepository.queryVolume[Database_1.default.type],
        values: [
            Enums_1.SwapUpdateEvent.TransactionClaimed,
            Enums_1.SwapUpdateEvent.InvoiceSettled,
            referral,
            referral,
            minYear,
            minMonth,
            minYear,
        ],
    });
};
StatsRepository.getTradeCounts = (minYear = 0, minMonth = 0, referral = null) => {
    return StatsRepository.query({
        query: StatsRepository.queryTradeCounts[Database_1.default.type],
        values: [
            Enums_1.SwapUpdateEvent.TransactionClaimed,
            Enums_1.SwapUpdateEvent.InvoiceSettled,
            referral,
            referral,
            minYear,
            minMonth,
            minYear,
        ],
    });
};
StatsRepository.getFailureRates = (minYear = 0, minMonth = 0, referral = null) => {
    return StatsRepository.query({
        query: StatsRepository.queryFailureRates[Database_1.default.type],
        values: [
            [
                Enums_1.SwapUpdateEvent.TransactionFailed,
                Enums_1.SwapUpdateEvent.InvoiceFailedToPay,
                Enums_1.SwapUpdateEvent.TransactionRefunded,
            ],
            referral,
            referral,
            minYear,
            minMonth,
            minYear,
        ],
    });
};
StatsRepository.getSwapCounts = () => {
    return StatsRepository.query({
        query: StatsRepository.querySwapCounts[Database_1.default.type],
        values: [
            Enums_1.SwapUpdateEvent.TransactionClaimed,
            Enums_1.SwapUpdateEvent.InvoiceFailedToPay,
            Enums_1.SwapUpdateEvent.InvoiceSettled,
            [
                Enums_1.SwapUpdateEvent.TransactionFailed,
                Enums_1.SwapUpdateEvent.TransactionRefunded,
            ],
        ],
    });
};
StatsRepository.getVolumePerPairType = () => {
    return StatsRepository.query({
        query: StatsRepository.queryVolumePerPairType[Database_1.default.type],
        values: [
            Enums_1.SwapUpdateEvent.TransactionClaimed,
            Enums_1.SwapUpdateEvent.InvoiceSettled,
        ],
    });
};
StatsRepository.getPendingSwapsCounts = () => {
    return StatsRepository.query({
        query: StatsRepository.queryPendingSwapsCounts[Database_1.default.type],
        values: [Enums_1.NotPendingSwapEvents, Enums_1.NotPendingReverseSwapEvents],
    });
};
StatsRepository.getLockedFunds = () => {
    return StatsRepository.query({
        query: StatsRepository.queryLockedFunds[Database_1.default.type],
        values: [
            [
                Enums_1.SwapUpdateEvent.TransactionMempool,
                Enums_1.SwapUpdateEvent.TransactionConfirmed,
            ],
        ],
    });
};
StatsRepository.query = (query) => {
    return Database_1.default.sequelize.query(query, {
        type: sequelize_1.QueryTypes.SELECT,
    });
};
exports.default = StatsRepository;
//# sourceMappingURL=StatsRepository.js.map