"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sequelize_1 = require("sequelize");
const Enums_1 = require("../../consts/Enums");
const Swap_1 = __importDefault(require("../models/Swap"));
class SwapRepository {
}
SwapRepository.getSwaps = (options) => {
    return Swap_1.default.findAll({
        where: options,
    });
};
SwapRepository.getSwapsExpirable = (height) => {
    return Swap_1.default.findAll({
        where: {
            status: {
                [sequelize_1.Op.notIn]: [
                    Enums_1.SwapUpdateEvent.SwapExpired,
                    Enums_1.SwapUpdateEvent.InvoiceFailedToPay,
                    Enums_1.SwapUpdateEvent.TransactionClaimed,
                ],
            },
            timeoutBlockHeight: {
                [sequelize_1.Op.lte]: height,
            },
        },
    });
};
SwapRepository.getSwapsClaimable = () => {
    return Swap_1.default.findAll({
        where: {
            status: Enums_1.SwapUpdateEvent.TransactionClaimPending,
        },
    });
};
SwapRepository.getSwap = (options) => {
    return Swap_1.default.findOne({
        where: options,
    });
};
SwapRepository.addSwap = (swap) => {
    return Swap_1.default.create(swap);
};
SwapRepository.setSwapStatus = (swap, status, failureReason) => {
    if (swap.failureReason) {
        return swap.update({
            status,
        });
    }
    return swap.update({
        status,
        failureReason,
    });
};
SwapRepository.setInvoice = (swap, invoice, invoiceAmount, expectedAmount, fee, acceptZeroConf) => {
    return swap.update({
        fee,
        invoice,
        invoiceAmount,
        acceptZeroConf,
        expectedAmount,
        status: Enums_1.SwapUpdateEvent.InvoiceSet,
    });
};
SwapRepository.setLockupTransaction = (swap, lockupTransactionId, onchainAmount, confirmed, lockupTransactionVout) => {
    return swap.update({
        onchainAmount,
        lockupTransactionId,
        lockupTransactionVout,
        status: confirmed
            ? Enums_1.SwapUpdateEvent.TransactionConfirmed
            : Enums_1.SwapUpdateEvent.TransactionMempool,
    });
};
SwapRepository.setRate = (swap, rate) => {
    return swap.update({
        rate,
    });
};
SwapRepository.setInvoicePaid = (swap, routingFee) => {
    return swap.update({
        routingFee,
        status: Enums_1.SwapUpdateEvent.InvoicePaid,
    });
};
SwapRepository.setMinerFee = (swap, minerFee) => {
    return swap.update({
        minerFee,
        status: Enums_1.SwapUpdateEvent.TransactionClaimed,
    });
};
SwapRepository.dropTable = () => {
    return Swap_1.default.drop();
};
exports.default = SwapRepository;
//# sourceMappingURL=SwapRepository.js.map