"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_js_1 = require("@grpc/grpc-js");
const assert_1 = __importDefault(require("assert"));
const boltzrpc_grpc_pb_1 = require("../proto/boltzrpc_grpc_pb");
const Errors_1 = __importDefault(require("./Errors"));
class GrpcServer {
    constructor(logger, grpcConfig, grpcService) {
        this.logger = logger;
        this.grpcConfig = grpcConfig;
        this.listen = () => {
            const { port, host } = this.grpcConfig;
            (0, assert_1.default)(Number.isInteger(port) && port > 1023 && port < 65536, 'port must be an integer between 1024 and 65536');
            return new Promise((resolve, reject) => {
                this.server.bindAsync(`${host}:${port}`, grpc_js_1.ServerCredentials.createInsecure(), (error, bindPort) => {
                    if (error) {
                        reject(Errors_1.default.COULD_NOT_BIND(host, port, error.message));
                    }
                    else {
                        this.logger.info(`gRPC server listening on: ${host}:${bindPort}`);
                        resolve();
                    }
                });
            });
        };
        this.close = () => {
            return new Promise((resolve) => {
                this.server.tryShutdown(() => {
                    this.logger.info('gRPC server completed shutdown');
                    resolve();
                });
            });
        };
        this.server = new grpc_js_1.Server();
        this.server.addService(boltzrpc_grpc_pb_1.BoltzService, {
            getInfo: grpcService.getInfo,
            getBalance: grpcService.getBalance,
            deriveKeys: grpcService.deriveKeys,
            deriveBlindingKeys: grpcService.deriveBlindingKeys,
            unblindOutputs: grpcService.unblindOutputs,
            getAddress: grpcService.getAddress,
            sendCoins: grpcService.sendCoins,
            updateTimeoutBlockDelta: grpcService.updateTimeoutBlockDelta,
            addReferral: grpcService.addReferral,
            sweepSwaps: grpcService.sweepSwaps,
            rescan: grpcService.rescan,
            setSwapStatus: grpcService.setSwapStatus,
            devHeapDump: grpcService.devHeapDump,
            getLockedFunds: grpcService.getLockedFunds,
            getPendingSweeps: grpcService.getPendingSweeps,
        });
    }
}
exports.default = GrpcServer;
//# sourceMappingURL=GrpcServer.js.map