"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Core_1 = require("../Core");
const HeapDump_1 = require("../HeapDump");
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const boltzrpc = __importStar(require("../proto/boltzrpc_pb"));
class GrpcService {
    constructor(logger, service) {
        this.logger = logger;
        this.service = service;
        this.getInfo = async (call, callback) => {
            await this.handleCallback(call, callback, () => this.service.getInfo());
        };
        this.getBalance = async (call, callback) => {
            await this.handleCallback(call, callback, () => this.service.getBalance());
        };
        this.deriveKeys = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { symbol, index } = call.request.toObject();
                return this.service.deriveKeys(symbol, index);
            });
        };
        this.deriveBlindingKeys = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { address } = call.request.toObject();
                const { publicKey, privateKey } = this.service.elementsService.deriveBlindingKeys(address);
                const res = new boltzrpc.DeriveBlindingKeyResponse();
                res.setPublicKey((0, Utils_1.getHexString)(publicKey));
                res.setPrivateKey((0, Utils_1.getHexString)(privateKey));
                return res;
            });
        };
        this.unblindOutputs = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const outputs = call.request.hasId() && call.request.getId() !== ''
                    ? await this.service.elementsService.unblindOutputsFromId(call.request.getId())
                    : await this.service.elementsService.unblindOutputs((0, Core_1.parseTransaction)(Enums_1.CurrencyType.Liquid, call.request.getHex()));
                const res = new boltzrpc.UnblindOutputsResponse();
                res.setOutputsList(outputs.map((out) => {
                    const rpcOut = new boltzrpc.UnblindOutputsResponse.UnblindedOutput();
                    rpcOut.setValue(out.value);
                    rpcOut.setAsset(out.asset);
                    rpcOut.setIsLbtc(out.isLbtc);
                    rpcOut.setScript(out.script);
                    rpcOut.setNonce(out.nonce);
                    if (out.rangeProof) {
                        rpcOut.setRangeProof(out.rangeProof);
                    }
                    if (out.surjectionProof) {
                        rpcOut.setSurjectionProof(out.surjectionProof);
                    }
                    return rpcOut;
                }));
                return res;
            });
        };
        this.getAddress = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { symbol } = call.request.toObject();
                const address = await this.service.getAddress(symbol);
                const response = new boltzrpc.GetAddressResponse();
                response.setAddress(address);
                return response;
            });
        };
        this.sendCoins = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { vout, transactionId } = await this.service.sendCoins(call.request.toObject());
                const response = new boltzrpc.SendCoinsResponse();
                response.setVout(vout);
                response.setTransactionId(transactionId);
                return response;
            });
        };
        this.updateTimeoutBlockDelta = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { pair, swapTaproot, reverseTimeout, swapMinimalTimeout, swapMaximalTimeout, } = call.request.toObject();
                this.service.updateTimeoutBlockDelta(pair, {
                    swapTaproot,
                    reverse: reverseTimeout,
                    swapMinimal: swapMinimalTimeout,
                    swapMaximal: swapMaximalTimeout,
                });
                return new boltzrpc.UpdateTimeoutBlockDeltaResponse();
            });
        };
        this.addReferral = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { id, feeShare, routingNode } = call.request.toObject();
                const { apiKey, apiSecret } = await this.service.addReferral({
                    id,
                    feeShare,
                    routingNode: routingNode === '' ? undefined : routingNode,
                });
                const response = new boltzrpc.AddReferralResponse();
                response.setApiKey(apiKey);
                response.setApiSecret(apiSecret);
                return response;
            });
        };
        this.setSwapStatus = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { id, status } = call.request.toObject();
                await this.service.setSwapStatus(id, status);
                return new boltzrpc.SetSwapStatusResponse();
            });
        };
        this.getLockedFunds = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const response = new boltzrpc.GetLockedFundsResponse();
                const lockedFunds = await this.service.getLockedFunds();
                const lockedFundsGrpcMap = response.getLockedFundsMap();
                lockedFunds.forEach((swaps, currency) => {
                    const lockedFundsList = new boltzrpc.LockedFunds();
                    swaps
                        .map((swap) => {
                        const lockedFund = new boltzrpc.LockedFund();
                        lockedFund.setSwapId(swap.id);
                        lockedFund.setOnchainAmount(swap.onchainAmount);
                        return lockedFund;
                    })
                        .forEach((lockedFund) => lockedFundsList.addLockedFunds(lockedFund));
                    lockedFundsGrpcMap.set(currency, lockedFundsList);
                });
                return response;
            });
        };
        this.getPendingSweeps = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const response = new boltzrpc.GetPendingSweepsResponse();
                const pendingSweeps = this.service.getPendingSweeps();
                const pendingSweepsGrpcMap = response.getPendingSweepsMap();
                pendingSweeps.forEach((swapToClaim, currency) => {
                    const pendingSweepsList = new boltzrpc.PendingSweeps();
                    swapToClaim
                        .map((toClaim) => {
                        const pendingSweep = new boltzrpc.PendingSweep();
                        pendingSweep.setSwapId(toClaim.swap.id);
                        pendingSweep.setOnchainAmount(toClaim.swap.onchainAmount || 0);
                        return pendingSweep;
                    })
                        .forEach((pendingSweep) => pendingSweepsList.addPendingSweeps(pendingSweep));
                    pendingSweepsGrpcMap.set(currency, pendingSweepsList);
                });
                return response;
            });
        };
        this.sweepSwaps = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { symbol } = call.request.toObject();
                const claimed = symbol
                    ? new Map([
                        [
                            symbol,
                            await this.service.swapManager.deferredClaimer.sweepSymbol(symbol),
                        ],
                    ])
                    : await this.service.swapManager.deferredClaimer.sweep();
                const response = new boltzrpc.SweepSwapsResponse();
                const grpcMap = response.getClaimedSymbolsMap();
                for (const [symbol, swapIds] of claimed) {
                    const ids = new boltzrpc.SweepSwapsResponse.ClaimedSwaps();
                    ids.setClaimedIdsList(swapIds);
                    grpcMap.set(symbol, ids);
                }
                return response;
            });
        };
        this.rescan = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const { symbol, startHeight } = call.request.toObject();
                const endHeight = await this.service.rescan(symbol, startHeight);
                const response = new boltzrpc.RescanResponse();
                response.setStartHeight(startHeight);
                response.setEndHeight(endHeight);
                return response;
            });
        };
        this.devHeapDump = async (call, callback) => {
            await this.handleCallback(call, callback, async () => {
                const filePath = call.request.getPath() || `${(0, Utils_1.getUnixTime)()}.heapsnapshot`;
                this.logger.verbose(`Dumping heap at: ${filePath}`);
                await (0, HeapDump_1.dumpHeap)(filePath);
                return new boltzrpc.DevHeapDumpResponse();
            });
        };
        this.handleCallback = async (call, callback, handler) => {
            try {
                callback(null, await handler(call));
            }
            catch (error) {
                callback(typeof error === 'string' ? { message: error } : error, null);
            }
        };
    }
}
exports.default = GrpcService;
//# sourceMappingURL=GrpcService.js.map