"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.satToMsat = exports.msatToSat = exports.transactionToLndScid = exports.scidClnToLnd = exports.scidLndToCln = void 0;
const msatFactor = 1000;
const scidLndToCln = (s) => {
    const big = BigInt(s);
    const block = big >> BigInt(40);
    const tx = (big >> BigInt(16)) & BigInt(0xffffff);
    const output = big & BigInt(0xffff);
    return [block, tx, output].join('x');
};
exports.scidLndToCln = scidLndToCln;
const scidClnToLnd = (s) => {
    const parts = s.split('x').map((part) => BigInt(part));
    return ((parts[0] << BigInt(40)) |
        (parts[1] << BigInt(16)) |
        parts[2]).toString();
};
exports.scidClnToLnd = scidClnToLnd;
const transactionToLndScid = (blockHeight, transactionIndex, outputIndex) => {
    const scid = (BigInt(blockHeight) << 40n) |
        (BigInt(transactionIndex) << 16n) |
        BigInt(outputIndex);
    return scid.toString();
};
exports.transactionToLndScid = transactionToLndScid;
const msatToSat = (msat) => {
    return Math.round(msat / msatFactor);
};
exports.msatToSat = msatToSat;
const satToMsat = (sat) => {
    return Math.round(sat * msatFactor);
};
exports.satToMsat = satToMsat;
//# sourceMappingURL=ChannelUtils.js.map