"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
class ConnectionHelper {
    constructor(logger) {
        this.logger = logger;
        this.connectByPublicKey = async (lndClient, publicKey) => {
            this.logger.verbose(`Trying to connect to LND ${lndClient.symbol} node: ${publicKey}`);
            // Fetch the publicly advertised addresses of the other LND node
            const nodeInfo = await lndClient.getNodeInfo(publicKey);
            if (nodeInfo.node === undefined ||
                nodeInfo.node.addressesList.length === 0) {
                throw 'node does not advertise addresses';
            }
            let addressesMessage = `Found ${nodeInfo.node.addressesList.length} public addresses of ${publicKey}:`;
            nodeInfo.node.addressesList.forEach((address) => {
                addressesMessage += `\n - ${address.addr}`;
            });
            this.logger.debug(addressesMessage);
            for (const address of nodeInfo.node.addressesList) {
                this.logger.debug(`Trying to connect to LND ${lndClient.symbol} addresses: ${publicKey}@${address.addr}`);
                try {
                    await lndClient.connectPeer(publicKey, address.addr);
                }
                catch (error) {
                    this.logger.debug(`Could not connect to to LND ${lndClient.symbol} address ${publicKey}@${address.addr}: ${(0, Utils_1.formatError)(error)}`);
                    // Try the next URI if that one did not work
                    continue;
                }
                this.logger.verbose(`Connected to LND ${lndClient.symbol} node: ${publicKey}`);
                return;
            }
            throw 'could not connect to any of the advertised addresses';
        };
    }
}
exports.default = ConnectionHelper;
//# sourceMappingURL=ConnectionHelper.js.map