"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_js_1 = require("@grpc/grpc-js");
const fs_1 = __importDefault(require("fs"));
const BaseClient_1 = __importDefault(require("../BaseClient"));
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
const invoices_grpc_pb_1 = require("../proto/lnd/invoices_grpc_pb");
const invoicesrpc = __importStar(require("../proto/lnd/invoices_pb"));
const router_grpc_pb_1 = require("../proto/lnd/router_grpc_pb");
const routerrpc = __importStar(require("../proto/lnd/router_pb"));
const rpc_grpc_pb_1 = require("../proto/lnd/rpc_grpc_pb");
const lndrpc = __importStar(require("../proto/lnd/rpc_pb"));
const Errors_1 = __importDefault(require("./Errors"));
const GrpcUtils_1 = require("./GrpcUtils");
const LightningClient_1 = require("./LightningClient");
/**
 * A class representing a client to interact with LND
 */
class LndClient extends BaseClient_1.default {
    /**
     * Create an LND client
     */
    constructor(logger, symbol, config) {
        super(logger, symbol);
        this.symbol = symbol;
        this.throwFilesNotFound = () => {
            throw Errors_1.default.COULD_NOT_FIND_FILES(this.symbol, LndClient.serviceName);
        };
        this.serviceName = () => {
            return LndClient.serviceName;
        };
        /**
         * Returns a boolean determines whether LND is ready or not
         */
        this.connect = async (startSubscriptions = true) => {
            if (!this.isConnected()) {
                this.router = new router_grpc_pb_1.RouterClient(this.uri, this.credentials, GrpcUtils_1.grpcOptions);
                this.invoices = new invoices_grpc_pb_1.InvoicesClient(this.uri, this.credentials, GrpcUtils_1.grpcOptions);
                this.lightning = new rpc_grpc_pb_1.LightningClient(this.uri, this.credentials, GrpcUtils_1.grpcOptions);
                try {
                    await this.getInfo();
                    if (startSubscriptions) {
                        this.subscribePeerEvents();
                        this.subscribeChannelEvents();
                        this.subscribeChannelBackups();
                    }
                    this.clearReconnectTimer();
                    this.setClientStatus(Enums_1.ClientStatus.Connected);
                }
                catch (error) {
                    this.setClientStatus(Enums_1.ClientStatus.Disconnected);
                    this.logger.error(`Could not connect to ${LndClient.serviceName} ${this.symbol} at ${this.uri}: ${(0, Utils_1.formatError)(error)}`);
                    this.logger.info(`Retrying in ${this.RECONNECT_INTERVAL} ms`);
                    this.reconnectionTimer = setTimeout(this.connect, this.RECONNECT_INTERVAL);
                    return false;
                }
            }
            return true;
        };
        this.reconnect = async () => {
            this.setClientStatus(Enums_1.ClientStatus.Disconnected);
            try {
                await this.getInfo();
                this.logger.info(`Reestablished connection to ${LndClient.serviceName} ${this.symbol}`);
                this.clearReconnectTimer();
                this.subscribePeerEvents();
                this.subscribeChannelEvents();
                this.subscribeChannelBackups();
                this.setClientStatus(Enums_1.ClientStatus.Connected);
                this.emit('subscription.reconnected', null);
            }
            catch (err) {
                this.setClientStatus(Enums_1.ClientStatus.Disconnected);
                this.logger.error(`Could not reconnect to ${LndClient.serviceName} ${this.symbol}: ${err}`);
                this.logger.info(`Retrying in ${this.RECONNECT_INTERVAL} ms`);
                this.reconnectionTimer = setTimeout(this.reconnect, this.RECONNECT_INTERVAL);
            }
        };
        /**
         * End all subscriptions and reconnection attempts.
         */
        this.disconnect = () => {
            this.clearReconnectTimer();
            if (this.peerEventSubscription) {
                this.peerEventSubscription.cancel();
                this.peerEventSubscription = undefined;
            }
            if (this.channelEventSubscription) {
                this.channelEventSubscription.cancel();
                this.channelEventSubscription = undefined;
            }
            if (this.channelBackupSubscription) {
                this.channelBackupSubscription.cancel();
                this.channelBackupSubscription = undefined;
            }
            if (this.lightning) {
                this.lightning.close();
                this.lightning = undefined;
            }
            if (this.router) {
                this.router.close();
                this.router = undefined;
            }
            if (this.invoices) {
                this.invoices.close();
                this.invoices = undefined;
            }
            this.removeAllListeners();
            this.setClientStatus(Enums_1.ClientStatus.Disconnected);
        };
        this.unaryInvoicesCall = (methodName, params) => {
            return (0, GrpcUtils_1.unaryCall)(this.invoices, methodName, params, this.meta, true);
        };
        this.unaryRouterCall = (methodName, params) => {
            return (0, GrpcUtils_1.unaryCall)(this.router, methodName, params, this.meta, true);
        };
        this.unaryLightningCall = (methodName, params, toObject = true) => {
            return (0, GrpcUtils_1.unaryCall)(this.lightning, methodName, params, this.meta, toObject);
        };
        this.getInfo = async () => {
            const info = await this.unaryLightningCall('getInfo', new lndrpc.GetInfoRequest());
            return {
                version: info.version,
                pubkey: info.identityPubkey,
                uris: info.urisList,
                peers: info.numPeers,
                blockHeight: info.blockHeight,
                channels: {
                    active: info.numActiveChannels,
                    inactive: info.numInactiveChannels,
                    pending: info.numPendingChannels,
                },
            };
        };
        /**
         * Creates an invoice
         */
        this.addInvoice = (value, expiry, memo, routingHints) => {
            const request = new lndrpc.Invoice();
            request.setValue(value);
            if (expiry) {
                request.setExpiry(expiry);
            }
            if (memo) {
                request.setMemo(memo);
            }
            if (routingHints) {
                request.setRouteHintsList(LndClient.routingHintsToGrpc(routingHints));
            }
            return this.unaryLightningCall('addInvoice', request);
        };
        /**
         * Creates a hold invoice with the supplied preimage hash
         */
        this.addHoldInvoice = async (value, preimageHash, cltvExpiry, expiry, memo, routingHints) => {
            const request = new invoicesrpc.AddHoldInvoiceRequest();
            request.setValue(value);
            request.setHash(Uint8Array.from(preimageHash));
            if (cltvExpiry) {
                request.setCltvExpiry(cltvExpiry);
            }
            if (expiry) {
                request.setExpiry(expiry);
            }
            if (memo) {
                request.setMemo(memo);
            }
            if (routingHints) {
                request.setRouteHintsList(LndClient.routingHintsToGrpc(routingHints));
            }
            return (await this.unaryInvoicesCall('addHoldInvoice', request)).paymentRequest;
        };
        this.lookupHoldInvoice = async (preimageHash) => {
            const res = await this.lookupInvoice(preimageHash);
            return {
                state: LndClient.invoiceStateFromGrpc(res.state),
                htlcs: res.htlcsList.map(LndClient.htlcFromGrpc),
            };
        };
        this.lookupInvoice = (preimageHash) => {
            const request = new lndrpc.PaymentHash();
            request.setRHash(preimageHash);
            return this.unaryLightningCall('lookupInvoice', request);
        };
        this.trackPayment = (preimageHash) => {
            return new Promise((resolve, reject) => {
                const request = new routerrpc.TrackPaymentRequest();
                request.setNoInflightUpdates(false);
                request.setPaymentHash(preimageHash);
                const stream = this.router.trackPaymentV2(request, this.meta);
                const endStream = () => {
                    stream.removeAllListeners();
                    stream.destroy();
                };
                stream.on('data', (response) => {
                    endStream();
                    resolve(response.toObject());
                });
                stream.on('end', () => {
                    endStream();
                });
                stream.on('error', (error) => {
                    endStream();
                    reject(error);
                });
            });
        };
        /**
         * Pay an invoice through the Lightning Network.
         *
         * @param invoice an invoice for a payment within the Lightning Network
         * @param cltvDelta CLTV delta limit for the payment
         * @param outgoingChannelId channel through which the invoice should be paid
         */
        this.sendPayment = (invoice, cltvDelta, outgoingChannelId) => {
            return new Promise((resolve, reject) => {
                const request = new routerrpc.SendPaymentRequest();
                request.setMaxParts(LndClient.paymentMaxParts);
                request.setTimeoutSeconds(LndClient.paymentTimeout);
                request.setTimePref(LndClient.paymentTimePreference);
                request.setFeeLimitSat((0, LightningClient_1.calculatePaymentFee)(invoice, this.maxPaymentFeeRatio, LndClient.paymentMinFee));
                request.setPaymentRequest(invoice);
                if (cltvDelta) {
                    request.setCltvLimit(cltvDelta);
                }
                if (outgoingChannelId) {
                    request.setOutgoingChanId(outgoingChannelId);
                }
                const stream = this.router.sendPaymentV2(request, this.meta);
                stream.on('data', (response) => {
                    switch (response.getStatus()) {
                        case lndrpc.Payment.PaymentStatus.SUCCEEDED:
                            stream.removeAllListeners();
                            resolve({
                                feeMsat: response.getFeeMsat(),
                                preimage: (0, Utils_1.getHexBuffer)(response.getPaymentPreimage()),
                            });
                            return;
                        case lndrpc.Payment.PaymentStatus.FAILED:
                            stream.removeAllListeners();
                            reject(response.getFailureReason());
                            break;
                    }
                });
                stream.on('end', () => {
                    stream.removeAllListeners();
                });
                stream.on('error', (error) => {
                    stream.removeAllListeners();
                    reject(error);
                });
            });
        };
        this.resetMissionControl = () => {
            return this.unaryRouterCall('resetMissionControl', new routerrpc.ResetMissionControlRequest());
        };
        /**
         * Cancel a hold invoice
         */
        this.cancelHoldInvoice = async (preimageHash) => {
            const request = new invoicesrpc.CancelInvoiceMsg();
            request.setPaymentHash(Uint8Array.from(preimageHash));
            await this.unaryInvoicesCall('cancelInvoice', request);
        };
        /**
         * Settle a hold invoice with an already accepted HTLC
         */
        this.settleHoldInvoice = async (preimage) => {
            const request = new invoicesrpc.SettleInvoiceMsg();
            request.setPreimage(Uint8Array.from(preimage));
            await this.unaryInvoicesCall('settleInvoice', request);
        };
        /**
         * Queries for a possible route to the target destination
         */
        this.queryRoutes = async (destination, amt, cltvLimit, finalCltvDelta, routingHints) => {
            const request = new lndrpc.QueryRoutesRequest();
            request.setAmt(amt);
            request.setPubKey(destination);
            request.setUseMissionControl(true);
            request.setTimePref(LndClient.paymentTimePreference);
            if (cltvLimit) {
                request.setCltvLimit(cltvLimit);
            }
            if (finalCltvDelta) {
                request.setFinalCltvDelta(finalCltvDelta);
            }
            if (routingHints) {
                request.setRouteHintsList(LndClient.routingHintsToGrpc(routingHints));
            }
            const res = await this.unaryLightningCall('queryRoutes', request);
            return res.routesList.map((route) => ({
                ctlv: route.totalTimeLock,
                feesMsat: route.totalFeesMsat,
            }));
        };
        this.decodeInvoice = async (invoice) => {
            const res = await this.decodePayReq(invoice);
            const features = new Set();
            for (const [, feature] of res.featuresMap) {
                switch (feature.name) {
                    case 'amp':
                        features.add(LightningClient_1.InvoiceFeature.AMP);
                        break;
                    case 'multi-path-payments':
                        features.add(LightningClient_1.InvoiceFeature.MPP);
                        break;
                }
            }
            return {
                features,
                cltvExpiry: res.cltvExpiry,
                destination: res.destination,
                value: (0, Utils_1.decodeInvoiceAmount)(invoice),
                routingHints: LndClient.routingHintsFromGrpc(res.routeHintsList),
            };
        };
        /**
         * Decode an encoded payment request
         *
         * @param paymentRequest encoded payment request
         */
        this.decodePayReq = (paymentRequest) => {
            const request = new lndrpc.PayReqString();
            request.setPayReq(paymentRequest);
            return this.unaryLightningCall('decodePayReq', request);
        };
        this.decodePayReqRawResponse = (paymentRequest) => {
            const request = new lndrpc.PayReqString();
            request.setPayReq(paymentRequest);
            return this.unaryLightningCall('decodePayReq', request, false);
        };
        /**
         * Returns the latest advertised, aggregated, and authenticated channel information
         * for the specified node identified by its public key
         */
        this.getNodeInfo = (publicKey) => {
            const request = new lndrpc.NodeInfoRequest();
            request.setPubKey(publicKey);
            request.setIncludeChannels(false);
            return this.unaryLightningCall('getNodeInfo', request);
        };
        /**
         * Establish a connection to a remote peer
         *
         * @param pubKey identity public key of the remote peer
         * @param host host of the remote peer
         */
        this.connectPeer = (pubKey, host) => {
            const address = new lndrpc.LightningAddress();
            address.setPubkey(pubKey);
            address.setHost(host);
            const request = new lndrpc.ConnectPeerRequest();
            request.setAddr(address);
            return this.unaryLightningCall('connectPeer', request);
        };
        /**
         * Sends coins to a particular address
         *
         * @param address address to which coins should be sent
         * @param amount number of satoshis or litoshis to send
         * @param satPerByte satoshis or litoshis per byte that should be sent as fee
         */
        this.sendCoins = (address, amount, satPerByte) => {
            const request = new lndrpc.SendCoinsRequest();
            request.setAddr(address);
            request.setAmount(amount);
            if (satPerByte) {
                request.setSatPerByte(satPerByte);
            }
            return this.unaryLightningCall('sendCoins', request);
        };
        /**
         * Sends all coins of the wallet to a particular address
         *
         * @param address address to which coins should be sent
         * @param satPerByte satoshis or litoshis per byte that should be sent as fee
         */
        this.sweepWallet = (address, satPerByte) => {
            const request = new lndrpc.SendCoinsRequest();
            request.setAddr(address);
            request.setSendAll(true);
            if (satPerByte) {
                request.setSatPerByte(satPerByte);
            }
            return this.unaryLightningCall('sendCoins', request);
        };
        /**
         * Returns a list describing all the known transactions relevant to the wallet
         */
        this.getOnchainTransactions = (startHeight) => {
            const request = new lndrpc.GetTransactionsRequest();
            request.setStartHeight(startHeight);
            return this.unaryLightningCall('getTransactions', request);
        };
        /**
         * Creates a new address
         *
         * @param addressType type of the address
         */
        this.newAddress = (addressType = lndrpc.AddressType.NESTED_PUBKEY_HASH) => {
            const request = new lndrpc.NewAddressRequest();
            request.setType(addressType);
            return this.unaryLightningCall('newAddress', request);
        };
        /**
         * Attempts to open a channel to a remote peer
         *
         * @param pubKey identity public key of the remote peer
         * @param fundingAmount the number of satoshis the local wallet should commit
         * @param privateChannel whether the channel should be private
         * @param satPerByte sat/vbyte fee of the funding transaction
         */
        this.openChannel = (pubKey, fundingAmount, privateChannel, satPerByte) => {
            const request = new lndrpc.OpenChannelRequest();
            request.setPrivate(privateChannel);
            request.setNodePubkeyString(pubKey);
            request.setLocalFundingAmount(fundingAmount);
            if (satPerByte) {
                request.setSatPerByte(satPerByte);
            }
            return this.unaryLightningCall('openChannelSync', request);
        };
        /**
         * Gets a list of all open channels
         */
        this.listChannels = async (activeOnly = false, privateOnly = false) => {
            const request = new lndrpc.ListChannelsRequest();
            request.setActiveOnly(activeOnly);
            request.setPrivateOnly(privateOnly);
            const res = await this.unaryLightningCall('listChannels', request);
            return res.channelsList.map((chan) => {
                const { id, vout } = (0, Utils_1.splitChannelPoint)(chan.channelPoint);
                return {
                    chanId: chan.chanId,
                    capacity: chan.capacity,
                    private: chan.pb_private,
                    fundingTransactionId: id,
                    fundingTransactionVout: vout,
                    remotePubkey: chan.remotePubkey,
                    localBalance: chan.localBalance,
                    remoteBalance: chan.remoteBalance,
                };
            });
        };
        /**
         * Gets the latest routing information of a given channel
         */
        this.getChannelInfo = (channelId) => {
            const request = new lndrpc.ChanInfoRequest();
            request.setChanId(channelId);
            return this.unaryLightningCall('getChanInfo', request);
        };
        /**
         * Gets a list of all peers
         */
        this.listPeers = () => {
            return this.unaryLightningCall('listPeers', new lndrpc.ListPeersRequest());
        };
        this.getBalance = async () => {
            const res = await this.getWalletBalance();
            return {
                confirmedBalance: res.confirmedBalance,
                unconfirmedBalance: res.unconfirmedBalance,
            };
        };
        /**
         * Gets the balance of the onchain wallet
         */
        this.getWalletBalance = () => {
            const request = new lndrpc.WalletBalanceRequest();
            return this.unaryLightningCall('walletBalance', request);
        };
        /**
         * Subscribe to events for a single invoice
         */
        this.subscribeSingleInvoice = (preimageHash) => {
            const request = new invoicesrpc.SubscribeSingleInvoiceRequest();
            request.setRHash(Uint8Array.from(preimageHash));
            const invoiceSubscription = this.invoices.subscribeSingleInvoice(request, this.meta);
            const deleteSubscription = () => {
                invoiceSubscription.removeAllListeners();
                invoiceSubscription.destroy();
            };
            invoiceSubscription
                .on('data', (invoice) => {
                if (invoice.getState() === lndrpc.Invoice.InvoiceState.ACCEPTED) {
                    const acceptedHtlcs = invoice
                        .getHtlcsList()
                        .filter((htlc) => htlc.getState() === lndrpc.InvoiceHTLCState.ACCEPTED);
                    this.logger.debug(`${LndClient.serviceName} ${this.symbol} accepted ${acceptedHtlcs.length} HTLC${acceptedHtlcs.length > 1 ? 's' : ''} for invoice: ${invoice.getPaymentRequest()}`);
                    this.emit('htlc.accepted', invoice.getPaymentRequest());
                    deleteSubscription();
                }
                else if (invoice.getState() === lndrpc.Invoice.InvoiceState.SETTLED) {
                    this.logger.debug(`${LndClient.serviceName} ${this.symbol} invoice settled: ${invoice.getPaymentRequest()}`);
                    this.emit('invoice.settled', invoice.getPaymentRequest());
                    deleteSubscription();
                }
            })
                .on('end', () => deleteSubscription())
                .on('error', (error) => {
                this.logger.error(`${LndClient.serviceName} ${this.symbol} invoice subscription errored: ${error.message}`);
                deleteSubscription();
            });
        };
        this.handleSubscriptionError = async (subscriptionName, error) => {
            this.logger.error(`${LndClient.serviceName} ${this.symbol} ${subscriptionName} subscription errored: ${(0, Utils_1.formatError)(error)}`);
            if (this.isConnected()) {
                this.emit('subscription.error', subscriptionName);
                await this.reconnect();
            }
        };
        this.subscribePeerEvents = () => {
            if (this.peerEventSubscription) {
                this.peerEventSubscription.cancel();
            }
            this.peerEventSubscription = this.lightning.subscribePeerEvents(new lndrpc.PeerEventSubscription(), this.meta)
                .on('data', (event) => {
                if (event.getType() === lndrpc.PeerEvent.EventType.PEER_ONLINE) {
                    this.emit('peer.online', event.getPubKey());
                }
            })
                .on('error', async (error) => {
                await this.handleSubscriptionError('peer event', error);
            });
        };
        this.subscribeChannelEvents = () => {
            if (this.channelEventSubscription) {
                this.channelEventSubscription.cancel();
            }
            this.channelEventSubscription = this.lightning.subscribeChannelEvents(new lndrpc.ChannelEventSubscription(), this.meta)
                .on('data', (event) => {
                if (event.getType() ===
                    lndrpc.ChannelEventUpdate.UpdateType.ACTIVE_CHANNEL) {
                    this.emit('channel.active', event.getActiveChannel().toObject());
                }
            })
                .on('error', async (error) => {
                await this.handleSubscriptionError('channel event', error);
            });
        };
        this.subscribeChannelBackups = () => {
            if (this.channelBackupSubscription) {
                this.channelBackupSubscription.cancel();
            }
            this.channelBackupSubscription = this.lightning.subscribeChannelBackups(new lndrpc.ChannelBackupSubscription(), this.meta)
                .on('data', (backupSnapshot) => {
                const multiBackup = backupSnapshot.getMultiChanBackup();
                if (multiBackup) {
                    const decodedBackup = Buffer.from(multiBackup.getMultiChanBackup_asB64(), 'base64');
                    this.emit('channel.backup', (0, Utils_1.getHexString)(decodedBackup));
                }
            })
                .on('error', async (error) => {
                this.logger.error(`${LndClient.serviceName} ${this.symbol} channel backup subscription errored: ${(0, Utils_1.formatError)(error)}`);
                this.emit('subscription.error', 'channel backup');
            });
        };
        const { host, port, certpath, macaroonpath, maxPaymentFeeRatio } = config;
        this.maxPaymentFeeRatio =
            maxPaymentFeeRatio > 0 ? maxPaymentFeeRatio : 0.01;
        if (fs_1.default.existsSync(certpath)) {
            this.uri = `${host}:${port}`;
            const lndCert = fs_1.default.readFileSync(certpath);
            this.credentials = grpc_js_1.credentials.createSsl(lndCert);
            this.meta = new grpc_js_1.Metadata();
            if (macaroonpath !== '') {
                if (fs_1.default.existsSync(macaroonpath)) {
                    const adminMacaroon = fs_1.default.readFileSync(macaroonpath);
                    this.meta.add('macaroon', adminMacaroon.toString('hex'));
                }
                else {
                    this.throwFilesNotFound();
                }
            }
        }
        else {
            this.throwFilesNotFound();
        }
    }
}
LndClient.serviceName = 'LND';
LndClient.paymentMinFee = 121;
LndClient.paymentMaxParts = 5;
LndClient.paymentTimeout = 300;
LndClient.paymentTimePreference = 0.9;
LndClient.formatPaymentFailureReason = (reason) => {
    switch (reason) {
        case lndrpc.PaymentFailureReason.FAILURE_REASON_TIMEOUT:
            return 'timeout';
        case lndrpc.PaymentFailureReason.FAILURE_REASON_NO_ROUTE:
            return 'no route';
        case lndrpc.PaymentFailureReason.FAILURE_REASON_INSUFFICIENT_BALANCE:
            return 'insufficient balance';
        case lndrpc.PaymentFailureReason.FAILURE_REASON_INCORRECT_PAYMENT_DETAILS:
            return 'incorrect payment details';
        default:
            return `unknown reason: ${reason.toString()}`;
    }
};
LndClient.routingHintsToGrpc = (routingHints) => {
    return routingHints.map((hints) => {
        const routeHint = new lndrpc.RouteHint();
        for (const hint of hints) {
            const hopHint = new lndrpc.HopHint();
            hopHint.setNodeId(hint.nodeId);
            hopHint.setChanId(hint.chanId);
            hopHint.setFeeBaseMsat(hint.feeBaseMsat);
            hopHint.setFeeProportionalMillionths(hint.feeProportionalMillionths);
            hopHint.setCltvExpiryDelta(hint.cltvExpiryDelta);
            routeHint.addHopHints(hopHint);
        }
        return routeHint;
    });
};
LndClient.routingHintsFromGrpc = (hints) => {
    return hints.map((hint) => hint.hopHintsList.map((hop) => hop));
};
LndClient.invoiceStateFromGrpc = (state) => {
    switch (state) {
        case lndrpc.Invoice.InvoiceState.OPEN:
            return LightningClient_1.InvoiceState.Open;
        case lndrpc.Invoice.InvoiceState.ACCEPTED:
            return LightningClient_1.InvoiceState.Accepted;
        case lndrpc.Invoice.InvoiceState.CANCELED:
            return LightningClient_1.InvoiceState.Cancelled;
        case lndrpc.Invoice.InvoiceState.SETTLED:
            return LightningClient_1.InvoiceState.Settled;
    }
};
LndClient.htlcFromGrpc = (htlc) => {
    return {
        valueMsat: htlc.amtMsat,
        state: LndClient.htlcStateFromGrpc(htlc.state),
    };
};
LndClient.htlcStateFromGrpc = (state) => {
    switch (state) {
        case lndrpc.InvoiceHTLCState.ACCEPTED:
            return LightningClient_1.HtlcState.Accepted;
        case lndrpc.InvoiceHTLCState.CANCELED:
            return LightningClient_1.HtlcState.Cancelled;
        case lndrpc.InvoiceHTLCState.SETTLED:
            return LightningClient_1.HtlcState.Settled;
    }
};
exports.default = LndClient;
//# sourceMappingURL=LndClient.js.map