/// <reference types="node" />
import BaseClient from '../../BaseClient';
import Logger from '../../Logger';
import * as noderpc from '../../proto/cln/node_pb';
import * as holdrpc from '../../proto/hold/hold_pb';
import { WalletBalance } from '../../wallet/providers/WalletProviderInterface';
import { ChannelInfo, DecodedInvoice, EventTypes, HopHint, Invoice, LightningClient, NodeInfo, PaymentResponse, Route, RoutingHintsProvider } from '../LightningClient';
import Mpay from './MpayClient';
import { ClnConfig } from './Types';
declare class ClnClient extends BaseClient<EventTypes> implements LightningClient, RoutingHintsProvider {
    readonly symbol: string;
    static readonly serviceName = "CLN";
    static readonly serviceNameHold = "hold";
    static readonly moddedVersionSuffix = "-modded";
    readonly mpay?: Mpay;
    private static readonly paymentMinFee;
    private static readonly paymentTimeout;
    private readonly maxPaymentFeeRatio;
    private readonly nodeUri;
    private readonly holdUri;
    private nodeClient?;
    private readonly nodeCreds;
    private readonly nodeMeta;
    private holdClient?;
    private readonly holdCreds;
    private readonly holdMeta;
    private trackAllSubscription?;
    constructor(logger: Logger, symbol: string, config: ClnConfig);
    static isRpcError: (error: any) => boolean;
    static formatPaymentFailureReason: (error: {
        details: string;
    }) => string;
    static errIsIncorrectPaymentDetails: (err: string) => boolean;
    serviceName: () => string;
    useMpay: () => boolean;
    connect: (startSubscriptions?: boolean) => Promise<boolean>;
    disconnect: () => void;
    private reconnect;
    getInfo: () => Promise<NodeInfo>;
    getBalance: () => Promise<WalletBalance>;
    listFunds: () => Promise<noderpc.ListfundsResponse.AsObject>;
    getHoldInfo: () => Promise<holdrpc.GetInfoResponse.AsObject>;
    listChannels: (activeOnly?: boolean, privateOnly?: boolean) => Promise<ChannelInfo[]>;
    stop: () => void;
    routingHints: (node: string) => Promise<HopHint[][]>;
    addHoldInvoice: (value: number, preimageHash: Buffer, cltvExpiry?: number, expiry?: number, memo?: string, routingHints?: HopHint[][]) => Promise<string>;
    lookupHoldInvoice: (preimageHash: Buffer) => Promise<Invoice>;
    cancelHoldInvoice: (preimageHash: Buffer) => Promise<void>;
    settleHoldInvoice: (preimage: Buffer) => Promise<void>;
    decodeInvoice: (invoice: string) => Promise<DecodedInvoice>;
    queryRoutes: (destination: string, amt: number, cltvLimit?: number, finalCltvDelta?: number, routingHints?: HopHint[][]) => Promise<Route[]>;
    sendPayment: (invoice: string, cltvDelta?: number, _?: string) => Promise<PaymentResponse>;
    subscribeSingleInvoice: (_: Buffer) => void;
    private static routingHintsToGrpc;
    private static routingHintsFromGrpc;
    private static invoiceStateFromGrpc;
    private static htlcFromGrpc;
    private static htlcStateFromGrpc;
    checkPayStatus: (invoice: string) => Promise<PaymentResponse | undefined>;
    private payStatus;
    private queryRoute;
    private subscribeTrackHoldInvoices;
    private unaryNodeCall;
    private unaryHoldCall;
    private handleSubscriptionError;
}
export default ClnClient;
export { ClnConfig };
