"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc_js_1 = require("@grpc/grpc-js");
const bolt11_1 = __importDefault(require("bolt11"));
const BaseClient_1 = __importDefault(require("../../BaseClient"));
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const node_grpc_pb_1 = require("../../proto/cln/node_grpc_pb");
const noderpc = __importStar(require("../../proto/cln/node_pb"));
const node_pb_1 = require("../../proto/cln/node_pb");
const primitivesrpc = __importStar(require("../../proto/cln/primitives_pb"));
const hold_grpc_pb_1 = require("../../proto/hold/hold_grpc_pb");
const holdrpc = __importStar(require("../../proto/hold/hold_pb"));
const ChannelUtils_1 = require("../ChannelUtils");
const Errors_1 = __importDefault(require("../Errors"));
const GrpcUtils_1 = require("../GrpcUtils");
const LightningClient_1 = require("../LightningClient");
const MpayClient_1 = __importDefault(require("./MpayClient"));
const Types_1 = require("./Types");
var ListpaysPaysStatus = node_pb_1.ListpaysPays.ListpaysPaysStatus;
class ClnClient extends BaseClient_1.default {
    constructor(logger, symbol, config) {
        super(logger, symbol);
        this.symbol = symbol;
        this.nodeMeta = new grpc_js_1.Metadata();
        this.holdMeta = new grpc_js_1.Metadata();
        this.serviceName = () => {
            return ClnClient.serviceName;
        };
        this.useMpay = () => this.mpay !== undefined && this.mpay.isConnected();
        this.connect = async (startSubscriptions = true) => {
            if (!this.isConnected()) {
                this.nodeClient = new node_grpc_pb_1.NodeClient(this.nodeUri, this.nodeCreds, {
                    ...GrpcUtils_1.grpcOptions,
                    'grpc.ssl_target_name_override': 'cln',
                });
                this.holdClient = new hold_grpc_pb_1.HoldClient(this.holdUri, this.holdCreds, {
                    ...GrpcUtils_1.grpcOptions,
                    'grpc.ssl_target_name_override': 'hold',
                });
                this.mpay?.connect();
                try {
                    await this.getInfo();
                    if (startSubscriptions) {
                        this.subscribeTrackHoldInvoices();
                    }
                    this.setClientStatus(Enums_1.ClientStatus.Connected);
                }
                catch (error) {
                    this.setClientStatus(Enums_1.ClientStatus.Disconnected);
                    this.logger.error(`Could not connect to ${ClnClient.serviceName} ${this.symbol}: ${(0, Utils_1.formatError)(error)}`);
                    this.logger.info(`Retrying in ${this.RECONNECT_INTERVAL} ms`);
                    this.reconnectionTimer = setTimeout(this.connect, this.RECONNECT_INTERVAL);
                    return false;
                }
            }
            return true;
        };
        this.disconnect = () => {
            this.clearReconnectTimer();
            if (this.trackAllSubscription) {
                this.trackAllSubscription.cancel();
                this.trackAllSubscription = undefined;
            }
            if (this.holdClient) {
                this.holdClient.close();
                this.holdClient = undefined;
            }
            this.mpay?.disconnect();
            this.removeAllListeners();
            this.setClientStatus(Enums_1.ClientStatus.Disconnected);
        };
        this.reconnect = async () => {
            this.setClientStatus(Enums_1.ClientStatus.Disconnected);
            try {
                await this.getInfo();
                this.logger.info(`Reestablished connection to ${ClnClient.serviceName} ${this.symbol}`);
                this.clearReconnectTimer();
                this.subscribeTrackHoldInvoices();
                this.setClientStatus(Enums_1.ClientStatus.Connected);
                this.emit('subscription.reconnected', null);
            }
            catch (err) {
                this.setClientStatus(Enums_1.ClientStatus.Disconnected);
                this.logger.error(`Could not reconnect to ${ClnClient.serviceName} ${this.symbol}: ${err}`);
                this.logger.info(`Retrying in ${this.RECONNECT_INTERVAL} ms`);
                this.reconnectionTimer = setTimeout(this.reconnect, this.RECONNECT_INTERVAL);
            }
        };
        this.getInfo = async () => {
            const info = await this.unaryNodeCall('getinfo', new noderpc.GetinfoRequest(), false);
            const pubkey = (0, Utils_1.getHexString)(Buffer.from(info.getId_asU8()));
            return {
                pubkey,
                version: info.getVersion(),
                uris: info
                    .getAddressList()
                    .filter((address) => address.hasAddress() && address.getAddress() !== undefined)
                    .map((address) => `${pubkey}@${address.getAddress()}:${address.getPort()}`),
                peers: info.getNumPeers(),
                blockHeight: info.getBlockheight(),
                channels: {
                    active: info.getNumActiveChannels(),
                    inactive: info.getNumInactiveChannels(),
                    pending: info.getNumPendingChannels(),
                },
            };
        };
        this.getBalance = async () => {
            const sumOutputs = (outs, status) => outs
                .filter((out) => !out.reserved)
                .filter((out) => out.status === status)
                .reduce((sum, out) => sum + (0, ChannelUtils_1.msatToSat)(out.amountMsat.msat), 0);
            const res = await this.listFunds();
            return {
                confirmedBalance: sumOutputs(res.outputsList, node_pb_1.ListfundsOutputs.ListfundsOutputsStatus.CONFIRMED),
                unconfirmedBalance: sumOutputs(res.outputsList, node_pb_1.ListfundsOutputs.ListfundsOutputsStatus.UNCONFIRMED),
            };
        };
        this.listFunds = () => {
            const req = new noderpc.ListfundsRequest();
            req.setSpent(false);
            return this.unaryNodeCall('listFunds', req);
        };
        this.getHoldInfo = () => {
            return this.unaryHoldCall('getInfo', new holdrpc.GetInfoRequest());
        };
        this.listChannels = async (activeOnly = false, privateOnly = false) => {
            const res = await this.unaryNodeCall('listPeerChannels', new noderpc.ListpeerchannelsRequest(), false);
            return res
                .getChannelsList()
                .filter((chan) => chan.getShortChannelId() !== undefined &&
                chan.getShortChannelId() !== '')
                .filter((chan) => !activeOnly ||
                (chan.getPeerConnected() &&
                    chan.getState() ===
                        noderpc.ListpeerchannelsChannels.ListpeerchannelsChannelsState
                            .CHANNELD_NORMAL))
                .filter((chan) => !privateOnly || chan.getPrivate())
                .map((chan) => {
                return {
                    private: chan.getPrivate(),
                    fundingTransactionId: (0, Utils_1.getHexString)(Buffer.from(chan.getFundingTxid_asU8())),
                    capacity: (0, ChannelUtils_1.msatToSat)(chan.getTotalMsat().getMsat()),
                    chanId: (0, ChannelUtils_1.scidClnToLnd)(chan.getShortChannelId()),
                    fundingTransactionVout: chan.getFundingOutnum(),
                    localBalance: (0, ChannelUtils_1.msatToSat)(chan.getSpendableMsat().getMsat()),
                    remoteBalance: (0, ChannelUtils_1.msatToSat)(chan.getReceivableMsat().getMsat()),
                    remotePubkey: (0, Utils_1.getHexString)(Buffer.from(chan.getPeerId_asU8())),
                };
            });
        };
        this.stop = () => {
            // Just here for interface compatibility;
        };
        this.routingHints = async (node) => {
            const req = new holdrpc.RoutingHintsRequest();
            req.setNode(node);
            const res = await this.unaryHoldCall('routingHints', req);
            return ClnClient.routingHintsFromGrpc(res.hintsList);
        };
        this.addHoldInvoice = async (value, preimageHash, cltvExpiry, expiry, memo, routingHints) => {
            const req = new holdrpc.InvoiceRequest();
            req.setAmountMsat((0, ChannelUtils_1.satToMsat)(value));
            req.setPaymentHash((0, Utils_1.getHexString)(preimageHash));
            if (cltvExpiry) {
                req.setMinFinalCltvExpiry(cltvExpiry);
            }
            if (expiry) {
                req.setExpiry(expiry);
            }
            if (memo) {
                req.setDescription(memo);
            }
            if (routingHints) {
                req.setRoutingHintsList(ClnClient.routingHintsToGrpc(routingHints));
            }
            return (await this.unaryHoldCall('invoice', req)).bolt11;
        };
        this.lookupHoldInvoice = async (preimageHash) => {
            const req = new holdrpc.ListRequest();
            req.setPaymentHash((0, Utils_1.getHexString)(preimageHash));
            const res = await this.unaryHoldCall('list', req);
            if (res.invoicesList.length === 0) {
                throw Errors_1.default.INVOICE_NOT_FOUND();
            }
            const invoice = res.invoicesList[0];
            return {
                state: ClnClient.invoiceStateFromGrpc(invoice.state),
                htlcs: invoice.htlcsList.map(ClnClient.htlcFromGrpc),
            };
        };
        this.cancelHoldInvoice = async (preimageHash) => {
            const req = new holdrpc.CancelRequest();
            req.setPaymentHash((0, Utils_1.getHexString)(preimageHash));
            await this.unaryHoldCall('cancel', req);
        };
        this.settleHoldInvoice = async (preimage) => {
            const req = new holdrpc.SettleRequest();
            req.setPaymentPreimage((0, Utils_1.getHexString)(preimage));
            await this.unaryHoldCall('settle', req);
        };
        this.decodeInvoice = async (invoice) => {
            // Just to make sure CLN can parse the invoice
            const req = new noderpc.DecodeRequest();
            req.setString(invoice);
            const clnRes = await this.unaryNodeCall('decode', req);
            const features = new Set();
            const dec = bolt11_1.default.decode(invoice);
            const featureBits = dec.tags.find((e) => e.tagName === 'feature_bits');
            if (featureBits && featureBits.data) {
                const mpp = featureBits.data['basic_mpp'];
                if (mpp.supported || mpp.required) {
                    features.add(LightningClient_1.InvoiceFeature.MPP);
                }
            }
            const routingHints = [];
            // TODO: routing hints with multiple hops?
            const routingHintsTag = dec.tags.find((e) => e.tagName === 'routing_info');
            if (routingHintsTag && routingHintsTag.data) {
                routingHintsTag.data.forEach((route) => {
                    const chanId = BigInt(`0x${route.short_channel_id}`).toString();
                    routingHints.push([
                        {
                            chanId,
                            nodeId: route.pubkey,
                            feeBaseMsat: route.fee_base_msat,
                            cltvExpiryDelta: route.cltv_expiry_delta,
                            feeProportionalMillionths: route.fee_proportional_millionths,
                        },
                    ]);
                });
            }
            return {
                features,
                routingHints,
                value: dec.satoshis || 0,
                destination: dec.payeeNodeKey,
                cltvExpiry: clnRes.minFinalCltvExpiry || 0,
            };
        };
        this.queryRoutes = async (destination, amt, cltvLimit, finalCltvDelta, routingHints) => {
            const prms = [
                this.queryRoute(destination, amt, cltvLimit, finalCltvDelta),
            ];
            if (routingHints) {
                routingHints.forEach((hint) => {
                    const hintCtlvs = hint.reduce((sum, hint) => sum + hint.cltvExpiryDelta, 0);
                    prms.push(this.queryRoute(hint[0].nodeId, amt, cltvLimit, (finalCltvDelta || 0) + hintCtlvs));
                });
            }
            const routes = (await Promise.allSettled(prms))
                .filter((res) => res.status === 'fulfilled')
                .map((res) => res.value);
            if (routes.length === 0) {
                throw Errors_1.default.NO_ROUTE();
            }
            return routes;
        };
        // TODO: support for setting outgoing channel id
        this.sendPayment = async (invoice, cltvDelta, 
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        _) => {
            const payStatus = await this.checkPayStatus(invoice);
            if (payStatus !== undefined) {
                return payStatus;
            }
            const maxFee = (0, ChannelUtils_1.satToMsat)((0, LightningClient_1.calculatePaymentFee)(invoice, this.maxPaymentFeeRatio, ClnClient.paymentMinFee));
            const useMpay = this.useMpay();
            this.logger.verbose(`Using ${useMpay ? 'mpay' : 'pay'} for ${ClnClient.serviceName} ${this.symbol} invoice payment`);
            if (useMpay) {
                return this.mpay.sendPayment(invoice, maxFee, ClnClient.paymentTimeout, cltvDelta);
            }
            const req = new noderpc.PayRequest();
            req.setBolt11(invoice);
            req.setRetryFor(ClnClient.paymentTimeout);
            const feeAmount = new primitivesrpc.Amount();
            feeAmount.setMsat(maxFee);
            req.setMaxfee(feeAmount);
            if (cltvDelta) {
                req.setMaxdelay(cltvDelta);
            }
            const res = await this.unaryNodeCall('pay', req, false);
            if (res.getStatus() !== noderpc.PayResponse.PayStatus.COMPLETE) {
                // TODO: error message?
                throw 'payment failed';
            }
            const fee = BigInt(res.getAmountSentMsat().getMsat()) -
                BigInt(res.getAmountMsat().getMsat());
            return {
                feeMsat: Number(fee),
                preimage: Buffer.from(res.getPaymentPreimage_asU8()),
            };
        };
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        this.subscribeSingleInvoice = (_) => {
            // Just here for interface compatibility;
            // with CLN we can subscribe to all hold invoice with one gRPC subscription
            return;
        };
        this.checkPayStatus = async (invoice) => {
            const listPayReq = new noderpc.ListpaysRequest();
            listPayReq.setBolt11(invoice);
            const pays = (await this.unaryNodeCall('listPays', listPayReq, false)).getPaysList();
            // Check if the payment succeeded...
            const completedAttempts = pays.filter((attempt) => attempt.getStatus() === ListpaysPaysStatus.COMPLETE);
            if (completedAttempts.length > 0) {
                const fee = completedAttempts.reduce((sum, attempt) => sum + BigInt(attempt.getAmountSentMsat()?.getMsat() || 0), 0n) -
                    completedAttempts.reduce((sum, attempt) => sum + BigInt(attempt.getAmountMsat()?.getMsat() || 0), 0n);
                return {
                    feeMsat: Number(fee),
                    preimage: Buffer.from(completedAttempts[0].getPreimage_asU8()),
                };
            }
            // ... has failed
            // TODO: mpay incorrect payment details detection
            for (const payStatus of (await this.payStatus(invoice)).statusList) {
                for (const attempt of payStatus.attemptsList) {
                    if (attempt.state ===
                        holdrpc.PayStatusResponse.PayStatus.Attempt.AttemptState
                            .ATTEMPT_PENDING ||
                        attempt.failure === undefined) {
                        continue;
                    }
                    if (ClnClient.errIsIncorrectPaymentDetails(attempt.failure.message)) {
                        throw attempt.failure.message;
                    }
                }
            }
            // ... or is still pending
            const hasPendingPayments = pays.some((pay) => pay.getStatus() === ListpaysPaysStatus.PENDING);
            if (hasPendingPayments) {
                const channels = await this.unaryNodeCall('listPeerChannels', new noderpc.ListpeerchannelsRequest(), false);
                const paymentHash = (0, Utils_1.getHexBuffer)((0, Utils_1.decodeInvoice)(invoice).paymentHash);
                const hasPendingHtlc = channels
                    .getChannelsList()
                    .some((channel) => channel
                    .getHtlcsList()
                    .some((htlc) => paymentHash.equals(Buffer.from(htlc.getPaymentHash_asU8()))));
                if (hasPendingHtlc) {
                    throw 'payment already pending';
                }
            }
            return undefined;
        };
        this.payStatus = (invoice) => {
            const req = new holdrpc.PayStatusRequest();
            req.setBolt11(invoice);
            return this.unaryHoldCall('payStatus', req);
        };
        this.queryRoute = async (destination, amt, cltvLimit, finalCltvDelta) => {
            const req = new holdrpc.GetRouteRequest();
            req.setDestination(destination);
            req.setRiskFactor(0);
            req.setAmountMsat(amt);
            if (cltvLimit) {
                req.setMaxCltv(cltvLimit);
            }
            if (finalCltvDelta) {
                req.setFinalCltvDelta(finalCltvDelta);
            }
            const res = await this.unaryHoldCall('getRoute', req);
            return {
                ctlv: res.hopsList[0].delay,
                feesMsat: res.feesMsat,
            };
        };
        this.subscribeTrackHoldInvoices = () => {
            if (this.trackAllSubscription) {
                this.trackAllSubscription.cancel();
            }
            this.trackAllSubscription = this.holdClient.trackAll(new holdrpc.TrackAllRequest());
            this.trackAllSubscription.on('data', (update) => {
                switch (update.getState()) {
                    case holdrpc.InvoiceState.INVOICE_ACCEPTED:
                        this.logger.debug(`${ClnClient.serviceName} ${this.symbol} accepted invoice: ${update.getBolt11()}`);
                        this.emit('htlc.accepted', update.getBolt11());
                        break;
                    case holdrpc.InvoiceState.INVOICE_PAID:
                        this.logger.debug(`${ClnClient.serviceName} ${this.symbol} invoice settled: ${update.getBolt11()}`);
                        this.emit('invoice.settled', update.getBolt11());
                        break;
                }
            });
            this.trackAllSubscription.on('error', async (error) => {
                await this.handleSubscriptionError('track hold invoices', error);
            });
        };
        this.unaryNodeCall = (methodName, params, toObject = true) => {
            return (0, GrpcUtils_1.unaryCall)(this.nodeClient, methodName, params, this.nodeMeta, toObject);
        };
        this.unaryHoldCall = (methodName, params) => {
            return (0, GrpcUtils_1.unaryCall)(this.holdClient, methodName, params, this.holdMeta, true);
        };
        this.handleSubscriptionError = async (subscriptionName, error) => {
            this.logger.error(`${ClnClient.serviceName} ${this.symbol} ${subscriptionName} subscription errored: ${(0, Utils_1.formatError)(error)}`);
            if (this.isConnected()) {
                this.emit('subscription.error', subscriptionName);
                setTimeout(() => this.reconnect(), this.RECONNECT_INTERVAL);
            }
        };
        this.maxPaymentFeeRatio =
            config.maxPaymentFeeRatio > 0 ? config.maxPaymentFeeRatio : 0.01;
        this.nodeCreds = (0, Types_1.createSsl)(ClnClient.serviceName, symbol, config);
        this.holdCreds = (0, Types_1.createSsl)(ClnClient.serviceNameHold, symbol, config.hold);
        this.nodeUri = `${config.host}:${config.port}`;
        this.holdUri = `${config.hold.host}:${config.hold.port}`;
        if (config.mpay) {
            this.logger.verbose(`Using mpay for ${ClnClient.serviceName} ${this.symbol}`);
            this.mpay = new MpayClient_1.default(logger, this.symbol, config.mpay);
        }
        else {
            this.logger.warn(`Mpay not configured for ${ClnClient.serviceName} ${this.symbol}; using pay`);
        }
    }
}
ClnClient.serviceName = 'CLN';
ClnClient.serviceNameHold = 'hold';
ClnClient.moddedVersionSuffix = '-modded';
ClnClient.paymentMinFee = 121;
ClnClient.paymentTimeout = 300;
ClnClient.isRpcError = (error) => {
    return (typeof error === 'object' &&
        typeof error.details === 'string' &&
        error.details !== '');
};
ClnClient.formatPaymentFailureReason = (error) => {
    const split = error.details.split('"');
    if (split.length !== 3) {
        return error.details;
    }
    return split[1];
};
ClnClient.errIsIncorrectPaymentDetails = (err) => {
    return err.includes('WIRE_INCORRECT_OR_UNKNOWN_PAYMENT_DETAILS');
};
ClnClient.routingHintsToGrpc = (routingHints) => {
    return routingHints.map((hints) => {
        const routeHint = new holdrpc.RoutingHint();
        for (const hint of hints) {
            const hopHint = new holdrpc.Hop();
            hopHint.setPublicKey(hint.nodeId);
            hopHint.setShortChannelId((0, ChannelUtils_1.scidLndToCln)(hint.chanId));
            hopHint.setBaseFee(hint.feeBaseMsat);
            hopHint.setPpmFee(hint.feeProportionalMillionths);
            hopHint.setCltvExpiryDelta(hint.cltvExpiryDelta);
            routeHint.addHops(hopHint);
        }
        return routeHint;
    });
};
ClnClient.routingHintsFromGrpc = (routingHints) => {
    return routingHints.map((hint) => hint.hopsList.map((hop) => ({
        nodeId: hop.publicKey,
        chanId: hop.shortChannelId,
        feeBaseMsat: hop.baseFee,
        feeProportionalMillionths: hop.ppmFee,
        cltvExpiryDelta: hop.cltvExpiryDelta,
    })));
};
ClnClient.invoiceStateFromGrpc = (state) => {
    switch (state) {
        case holdrpc.InvoiceState.INVOICE_UNPAID:
            return LightningClient_1.InvoiceState.Open;
        case holdrpc.InvoiceState.INVOICE_ACCEPTED:
            return LightningClient_1.InvoiceState.Accepted;
        case holdrpc.InvoiceState.INVOICE_CANCELLED:
            return LightningClient_1.InvoiceState.Cancelled;
        case holdrpc.InvoiceState.INVOICE_PAID:
            return LightningClient_1.InvoiceState.Settled;
    }
};
ClnClient.htlcFromGrpc = (htlc) => {
    return {
        state: ClnClient.htlcStateFromGrpc(htlc.state),
        valueMsat: htlc.msat,
    };
};
ClnClient.htlcStateFromGrpc = (state) => {
    switch (state) {
        case holdrpc.HtlcState.HTLC_ACCEPTED:
            return LightningClient_1.HtlcState.Accepted;
        case holdrpc.HtlcState.HTLC_CANCELLED:
            return LightningClient_1.HtlcState.Cancelled;
        case holdrpc.HtlcState.HTLC_SETTLED:
            return LightningClient_1.HtlcState.Settled;
    }
};
exports.default = ClnClient;
//# sourceMappingURL=ClnClient.js.map