"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const Markup_1 = require("./Markup");
class DiskUsageChecker {
    constructor(logger, notificationClient) {
        this.logger = logger;
        this.notificationClient = notificationClient;
        this.alertSent = false;
        this.checkUsage = async () => {
            const { available, total } = await this.getUsage();
            const used = total - available;
            const usedPercentage = used / total;
            if (usedPercentage >= DiskUsageChecker.warningThreshold) {
                if (!this.alertSent) {
                    const message = `${Markup_1.Emojis.RotatingLight} Disk usage is **${this.formatNumber(usedPercentage * 100)}%**: ` +
                        `**${this.formatNumber(this.convertToGb(available))} GB** of **${this.formatNumber(this.convertToGb(total))} GB** available ${Markup_1.Emojis.RotatingLight}`;
                    this.logger.warn(message);
                    await this.notificationClient.sendMessage(message);
                    this.alertSent = true;
                }
            }
            else {
                this.alertSent = false;
            }
        };
        this.getUsage = async () => {
            return new Promise((resolve, reject) => {
                (0, child_process_1.execFile)('df', ['-P', '-k', DiskUsageChecker.rootDir], (error, stdout) => {
                    if (error) {
                        reject(error);
                        return;
                    }
                    const lines = stdout.split('\n');
                    if (lines.length < 2) {
                        throw new Error(`unexpected df output: ${stdout}`);
                    }
                    const parts = lines[1].split(' ').filter((x) => {
                        return x !== '';
                    });
                    resolve({
                        total: Number(parts[1]),
                        available: Number(parts[3]),
                    });
                });
            });
        };
        this.formatNumber = (toFormat) => {
            return Number(toFormat.toFixed(2));
        };
        this.convertToGb = (bytes) => {
            return bytes / DiskUsageChecker.gigabyte;
        };
    }
}
DiskUsageChecker.rootDir = '/';
DiskUsageChecker.gigabyte = 1024 ** 2;
DiskUsageChecker.warningThreshold = 0.9;
exports.default = DiskUsageChecker;
//# sourceMappingURL=DiskUsageChecker.js.map