"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const otplib_1 = require("otplib");
const path_1 = require("path");
class OtpManager {
    constructor(logger, config) {
        this.logger = logger;
        // This variable keeps track of the last used token to prevent that the same token can be used multiple times
        this.lastUsedToken = '';
        this.verify = (token) => {
            try {
                if (token !== this.lastUsedToken) {
                    const valid = otplib_1.authenticator.check(token, this.secret);
                    if (valid) {
                        this.lastUsedToken = token;
                    }
                    return valid;
                }
            }
            catch (error) {
                this.logger.debug(`Could not check OTP token ${token}: ${error}`);
            }
            return false;
        };
        this.generateUri = (prefix, secretPath) => {
            const uri = otplib_1.authenticator.keyuri(encodeURIComponent(prefix), encodeURIComponent(OtpManager.service), this.secret);
            const path = (0, path_1.join)((0, path_1.dirname)(secretPath), OtpManager.uriFile);
            (0, fs_1.writeFileSync)(path, uri);
        };
        if ((0, fs_1.existsSync)(config.otpsecretpath)) {
            this.secret = (0, fs_1.readFileSync)(config.otpsecretpath, { encoding: 'utf8' });
            this.logger.info('Loaded existing OTP secret');
        }
        else {
            this.secret = otplib_1.authenticator.generateSecret();
            (0, fs_1.writeFileSync)(config.otpsecretpath, this.secret);
            this.logger.warn('Generated new OTP secret');
            this.generateUri(config.prefix, config.otpsecretpath);
        }
    }
}
OtpManager.service = 'Boltz';
OtpManager.uriFile = 'otpUri.txt';
exports.default = OtpManager;
//# sourceMappingURL=OtpManager.js.map