"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@mattermost/client");
const ws_1 = __importDefault(require("ws"));
const Markup_1 = require("../Markup");
const NotificationClient_1 = __importDefault(require("./NotificationClient"));
class MattermostClient extends NotificationClient_1.default {
    constructor(logger, config) {
        super(MattermostClient.serviceName, logger, config);
        this.userId = undefined;
        this.channelId = undefined;
        this.channelAlertsId = undefined;
        this.init = async () => {
            const [user, channels] = await Promise.all([
                this.client.getMe(),
                this.client.getAllTeamsChannels(),
            ]);
            this.userId = user.id;
            this.channelId = channels.find((chan) => chan.name === this.config.channel)?.id;
            if (this.channelId === undefined) {
                throw `Could not find Mattermost channel: ${this.config.channel}`;
            }
            if (this.config.channelAlerts) {
                this.channelAlertsId = channels.find((chan) => chan.name === this.config.channelAlerts)?.id;
                if (this.channelAlertsId === undefined) {
                    this.logger.warn(`Could not find Mattermost notification channel: ${this.config.channelAlerts}`);
                }
            }
            this.wsClient.initialize(this.client.getWebSocketUrl(), this.config.token);
            this.wsClient.addMessageListener(this.listenForMessages);
            this.wsClient.addErrorListener((error) => {
                this.emit('error', error.type);
            });
        };
        this.destroy = () => {
            this.wsClient.close();
        };
        this.sendMessage = async (message, isAlert) => {
            const channel = this.selectChannel(this.channelId, this.channelAlertsId, isAlert);
            if (channel === undefined) {
                return;
            }
            if (message.includes(Markup_1.codeBlock)) {
                message = message.replace(/```/, '```json\n');
                if (!message.startsWith('\n')) {
                    message = `\n${message}`;
                }
                message =
                    message.substring(0, message.length - Markup_1.codeBlock.length) + '\n```';
            }
            await this.client.createPost({
                channel_id: channel,
                message: `${this.prefix}${message}`,
            });
        };
        this.listenForMessages = (msg) => {
            if (msg.event !== 'posted' || msg.broadcast.channel_id !== this.channelId) {
                return;
            }
            const post = JSON.parse(msg.data.post);
            // Ignore own messages
            if (post.user_id === this.userId) {
                return;
            }
            this.emit('message', post.message);
        };
        // Polyfill a WebSocket library
        Object.assign(global, { WebSocket: ws_1.default });
        this.client = new client_1.Client4();
        if (config.mattermostUrl === undefined) {
            throw 'missing Mattermost URL';
        }
        this.client.setUrl(config.mattermostUrl);
        if (config.token === '') {
            throw 'missing Mattermost token';
        }
        this.client.setToken(config.token);
        this.wsClient = new client_1.WebSocketClient();
    }
}
MattermostClient.serviceName = 'Mattermost';
exports.default = MattermostClient;
//# sourceMappingURL=MattermostClient.js.map