import Logger from '../Logger';
import { BaseFeeType, CurrencyType, OrderSide, SwapVersion } from '../consts/Enums';
import { PairConfig } from '../consts/Types';
import WalletManager from '../wallet/WalletManager';
import DataAggregator from './data/DataAggregator';
type TransactionSizesForVersion = {
    normalClaim: number;
    reverseLockup: number;
    reverseClaim: number;
};
type TransactionSizes = {
    [SwapVersion.Legacy]: TransactionSizesForVersion;
    [SwapVersion.Taproot]: TransactionSizesForVersion;
};
type PercentageFees = {
    percentage: number;
    percentageSwapIn: number;
};
type ReverseMinerFees = {
    lockup: number;
    claim: number;
};
type MinerFeesForVersion = {
    normal: number;
    reverse: ReverseMinerFees;
};
type MinerFees = {
    [SwapVersion.Legacy]: MinerFeesForVersion;
    [SwapVersion.Taproot]: MinerFeesForVersion;
};
declare class FeeProvider {
    private logger;
    private walletManager;
    private dataAggregator;
    private getFeeEstimation;
    percentageFees: Map<string, number>;
    percentageSwapInFees: Map<string, number>;
    minerFees: Map<string, MinerFees>;
    static transactionSizes: {
        [CurrencyType.BitcoinLike]: TransactionSizes;
        [CurrencyType.Liquid]: TransactionSizes;
    };
    static gasUsage: {
        EtherSwap: {
            lockup: number;
            claim: number;
            refund: number;
        };
        ERC20Swap: {
            lockup: number;
            claim: number;
            refund: number;
        };
    };
    private static readonly defaultFee;
    constructor(logger: Logger, walletManager: WalletManager, dataAggregator: DataAggregator, getFeeEstimation: (symbol: string) => Promise<Map<string, number>>);
    init: (pairs: PairConfig[]) => void;
    getPercentageFees: (pairId: string) => PercentageFees;
    getPercentageFee: (pair: string, isReverse: boolean) => number;
    getFees: (pair: string, swapVersion: SwapVersion, rate: number, orderSide: OrderSide, amount: number, type: BaseFeeType) => {
        baseFee: number;
        percentageFee: number;
    };
    getBaseFee: (chainCurrency: string, swapVersion: SwapVersion, type: BaseFeeType) => number;
    updateMinerFees: (chainCurrency: string) => Promise<void>;
    private static calculateTokenGasCosts;
    private static calculateEtherGasCost;
}
export default FeeProvider;
export { ReverseMinerFees, MinerFees, PercentageFees, MinerFeesForVersion };
