"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Errors_1 = __importDefault(require("./Errors"));
class RateCalculator {
    constructor(aggregator) {
        this.aggregator = aggregator;
        this.calculateRate = (base, quote) => {
            const pair = (0, Utils_1.getPairId)({
                base,
                quote,
            });
            let rate = this.aggregator.latestRates.get(pair);
            if (rate === undefined) {
                // Try the reverse rate
                rate = this.aggregator.latestRates.get((0, Utils_1.getPairId)({
                    base: quote,
                    quote: base,
                }));
                if (rate !== undefined) {
                    rate = 1 / rate;
                }
                else {
                    // TODO: also try routing via USD(T)
                    // Get the route via BTC
                    const baseBtc = this.aggregator.latestRates.get((0, Utils_1.getPairId)({
                        base,
                        quote: 'BTC',
                    }));
                    if (baseBtc === undefined) {
                        throw Errors_1.default.COULD_NOT_FIND_RATE(pair);
                    }
                    const quoteBtc = this.aggregator.latestRates.get((0, Utils_1.getPairId)({
                        base: quote,
                        quote: 'BTC',
                    }));
                    if (quoteBtc === undefined) {
                        throw Errors_1.default.COULD_NOT_FIND_RATE(pair);
                    }
                    return baseBtc / quoteBtc;
                }
            }
            return rate;
        };
    }
}
exports.default = RateCalculator;
//# sourceMappingURL=RateCalculator.js.map