import Logger from '../Logger';
import { SwapVersion } from '../consts/Enums';
import { PairConfig } from '../consts/Types';
import WalletManager, { Currency } from '../wallet/WalletManager';
import FeeProvider from './FeeProvider';
import RateCalculator from './RateCalculator';
import DataAggregator from './data/DataAggregator';
import RateProviderLegacy from './providers/RateProviderLegacy';
import RateProviderTaproot from './providers/RateProviderTaproot';
declare class RateProvider {
    private readonly logger;
    private readonly rateUpdateInterval;
    private readonly currencies;
    private readonly walletManager;
    readonly feeProvider: FeeProvider;
    readonly providers: {
        [SwapVersion.Taproot]: RateProviderTaproot;
        [SwapVersion.Legacy]: RateProviderLegacy;
    };
    readonly dataAggregator: DataAggregator;
    readonly rateCalculator: RateCalculator;
    private readonly configPairs;
    private readonly hardcodedPairs;
    private readonly zeroConfAmounts;
    private readonly pairConfigs;
    private timer;
    constructor(logger: Logger, rateUpdateInterval: number, currencies: Map<string, Currency>, walletManager: WalletManager, getFeeEstimation: (symbol: string) => Promise<Map<string, number>>);
    init: (pairs: PairConfig[]) => Promise<void>;
    disconnect: () => void;
    /**
     * Returns whether 0-conf should be accepted for a specific amount on a specified chain
     */
    acceptZeroConf: (chainCurrency: string, amount: number) => boolean;
    has: (pairId: string) => boolean;
    setZeroConfAmount: (symbol: string, amount: number) => Promise<void>;
    private updateRates;
    private parseCurrencies;
    private updateMinerFees;
    private forProviders;
}
export default RateProvider;
