"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const NodeSwitch_1 = __importDefault(require("../../swap/NodeSwitch"));
class RateProviderBase {
    constructor(currencies, feeProvider) {
        this.currencies = currencies;
        this.feeProvider = feeProvider;
        this.adjustMinimaForFees = (base, quote, rate, configuredMinima, orderSide, isReverse) => {
            const minima = [configuredMinima];
            const pairsToCheck = orderSide === undefined
                ? [
                    [base, quote],
                    [quote, base],
                ]
                : [
                    [
                        (0, Utils_1.getChainCurrency)(base, quote, orderSide, isReverse),
                        (0, Utils_1.getLightningCurrency)(base, quote, orderSide, isReverse),
                    ],
                ];
            pairsToCheck
                .filter(([chain, lightning]) => this.isPossibleChainCurrency(chain, lightning))
                .map(([currency]) => currency)
                .forEach((currency) => {
                let limit = this.feeProvider.getBaseFee(currency, Enums_1.SwapVersion.Legacy, Enums_1.BaseFeeType.NormalClaim) * RateProviderBase.minLimitFactor;
                if (currency === base) {
                    limit *= rate;
                }
                minima.push(limit);
            });
            return Math.ceil(Math.max(...minima));
        };
        this.isPossibleChainCurrency = (chainCurrency, lightningCurrency) => {
            return (this.currencies.get(chainCurrency).chainClient !== undefined &&
                NodeSwitch_1.default.hasClient(this.currencies.get(lightningCurrency)));
        };
    }
}
RateProviderBase.minLimitFactor = 2;
exports.default = RateProviderBase;
//# sourceMappingURL=RateProviderBase.js.map