"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
const Errors_1 = __importDefault(require("../../service/Errors"));
const RateProviderBase_1 = __importDefault(require("./RateProviderBase"));
const emptyMinerFees = {
    normal: 0,
    reverse: {
        claim: 0,
        lockup: 0,
    },
};
class RateProviderLegacy extends RateProviderBase_1.default {
    constructor(currencies, feeProvider, pairConfigs, zeroConfAmounts) {
        super(currencies, feeProvider);
        this.pairConfigs = pairConfigs;
        this.zeroConfAmounts = zeroConfAmounts;
        this.pairs = new Map();
        this.setHardcodedPair = (pair) => {
            const id = (0, Utils_1.getPairId)(pair);
            const hardcodedPair = {
                hash: '',
                rate: pair.rate,
                limits: this.getLimits(id, pair.rate),
                fees: {
                    ...this.feeProvider.getPercentageFees(id),
                    minerFees: {
                        baseAsset: emptyMinerFees,
                        quoteAsset: emptyMinerFees,
                    },
                },
            };
            hardcodedPair.hash = this.hashPair(hardcodedPair);
            this.pairs.set(id, hardcodedPair);
        };
        this.updatePair = (pairId, rate) => {
            const { base, quote } = (0, Utils_1.splitPairId)(pairId);
            const pair = {
                rate,
                hash: '',
                limits: this.getLimits(pairId, rate),
                fees: {
                    ...this.feeProvider.getPercentageFees(pairId),
                    minerFees: {
                        baseAsset: this.feeProvider.minerFees.get(base)[Enums_1.SwapVersion.Legacy],
                        quoteAsset: this.feeProvider.minerFees.get(quote)[Enums_1.SwapVersion.Legacy],
                    },
                },
            };
            pair.hash = this.hashPair(pair);
            this.pairs.set(pairId, pair);
        };
        this.updateHardcodedPair = (pairId) => {
            const { base, quote } = (0, Utils_1.splitPairId)(pairId);
            const pairInfo = this.pairs.get(pairId);
            pairInfo.fees.minerFees = {
                baseAsset: this.feeProvider.minerFees.get(base)[Enums_1.SwapVersion.Legacy],
                quoteAsset: this.feeProvider.minerFees.get(quote)[Enums_1.SwapVersion.Legacy],
            };
            pairInfo.limits = this.getLimits(pairId, pairInfo.rate);
            pairInfo.hash = this.hashPair(pairInfo);
            this.pairs.set(pairId, pairInfo);
        };
        this.validatePairHash = (hash, pairId) => {
            const pair = this.pairs.get(pairId);
            if (pair === undefined || hash !== pair.hash) {
                throw Errors_1.default.INVALID_PAIR_HASH();
            }
        };
        this.hashPair = (pair) => (0, Utils_1.hashString)(JSON.stringify({
            rate: pair.rate,
            fees: pair.fees,
            limits: pair.limits,
        }));
        this.getLimits = (pair, rate) => {
            const config = this.pairConfigs.get(pair);
            if (config === undefined) {
                throw `Could not get limits for pair: ${pair}`;
            }
            const { base, quote } = (0, Utils_1.splitPairId)(pair);
            return {
                maximal: config.maxSwapAmount,
                minimal: this.adjustMinimaForFees(base, quote, rate, config.minSwapAmount),
                maximalZeroConf: {
                    baseAsset: this.zeroConfAmounts.get(base),
                    quoteAsset: this.zeroConfAmounts.get(quote),
                },
            };
        };
    }
}
exports.default = RateProviderLegacy;
//# sourceMappingURL=RateProviderLegacy.js.map