import { OrderSide } from '../../consts/Enums';
import { PairConfig } from '../../consts/Types';
import { Currency } from '../../wallet/WalletManager';
import FeeProvider, { ReverseMinerFees } from '../FeeProvider';
import RateProviderBase from './RateProviderBase';
type PairLimits = {
    minimal: number;
    maximal: number;
};
type PairTypeTaproot = {
    hash: string;
    rate: number;
};
type SubmarinePairTypeTaproot = PairTypeTaproot & {
    limits: PairLimits & {
        maximalZeroConf: number;
    };
    fees: {
        percentage: number;
        minerFees: number;
    };
};
type ReversePairTypeTaproot = PairTypeTaproot & {
    limits: PairLimits;
    fees: {
        percentage: number;
        minerFees: ReverseMinerFees;
    };
};
type SwapTypes = SubmarinePairTypeTaproot | ReversePairTypeTaproot;
declare class RateProviderTaproot extends RateProviderBase<SwapTypes> {
    private readonly pairConfigs;
    private readonly zeroConfAmounts;
    readonly submarinePairs: Map<string, Map<string, SubmarinePairTypeTaproot>>;
    readonly reversePairs: Map<string, Map<string, ReversePairTypeTaproot>>;
    constructor(currencies: Map<string, Currency>, feeProvider: FeeProvider, pairConfigs: Map<string, PairConfig>, zeroConfAmounts: Map<string, number>);
    static serializePairs: <T>(map: Map<string, Map<string, T>>) => Record<string, Record<string, T>>;
    setHardcodedPair: (pair: PairConfig) => void;
    updatePair: (pairId: string, rawRate: number) => void;
    updateHardcodedPair: (pairId: string) => void;
    validatePairHash: (hash: string, pairId: string, orderSide: OrderSide, isReverse: boolean) => void;
    private getToMap;
    protected hashPair: (pair: SwapTypes) => string;
    private setPair;
    private getLimits;
    private isPossibleCombination;
    private canLightning;
    private canOnchain;
}
export default RateProviderTaproot;
export { SubmarinePairTypeTaproot, ReversePairTypeTaproot };
