"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const csv_parse_1 = __importDefault(require("csv-parse"));
const ip_address_1 = require("ip-address");
class CountryCodes {
    constructor(logger, config) {
        this.logger = logger;
        this.config = config;
        this.ranges = [];
        this.downloadRanges = async () => {
            if (this.config.countries === undefined ||
                this.config.countries.length === 0) {
                this.logger.verbose('Not updating country IP ranges because not relevant countries were set');
                return;
            }
            this.ranges = (await Promise.all((await Promise.all([this.config.ipV4Ranges, this.config.ipV6Ranges].map((url) => axios_1.default.get(url)))).map((res) => _a.parseDatabase(res.data)))).flat();
            this.logger.verbose('Updated country IP ranges database');
        };
        this.isRelevantCountry = (country) => country !== undefined &&
            this.config.countries !== undefined &&
            this.config.countries.includes(country);
        this.getCountryCode = (ip, isIpV6) => {
            const addr = _a.tryGetAddress(ip, isIpV6);
            if (addr === undefined) {
                return;
            }
            return this.ranges.find((country) => addr >= country.start && addr <= country.end)?.country;
        };
        if (this.config.countries && this.config.countries.length > 0) {
            this.logger.verbose(`Marking swaps from countries: ${this.config.countries}`);
        }
    }
}
_a = CountryCodes;
CountryCodes.parseDatabase = async (data) => {
    const parsed = await new Promise((resolve, reject) => {
        csv_parse_1.default.parse(data, {
            delimiter: ',',
        }, (error, data) => {
            if (error !== undefined) {
                reject(error);
            }
            resolve(data);
        });
    });
    return parsed.map((row) => ({
        start: BigInt(row[0]),
        end: BigInt(row[1]),
        country: row[2],
    }));
};
CountryCodes.getAddress = (address, isIpV6) => {
    const addr = isIpV6 ? ip_address_1.Address6 : ip_address_1.Address4;
    return BigInt(new addr(address).bigInteger());
};
CountryCodes.tryGetAddress = (address, isIpV6) => {
    if (isIpV6 !== undefined) {
        return _a.getAddress(address, isIpV6);
    }
    for (const isV6 of [false, true]) {
        try {
            return _a.getAddress(address, isV6);
        }
        catch (e) {
            /* empty */
        }
    }
    return undefined;
};
exports.default = CountryCodes;
//# sourceMappingURL=CountryCodes.js.map