"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const liquidjs_lib_1 = require("liquidjs-lib");
const Core_1 = require("../Core");
const Utils_1 = require("../Utils");
const ElementsClient_1 = __importDefault(require("../chain/ElementsClient"));
const Errors_1 = __importDefault(require("./Errors"));
class ElementsService {
    constructor(currencies, walletManager) {
        this.currencies = currencies;
        this.walletManager = walletManager;
        this.unblindOutputsFromId = async (txId) => {
            const { chainClient } = this.getElementsClients();
            return this.unblindOutputs(liquidjs_lib_1.Transaction.fromHex(await chainClient.getRawTransaction(txId)));
        };
        this.unblindOutputs = async (tx) => {
            const { chainClient, wallet } = this.getElementsClients();
            return await Promise.all(tx.outs.map(async (out) => {
                if (out.rangeProof !== undefined && out.rangeProof.length > 0) {
                    const keys = [
                        wallet.deriveBlindingKeyFromScript(out.script).privateKey,
                        (0, Utils_1.getHexBuffer)(await chainClient.dumpBlindingKey(wallet.encodeAddress(out.script, false))),
                    ];
                    for (const blindingKey of keys) {
                        try {
                            return (0, Core_1.unblindOutput)(wallet, out, blindingKey);
                        }
                        catch (e) {
                            /* empty */
                        }
                    }
                }
                return (0, Core_1.unblindOutput)(wallet, out);
            }));
        };
        this.deriveBlindingKeys = (address) => {
            const { wallet } = this.getElementsClients();
            const { publicKey, privateKey } = wallet.deriveBlindingKeyFromScript(wallet.decodeAddress(address));
            return {
                publicKey: publicKey,
                privateKey: privateKey,
            };
        };
        this.getElementsClients = () => {
            const currency = this.currencies.get(ElementsClient_1.default.symbol);
            const wallet = this.walletManager.wallets.get(ElementsClient_1.default.symbol);
            if (currency === undefined || wallet === undefined) {
                throw Errors_1.default.CURRENCY_NOT_FOUND(ElementsClient_1.default.symbol);
            }
            return {
                wallet,
                chainClient: currency.chainClient,
            };
        };
    }
}
exports.default = ElementsService;
//# sourceMappingURL=ElementsService.js.map