"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
exports.default = {
    CURRENCY_NOT_FOUND: (currency) => ({
        message: `could not find currency: ${currency}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 0),
    }),
    ORDER_SIDE_NOT_FOUND: (side) => ({
        message: `could not find order side: ${side}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 1),
    }),
    SWAP_NOT_FOUND: (id) => ({
        message: `could not find swap with id: ${id}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 2),
    }),
    SCRIPT_TYPE_NOT_FOUND: (scriptType) => ({
        message: `could not find script type of address: ${scriptType}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 3),
    }),
    PAIR_NOT_FOUND: (pairId) => ({
        message: `could not find pair with id: ${pairId}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 4),
    }),
    SWAP_WITH_INVOICE_EXISTS: () => ({
        message: 'a swap with this invoice exists already',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 5),
    }),
    EXCEED_MAXIMAL_AMOUNT: (amount, maximalAmount) => ({
        message: `${amount} is exceeds maximal of ${maximalAmount}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 6),
    }),
    BENEATH_MINIMAL_AMOUNT: (amount, minimalAmount) => ({
        message: `${amount} is less than minimal of ${minimalAmount}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 7),
    }),
    REVERSE_SWAPS_DISABLED: () => ({
        message: 'reverse swaps are disabled',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 8),
    }),
    ONCHAIN_AMOUNT_TOO_LOW: () => ({
        message: 'onchain amount is too low',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 9),
    }),
    BLOCK_TIME_NOT_FOUND: (symbol) => ({
        message: `could not find block time of currency: ${symbol}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 12),
    }),
    INVALID_TIMEOUT_BLOCK_DELTA: () => ({
        message: 'invalid timeout block delta',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 13),
    }),
    NO_TIMEOUT_DELTA: (pairId) => ({
        message: `could not find timeout delta of pair: ${pairId}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 14),
    }),
    SWAP_WITH_PREIMAGE_EXISTS: () => ({
        message: 'a swap with this preimage hash exists already',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 17),
    }),
    SWAP_HAS_INVOICE_ALREADY: (id) => ({
        message: `swap ${id} has an invoice already`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 18),
    }),
    SWAP_NO_LOCKUP: () => ({
        message: 'no coins were locked up yet',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 20),
    }),
    INVALID_INVOICE_AMOUNT: (maxInvoiceAmount) => ({
        message: `invoice amount exceeds the maximal of ${maxInvoiceAmount}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 21),
    }),
    EXCEEDS_MAX_INBOUND_LIQUIDITY: (inboundLiquidity, maxInboundLiquidity) => ({
        message: `inbound liquidity ${inboundLiquidity} exceeds maximal ${maxInboundLiquidity}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 22),
    }),
    BENEATH_MIN_INBOUND_LIQUIDITY: (inboundLiquidity, minInboundLiquidity) => ({
        message: `inbound liquidity ${inboundLiquidity} is less than minimal ${minInboundLiquidity}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 23),
    }),
    NOT_SUPPORTED_BY_SYMBOL: (symbol) => ({
        message: `this action is not supported by ${symbol}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 24),
    }),
    ETHEREUM_NOT_ENABLED: () => ({
        message: 'the Ethereum integration is not enabled',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 25),
    }),
    INVALID_ETHEREUM_ADDRESS: () => ({
        message: 'an invalid Ethereum address was provided',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 26),
    }),
    INVALID_PAIR_HASH: () => ({
        message: 'invalid pair hash',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 27),
    }),
    NO_AMOUNT_SPECIFIED: () => ({
        message: 'no amount was specified',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 28),
    }),
    INVOICE_AND_ONCHAIN_AMOUNT_SPECIFIED: () => ({
        message: 'invoice and onchain amount were specified',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 29),
    }),
    NOT_WHOLE_NUMBER: (input) => ({
        message: `${input} is not a whole number`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 30),
    }),
    AMP_INVOICES_NOT_SUPPORTED: () => ({
        message: 'AMP invoices not supported',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 31),
    }),
    MIN_EXPIRY_TOO_BIG: (swapMaximal, minFinalCltvExpiry, routingOffset) => ({
        message: `minimal swap expiry ${minFinalCltvExpiry} plus the routing offset ${routingOffset} minutes is greater than max swap timeout ${swapMaximal}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 32),
    }),
    DESTINATION_BOLTZ_NODE: () => ({
        message: 'swap to Boltz lightning nodes are forbidden',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 33),
    }),
    UNSUPPORTED_SWAP_VERSION: () => ({
        message: 'swap version not supported for pair',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 34),
    }),
    NOT_ELIGIBLE_FOR_COOPERATIVE_REFUND: () => ({
        message: 'swap not eligible for a cooperative refund',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 35),
    }),
    INCORRECT_PREIMAGE: () => ({
        message: 'incorrect preimage',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 36),
    }),
    INVALID_VIN: () => ({
        message: 'input index is out of range',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 37),
    }),
    NOT_ELIGIBLE_FOR_COOPERATIVE_CLAIM: () => ({
        message: 'swap not eligible for a cooperative claim',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 38),
    }),
    NOT_ELIGIBLE_FOR_COOPERATIVE_CLAIM_BROADCAST: () => ({
        message: 'swap not eligible for a cooperative claim broadcast',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 39),
    }),
    NO_CHAIN_FOR_SYMBOL: () => ({
        message: 'no chain for symbol',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 40),
    }),
    INVALID_PARTIAL_SIGNATURE: () => ({
        message: 'invalid partial signature',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 41),
    }),
    INVOICE_EXPIRY_TOO_SHORT: () => ({
        message: 'invoice expiry too short',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 42),
    }),
    SET_SWAP_UPDATE_EVENT_NOT_ALLOWED: (event) => ({
        message: `setting swap update event ${event} is not allowed`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Service, 43),
    }),
};
//# sourceMappingURL=Errors.js.map