import Logger from '../Logger';
import { SwapUpdateEvent } from '../consts/Enums';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import ChannelCreation from '../db/models/ChannelCreation';
import ReverseSwap from '../db/models/ReverseSwap';
import Swap from '../db/models/Swap';
import SwapNursery from '../swap/SwapNursery';
import { Currency } from '../wallet/WalletManager';
type TransactionInfo = {
    eta?: number;
    id: string;
    hex?: string;
};
type SwapUpdate = {
    status: SwapUpdateEvent;
    failureReason?: string;
    zeroConfRejected?: boolean;
    transaction?: TransactionInfo;
    channel?: {
        fundingTransactionId: string;
        fundingTransactionVout: number;
    };
};
declare class EventHandler extends TypedEventEmitter<{
    'swap.update': {
        id: string;
        status: SwapUpdate;
    };
    'swap.success': {
        swap: Swap | ReverseSwap;
        isReverse: boolean;
        channelCreation?: ChannelCreation;
    };
    'swap.failure': {
        swap: Swap | ReverseSwap;
        isReverse: boolean;
        reason: string;
    };
    'channel.backup': {
        currency: string;
        channelBackup: string;
    };
}> {
    private logger;
    private currencies;
    private nursery;
    constructor(logger: Logger, currencies: Map<string, Currency>, nursery: SwapNursery);
    emitSwapCreation: (id: string) => void;
    emitSwapInvoiceSet: (id: string) => void;
    /**
     * Subscribes transaction related swap events
     */
    private subscribeTransactions;
    /**
     * Subscribes to invoice related Swap events
     */
    private subscribeInvoices;
    /**
     * Subscribes Swap events
     */
    private subscribeSwapEvents;
    /**
     * Subscribes to a stream of channel backups
     */
    private subscribeChannelBackups;
    private handleFailedSwap;
    private handleFailedReverseSwap;
}
export default EventHandler;
export { SwapUpdate };
