"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const TimeoutDeltaProvider_1 = __importDefault(require("./TimeoutDeltaProvider"));
class InvoiceExpiryHelper {
    constructor(pairs, timeoutDeltaProvider) {
        this.invoiceExpiry = new Map();
        this.getExpiry = (pair) => {
            return (this.invoiceExpiry.get(pair) || InvoiceExpiryHelper.defaultInvoiceExpiry);
        };
        for (const pair of pairs) {
            const pairId = (0, Utils_1.getPairId)(pair);
            if (pair.invoiceExpiry) {
                this.invoiceExpiry.set(pairId, pair.invoiceExpiry);
                continue;
            }
            const delta = timeoutDeltaProvider.timeoutDeltas.get((0, Utils_1.getPairId)(pair))
                .quote.reverse;
            // Convert to seconds and divide by 2
            const expiry = Math.ceil((delta * TimeoutDeltaProvider_1.default.blockTimes.get(pair.quote) * 60) / 2);
            this.invoiceExpiry.set(pairId, expiry);
        }
    }
}
InvoiceExpiryHelper.defaultInvoiceExpiry = 3600;
/**
 * Calculates the expiry of an invoice
 * Reference: https://github.com/lightningnetwork/lightning-rfc/blob/master/11-payment-encoding.md#tagged-fields
 *
 * @param timestamp creation timestamp of the invoice
 * @param timeExpireDate expiry timestamp of the invoice
 */
InvoiceExpiryHelper.getInvoiceExpiry = (timestamp, timeExpireDate) => {
    let invoiceExpiry = timestamp || 0;
    if (timeExpireDate) {
        invoiceExpiry = timeExpireDate;
    }
    else {
        invoiceExpiry += InvoiceExpiryHelper.defaultInvoiceExpiry;
    }
    return invoiceExpiry;
};
exports.default = InvoiceExpiryHelper;
//# sourceMappingURL=InvoiceExpiryHelper.js.map