"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const liquidjs_lib_1 = require("liquidjs-lib");
const querystring_1 = require("querystring");
const DenominationConverter_1 = require("../DenominationConverter");
const ElementsClient_1 = __importDefault(require("../chain/ElementsClient"));
class PaymentRequestUtils {
    constructor(liquid) {
        this.prefixes = new Map([
            ['BTC', 'bitcoin'],
            ['LTC', 'litecoin'],
            [ElementsClient_1.default.symbol, 'liquidnetwork'],
        ]);
        /**
         * Encode a BIP21 payment request
         */
        this.encodeBip21 = (symbol, address, satoshis, label) => {
            const prefix = this.getBip21Prefix(symbol);
            const isLbtc = symbol === ElementsClient_1.default.symbol;
            if (prefix === undefined || (isLbtc && this.lbtcAssetHash === undefined)) {
                return;
            }
            const params = {
                amount: `${(0, DenominationConverter_1.satoshisToCoins)(satoshis)}`,
            };
            if (label) {
                params['label'] = label;
            }
            if (isLbtc) {
                params['assetid'] = this.lbtcAssetHash;
            }
            return `${prefix}:${address}?${(0, querystring_1.encode)(params)}`;
        };
        /**
         * Get the BIP21 prefix for a currency
         */
        this.getBip21Prefix = (symbol) => {
            return this.prefixes.get(symbol);
        };
        if (liquid) {
            this.lbtcAssetHash = liquid.network.assetHash;
            if (liquid.network === liquidjs_lib_1.networks.testnet) {
                this.prefixes.set(ElementsClient_1.default.symbol, 'liquidtestnet');
            }
        }
    }
}
exports.default = PaymentRequestUtils;
//# sourceMappingURL=PaymentRequestUtils.js.map