"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
class RoutingOffsets {
    constructor(logger, config) {
        this.logger = logger;
        this.params = new Map();
        this.exceptions = new Map();
        this.getOffset = (pair, amount, lightningCurrency, destinations) => {
            const exceptions = destinations
                .map((destination) => [
                destination,
                this.getOffsetException(lightningCurrency, destination),
            ])
                .filter((exception) => exception[1] !== undefined);
            if (exceptions.length > 0) {
                const exception = exceptions.reduce((e, max) => {
                    return e[1] > max[1] ? e : max;
                });
                this.logger.debug(`Using routing exception of ${exception[0]}: ${exception[1]}`);
                return exception[1];
            }
            return this.calculateOffset(pair, amount);
        };
        this.getOffsetException = (lightningCurrency, destination) => {
            const exceptions = this.exceptions.get(lightningCurrency);
            if (exceptions === undefined) {
                return undefined;
            }
            return exceptions.get(destination);
        };
        this.calculateOffset = (pair, amount) => {
            const params = this.params.get(pair);
            if (params === undefined) {
                return RoutingOffsets.maxOffset;
            }
            return Math.max(Math.min(Math.ceil(amount * params.k + params.d), RoutingOffsets.maxOffset), RoutingOffsets.minOffset);
        };
        config.pairs.forEach((pair) => {
            const k = (RoutingOffsets.maxOffset - RoutingOffsets.minOffset) /
                (pair.maxSwapAmount - pair.minSwapAmount);
            this.params.set((0, Utils_1.getPairId)(pair), {
                k,
                d: RoutingOffsets.minOffset - k * pair.minSwapAmount,
            });
        });
        config.currencies.forEach((currency) => {
            const exceptions = new Map((currency.routingOffsetExceptions || []).map((exception) => [
                exception.nodeId,
                exception.offset,
            ]));
            this.exceptions.set(currency.symbol, exceptions);
            this.logger.debug(`Using routing offset exceptions for ${currency.symbol}: ${(0, Utils_1.stringify)((0, Utils_1.mapToObject)(exceptions))}`);
        });
    }
}
RoutingOffsets.minOffset = 60;
RoutingOffsets.maxOffset = 900;
exports.default = RoutingOffsets;
//# sourceMappingURL=RoutingOffsets.js.map