import { ConfigType } from '../Config';
import Logger from '../Logger';
import { OrderSide, SwapVersion } from '../consts/Enums';
import { PairConfig } from '../consts/Types';
import Swap from '../db/models/Swap';
import { DecodedInvoice, LightningClient } from '../lightning/LightningClient';
import NodeSwitch from '../swap/NodeSwitch';
import { Currency } from '../wallet/WalletManager';
import EthereumManager from '../wallet/ethereum/EthereumManager';
type PairTimeoutBlocksDelta = {
    reverse: number;
    swapMinimal: number;
    swapMaximal: number;
    swapTaproot: number;
};
type PairTimeoutBlockDeltas = {
    base: PairTimeoutBlocksDelta;
    quote: PairTimeoutBlocksDelta;
};
declare class TimeoutDeltaProvider {
    private readonly logger;
    private readonly config;
    private readonly currencies;
    private readonly nodeSwitch;
    static readonly noRoutes = -1;
    static blockTimes: Map<string, number>;
    timeoutDeltas: Map<string, PairTimeoutBlockDeltas>;
    private routingOffsets;
    constructor(logger: Logger, config: ConfigType, currencies: Map<string, Currency>, nodeSwitch: NodeSwitch);
    static convertBlocks: (fromSymbol: string, toSymbol: string, blocks: number) => number;
    init: (pairs: PairConfig[], ethereumManagers: EthereumManager[]) => void;
    setTimeout: (pairId: string, newDeltas: PairTimeoutBlocksDelta) => void;
    getCltvLimit: (swap: Swap) => Promise<number>;
    getTimeout: (pairId: string, orderSide: OrderSide, isReverse: boolean, version: SwapVersion, invoice?: string, referralId?: string) => Promise<[number, boolean]>;
    checkRoutability: (lightningClient: LightningClient, decodedInvoice: DecodedInvoice, cltvLimit: number) => Promise<number>;
    private getTimeoutInvoice;
    private minutesToBlocks;
}
export default TimeoutDeltaProvider;
export { PairTimeoutBlocksDelta };
