/// <reference types="node" />
import { Transaction } from 'bitcoinjs-lib';
import { Musig } from 'boltz-core';
import { Transaction as LiquidTransaction } from 'liquidjs-lib';
import Logger from '../../Logger';
import TypedEventEmitter from '../../consts/TypedEventEmitter';
import ChannelCreation from '../../db/models/ChannelCreation';
import Swap from '../../db/models/Swap';
import SwapOutputType from '../../swap/SwapOutputType';
import WalletManager, { Currency } from '../../wallet/WalletManager';
type SwapConfig = {
    deferredClaimSymbols: string[];
    batchClaimInterval: string;
    expiryTolerance: number;
};
type CooperativeDetails = {
    musig: Musig;
    sweepAddress: string;
    transaction: Transaction | LiquidTransaction;
};
export type SwapToClaim = {
    swap: Swap;
    preimage: Buffer;
    cooperative?: CooperativeDetails;
};
declare class DeferredClaimer extends TypedEventEmitter<{
    claim: {
        swap: Swap;
        channelCreation?: ChannelCreation;
    };
}> {
    private readonly logger;
    private readonly currencies;
    private readonly walletManager;
    private swapOutputType;
    private readonly config;
    private static readonly batchClaimLock;
    private static readonly swapsToClaimLock;
    private readonly lock;
    private readonly swapsToClaim;
    private batchClaimSchedule?;
    constructor(logger: Logger, currencies: Map<string, Currency>, walletManager: WalletManager, swapOutputType: SwapOutputType, config: SwapConfig);
    init: () => Promise<void>;
    close: () => void;
    pendingSweeps: () => Map<string, string[]>;
    pendingSweepsValues(): Map<string, SwapToClaim[]>;
    sweep: () => Promise<Map<string, string[]>>;
    sweepSymbol: (symbol: string) => Promise<string[]>;
    deferClaim: (swap: Swap, preimage: Buffer) => Promise<boolean>;
    getCooperativeDetails: (swap: Swap) => Promise<{
        preimage: Buffer;
        pubNonce: Buffer;
        publicKey: Buffer;
        transactionHash: Buffer;
    }>;
    broadcastCooperative: (swap: Swap, theirPubNonce: Buffer, theirPartialSignature: Buffer) => Promise<void>;
    private batchClaim;
    private broadcastClaim;
    private constructClaimDetails;
    private shouldBeDeferred;
    private expiryTooSoon;
    private logNotDeferringReason;
    private batchClaimLeftovers;
    private getToClaimDetails;
}
export default DeferredClaimer;
export { SwapConfig };
