"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
exports.default = {
    CURRENCY_NOT_FOUND: (currency) => ({
        message: `could not find currency: ${currency}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 0),
    }),
    NO_ROUTE_FOUND: () => ({
        message: 'could not find route to pay invoice',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 1),
    }),
    NO_LIGHTNING_SUPPORT: (symbol) => ({
        message: `${symbol} has no lightning support`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 2),
    }),
    INVOICE_INVALID_PREIMAGE_HASH: (preimageHash) => ({
        message: `the preimage hash of the invoice does not match the one of the Swap: ${preimageHash}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 3),
    }),
    INVOICE_EXPIRES_TOO_EARLY: (invoiceExpiry, timeoutTimestamp) => ({
        message: `invoice expiry ${invoiceExpiry} is before Swap timeout: ${timeoutTimestamp}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 4),
    }),
    COINS_COULD_NOT_BE_SENT: () => ({
        message: 'onchain coins could not be sent',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 5),
    }),
    INSUFFICIENT_AMOUNT: (actualAmount, expectedAmount) => ({
        message: `locked ${actualAmount} is less than expected ${expectedAmount}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 6),
    }),
    INVALID_TOKEN_LOCKED: (actualToken, expectedToken) => ({
        message: `the locked token ${actualToken} is not expected ${expectedToken}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 7),
    }),
    INVALID_CLAIM_ADDRESS: (actualAddress, expectedAddress) => ({
        message: `the specified claim address ${actualAddress} is not expected ${expectedAddress}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 8),
    }),
    INVALID_TIMELOCK: (actualTimelock, expectedTimelock) => ({
        message: `the specified timelock ${actualTimelock} is not expected ${expectedTimelock}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 9),
    }),
    INVOICE_COULD_NOT_BE_PAID: () => ({
        message: 'invoice could not be paid',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 10),
    }),
    REFUNDED_COINS: (refundedTransactionId) => ({
        message: `refunded onchain coins: ${refundedTransactionId}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 11),
    }),
    ONCHAIN_HTLC_TIMED_OUT: () => ({
        message: 'onchain HTLC timed out',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 12),
    }),
    INVOICE_EXPIRED_ALREADY: () => ({
        message: 'the provided invoice expired already',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 13),
    }),
    SWAP_DOES_NOT_ACCEPT_ZERO_CONF: () => ({
        message: 'Swap does not accept 0-conf',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 14),
    }),
    LOCKUP_TRANSACTION_SIGNALS_RBF: () => ({
        message: 'transaction or one of its unconfirmed ancestors signals RBF',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 15),
    }),
    LOCKUP_TRANSACTION_FEE_TOO_LOW: () => ({
        message: 'transaction fee is too low',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 16),
    }),
    LIGHTNING_CLIENT_CALL_TIMEOUT: () => ({
        message: 'lightning client call timeout',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 17),
    }),
    NO_AVAILABLE_LIGHTNING_CLIENT: () => ({
        message: 'no available lightning client',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 18),
    }),
    INVALID_ADDRESS: () => ({
        message: 'invalid address',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 19),
    }),
    BLOCKED_ADDRESS: () => ({
        message: 'blocked address',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 20),
    }),
    INVALID_ADDRESS_SIGNATURE: () => ({
        message: 'invalid address signature',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Swap, 21),
    }),
};
//# sourceMappingURL=Errors.js.map