/// <reference types="node" />
import { TransactionResponse } from 'ethers';
import Logger from '../Logger';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import { ERC20SwapValues, EtherSwapValues } from '../consts/Types';
import ReverseSwap from '../db/models/ReverseSwap';
import Swap from '../db/models/Swap';
import Blocks from '../service/Blocks';
import WalletManager from '../wallet/WalletManager';
import EthereumManager from '../wallet/ethereum/EthereumManager';
declare class EthereumNursery extends TypedEventEmitter<{
    'eth.lockup': {
        swap: Swap;
        transactionHash: string;
        etherSwapValues: EtherSwapValues;
    };
    'erc20.lockup': {
        swap: Swap;
        transactionHash: string;
        erc20SwapValues: ERC20SwapValues;
    };
    'swap.expired': {
        swap: Swap;
        isEtherSwap: boolean;
    };
    'lockup.failed': {
        swap: Swap;
        reason: string;
    };
    'reverseSwap.expired': {
        reverseSwap: ReverseSwap;
        isEtherSwap: boolean;
    };
    'lockup.failedToSend': {
        reverseSwap: ReverseSwap;
        reason: string;
    };
    'lockup.confirmed': {
        reverseSwap: ReverseSwap;
        transactionHash: string;
    };
    claim: {
        reverseSwap: ReverseSwap;
        preimage: Buffer;
    };
}> {
    private readonly logger;
    private readonly walletManager;
    readonly ethereumManager: EthereumManager;
    private readonly blocks;
    constructor(logger: Logger, walletManager: WalletManager, ethereumManager: EthereumManager, blocks: Blocks);
    init: () => Promise<void>;
    listenContractTransaction: (reverseSwap: ReverseSwap, transaction: TransactionResponse) => void;
    private listenEtherSwap;
    private listenERC20Swap;
    private listenBlocks;
    private checkExpiredSwaps;
    private checkExpiredReverseSwaps;
    /**
     * Returns a wallet in case there is one with the symbol, and it is an Ethereum one
     */
    private getEthereumWallet;
}
export default EthereumNursery;
