"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = __importDefault(require("./Errors"));
class NodeFallback {
    constructor(logger, nodeSwitch, routingHints) {
        this.logger = logger;
        this.nodeSwitch = nodeSwitch;
        this.routingHints = routingHints;
        this.getReverseSwapInvoice = async (id, referralId, routingNode, currency, holdInvoiceAmount, preimageHash, cltvExpiry, expiry, memo, routingHints) => {
            let nodeForSwap = this.nodeSwitch.getNodeForReverseSwap(id, currency, holdInvoiceAmount, referralId);
            while (nodeForSwap.lightningClient !== undefined) {
                try {
                    return {
                        ...nodeForSwap,
                        ...(await this.addHoldInvoice(nodeForSwap.nodeType, nodeForSwap.lightningClient, routingNode, currency, holdInvoiceAmount, preimageHash, cltvExpiry, expiry, memo, routingHints)),
                    };
                }
                catch (e) {
                    if (e.message === Errors_1.default.LIGHTNING_CLIENT_CALL_TIMEOUT().message) {
                        this.logger.warn(`${nodeForSwap.lightningClient.serviceName()} invoice creation timed out after ${NodeFallback.addInvoiceTimeout}ms; trying next node`);
                        nodeForSwap = this.nodeSwitch.getNodeForReverseSwap(id, currency, holdInvoiceAmount, referralId);
                        continue;
                    }
                    throw e;
                }
            }
            throw Errors_1.default.NO_AVAILABLE_LIGHTNING_CLIENT();
        };
        this.addHoldInvoice = async (nodeType, lightningClient, routingNode, currency, holdInvoiceAmount, preimageHash, cltvExpiry, expiry, memo, externalHints) => {
            const nodeHints = routingNode !== undefined
                ? await this.routingHints.getRoutingHints(currency.symbol, routingNode, nodeType)
                : undefined;
            const routingHints = [nodeHints, externalHints]
                .filter((hints) => hints !== undefined)
                .flat();
            return lightningClient.raceCall(async () => ({
                routingHints,
                paymentRequest: await lightningClient.addHoldInvoice(holdInvoiceAmount, preimageHash, cltvExpiry, expiry, memo, routingHints),
            }), (reject) => reject(Errors_1.default.LIGHTNING_CLIENT_CALL_TIMEOUT()), NodeFallback.addInvoiceTimeout);
        };
    }
}
NodeFallback.addInvoiceTimeout = 10000;
exports.default = NodeFallback;
//# sourceMappingURL=NodeFallback.js.map