/// <reference types="node" />
import { Transaction } from 'bitcoinjs-lib';
import { Transaction as LiquidTransaction } from 'liquidjs-lib/src/transaction';
import Logger from '../Logger';
import ChannelCreation from '../db/models/ChannelCreation';
import ReverseSwap from '../db/models/ReverseSwap';
import Swap from '../db/models/Swap';
import TimeoutDeltaProvider from '../service/TimeoutDeltaProvider';
import { Currency } from '../wallet/WalletManager';
import ChannelNursery from './ChannelNursery';
import NodeSwitch from './NodeSwitch';
type SwapNurseryEvents = {
    transaction: {
        swap: Swap | ReverseSwap;
        transaction: Transaction | LiquidTransaction | string;
        confirmed: boolean;
        isReverse: boolean;
    };
    expiration: {
        swap: Swap | ReverseSwap;
        isReverse: boolean;
    };
    'lockup.failed': Swap;
    'zeroconf.rejected': Swap;
    'invoice.pending': Swap;
    'invoice.failedToPay': Swap;
    'invoice.paid': Swap;
    'claim.pending': Swap;
    claim: {
        swap: Swap;
        channelCreation?: ChannelCreation;
    };
    'minerfee.paid': ReverseSwap;
    'invoice.expired': ReverseSwap;
    'coins.sent': {
        reverseSwap: ReverseSwap;
        transaction: Transaction | LiquidTransaction | string;
    };
    'coins.failedToSend': ReverseSwap;
    refund: {
        reverseSwap: ReverseSwap;
        refundTransaction: string;
    };
    'invoice.settled': ReverseSwap;
};
declare class PaymentHandler {
    private readonly logger;
    private readonly nodeSwitch;
    private readonly currencies;
    readonly channelNursery: ChannelNursery;
    private readonly timeoutDeltaProvider;
    private emit;
    private static readonly raceTimeout;
    private static readonly resetMissionControlInterval;
    private static readonly errCltvTooSmall;
    private lastResetMissionControl;
    constructor(logger: Logger, nodeSwitch: NodeSwitch, currencies: Map<string, Currency>, channelNursery: ChannelNursery, timeoutDeltaProvider: TimeoutDeltaProvider, emit: <K extends keyof SwapNurseryEvents>(eventName: K, arg: SwapNurseryEvents[K]) => void);
    payInvoice: (swap: Swap, channelCreation: ChannelCreation | null, outgoingChannelId?: string) => Promise<Buffer | undefined>;
    private racePayInvoice;
    private handlePaymentFailure;
    private handleLndPaymentFailure;
    private handleClnPaymentFailure;
    private abandonSwap;
    private settleInvoice;
    private logPaymentFailure;
}
export default PaymentHandler;
export { SwapNurseryEvents };
