"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const ECPairHelper_1 = require("../ECPairHelper");
const Utils_1 = require("../Utils");
const ChannelUtils_1 = require("../lightning/ChannelUtils");
const Errors_1 = __importDefault(require("./Errors"));
class ReverseRoutingHints {
    constructor(walletManager, rateProvider, paymentRequestUtils) {
        this.walletManager = walletManager;
        this.rateProvider = rateProvider;
        this.paymentRequestUtils = paymentRequestUtils;
        this.getHints = (sendingCurrency, args) => {
            const invoiceMemo = (0, Utils_1.getSwapMemo)(sendingCurrency.symbol, true);
            const receivedAmount = args.onchainAmount -
                this.rateProvider.feeProvider.minerFees.get(sendingCurrency.symbol)[args.version].reverse.claim;
            if (args.userAddress === undefined ||
                args.userAddressSignature === undefined) {
                return { invoiceMemo, receivedAmount };
            }
            try {
                this.walletManager.wallets
                    .get(sendingCurrency.symbol)
                    .decodeAddress(args.userAddress);
            }
            catch (e) {
                throw Errors_1.default.INVALID_ADDRESS();
            }
            const bip21 = this.paymentRequestUtils.encodeBip21(sendingCurrency.symbol, args.userAddress, receivedAmount);
            const routingHint = this.encodeRoutingHint(args.claimPublicKey, args.userAddress, args.userAddressSignature);
            return {
                bip21,
                routingHint,
                invoiceMemo,
                receivedAmount,
            };
        };
        this.encodeRoutingHint = (claimPublicKey, userAddress, userAddressSignature) => {
            if (claimPublicKey === undefined || userAddressSignature === undefined) {
                return undefined;
            }
            const sigValid = ECPairHelper_1.ECPair.fromPublicKey(claimPublicKey).verifySchnorr(bitcoinjs_lib_1.crypto.sha256(Buffer.from(userAddress, 'utf-8')), userAddressSignature);
            if (!sigValid) {
                throw Errors_1.default.INVALID_ADDRESS_SIGNATURE();
            }
            return [
                [
                    {
                        nodeId: (0, Utils_1.getHexString)(claimPublicKey),
                        chanId: ReverseRoutingHints.routingHintChanId,
                        feeBaseMsat: 0,
                        cltvExpiryDelta: 81,
                        feeProportionalMillionths: 21,
                    },
                ],
            ];
        };
    }
}
ReverseRoutingHints.routingHintChanId = (0, ChannelUtils_1.transactionToLndScid)(542409, 1308, 0);
exports.default = ReverseRoutingHints;
//# sourceMappingURL=ReverseRoutingHints.js.map