/// <reference types="node" />
import AsyncLock from 'async-lock';
import Logger from '../Logger';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import ReverseSwap from '../db/models/ReverseSwap';
import Swap from '../db/models/Swap';
import LockupTransactionTracker from '../rates/LockupTransactionTracker';
import RateProvider from '../rates/RateProvider';
import Blocks from '../service/Blocks';
import TimeoutDeltaProvider from '../service/TimeoutDeltaProvider';
import DeferredClaimer from '../service/cooperative/DeferredClaimer';
import WalletManager, { Currency } from '../wallet/WalletManager';
import ChannelNursery from './ChannelNursery';
import NodeSwitch from './NodeSwitch';
import { SwapNurseryEvents } from './PaymentHandler';
import SwapOutputType from './SwapOutputType';
declare class SwapNursery extends TypedEventEmitter<SwapNurseryEvents> {
    private logger;
    private nodeSwitch;
    private rateProvider;
    private walletManager;
    private swapOutputType;
    private retryInterval;
    private readonly claimer;
    static reverseSwapMempoolEta: number;
    private readonly utxoNursery;
    readonly channelNursery: ChannelNursery;
    private readonly invoiceNursery;
    private readonly paymentHandler;
    private readonly lightningNursery;
    private readonly ethereumNurseries;
    private currencies;
    readonly lock: AsyncLock;
    static readonly swapLock = "swap";
    static readonly reverseSwapLock = "reverseSwap";
    private static retryLock;
    constructor(logger: Logger, nodeSwitch: NodeSwitch, rateProvider: RateProvider, timeoutDeltaProvider: TimeoutDeltaProvider, walletManager: WalletManager, swapOutputType: SwapOutputType, retryInterval: number, blocks: Blocks, claimer: DeferredClaimer, lockupTransactionTracker: LockupTransactionTracker);
    init: (currencies: Currency[]) => Promise<void>;
    attemptSettleSwap: (currency: Currency, swap: Swap, outgoingChannelId?: string) => Promise<void>;
    settleReverseSwapInvoice: (reverseSwap: ReverseSwap, preimage: Buffer) => Promise<void>;
    private listenEthereumNursery;
    /**
     * Sets the rate for a Swap that doesn't have an invoice yet
     */
    private setSwapRate;
    private lockupUtxo;
    private lockupEther;
    private lockupERC20;
    private claimUtxo;
    private claimEther;
    private claimERC20;
    private handleReverseSwapSendFailed;
    private lockupFailed;
    private expireSwap;
    private expireReverseSwap;
    private refundUtxo;
    private refundEther;
    private refundERC20;
    private findEthereumNursery;
}
export default SwapNursery;
