/// <reference types="node" />
import { Transaction } from 'bitcoinjs-lib';
import { Transaction as LiquidTransaction } from 'liquidjs-lib';
import Logger from '../Logger';
import TypedEventEmitter from '../consts/TypedEventEmitter';
import ReverseSwap from '../db/models/ReverseSwap';
import Swap from '../db/models/Swap';
import LockupTransactionTracker from '../rates/LockupTransactionTracker';
import Blocks from '../service/Blocks';
import WalletManager, { Currency } from '../wallet/WalletManager';
declare class UtxoNursery extends TypedEventEmitter<{
    'swap.expired': Swap;
    'swap.lockup.failed': {
        swap: Swap;
        reason: string;
    };
    'swap.lockup.zeroconf.rejected': {
        swap: Swap;
        transaction: Transaction | LiquidTransaction;
        reason: string;
    };
    'swap.lockup': {
        swap: Swap;
        transaction: Transaction | LiquidTransaction;
        confirmed: boolean;
    };
    'reverseSwap.expired': ReverseSwap;
    'reverseSwap.lockup.confirmed': {
        reverseSwap: ReverseSwap;
        transaction: Transaction | LiquidTransaction;
    };
    'reverseSwap.claimed': {
        reverseSwap: ReverseSwap;
        preimage: Buffer;
    };
}> {
    private readonly logger;
    private readonly walletManager;
    private readonly blocks;
    private readonly lockupTransactionTracker;
    private static readonly maxParallelRequests;
    private lock;
    private static swapLockupLock;
    private static reverseSwapLockupConfirmationLock;
    constructor(logger: Logger, walletManager: WalletManager, blocks: Blocks, lockupTransactionTracker: LockupTransactionTracker);
    bindCurrency: (currencies: Currency[]) => void;
    private listenTransactions;
    private checkSwapOutputs;
    private checkReverseSwapClaims;
    private checkReverseSwapLockupsConfirmed;
    private listenBlocks;
    private checkSwapMempoolTransactions;
    private checkReverseSwapMempoolTransactions;
    private checkExpiredSwaps;
    private checkExpiredReverseSwaps;
    private reverseSwapLockupConfirmed;
    private checkSwapTransaction;
    /**
     * Detects whether the transaction signals RBF explicitly or inherently
     */
    private transactionSignalsRbf;
    private getPreviousAddresses;
    private getTransactions;
    private chunkArray;
}
export default UtxoNursery;
