"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ReverseSwap_1 = require("../../db/models/ReverseSwap");
const RoutingHintsLnd_1 = __importDefault(require("./RoutingHintsLnd"));
class RoutingHints {
    constructor(logger, currencies) {
        this.logger = logger;
        this.providers = new Map();
        this.start = async () => {
            const startPromises = Array.from(this.providers.values())
                .filter((prov) => prov.lnd !== undefined)
                .map((prov) => prov.lnd?.start());
            await Promise.all(startPromises);
        };
        this.stop = () => {
            Array.from(this.providers.values())
                .flatMap((prov) => [prov.lnd, prov.cln])
                .filter((prov) => prov !== undefined)
                .forEach((prov) => prov.stop());
        };
        this.getRoutingHints = async (symbol, nodeId, nodeType) => {
            const providers = this.providers.get(symbol);
            if (providers === undefined) {
                return [];
            }
            // Prefer the LND routing hints provider
            return ((RoutingHints.getProviderForNodeType(providers, nodeType) ||
                providers.lnd ||
                providers.cln)?.routingHints(nodeId) || []);
        };
        currencies
            .filter((cur) => cur.lndClient !== undefined || cur.clnClient !== undefined)
            .forEach((cur) => {
            this.providers.set(cur.symbol, {
                lnd: cur.lndClient !== undefined
                    ? new RoutingHintsLnd_1.default(this.logger, cur.lndClient)
                    : undefined,
                cln: cur.clnClient,
            });
        });
        this.logger.debug(`Initializing routing hints provider for: ${Array.from(this.providers.keys()).join(', ')}`);
    }
}
RoutingHints.getProviderForNodeType = (providers, nodeType) => {
    switch (nodeType) {
        case ReverseSwap_1.NodeType.LND:
            return providers.lnd;
        case ReverseSwap_1.NodeType.CLN:
            return providers.cln;
    }
    return undefined;
};
exports.default = RoutingHints;
//# sourceMappingURL=RoutingHints.js.map