"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const LndClient_1 = __importDefault(require("../../lightning/LndClient"));
class RoutingHintsLnd {
    constructor(logger, lnd) {
        this.logger = logger;
        this.lnd = lnd;
        this.channelInfos = new Map();
        this.start = async () => {
            const info = await this.lnd.getInfo();
            this.ourPubkey = info.pubkey;
            await this.update();
            this.logger.debug(`Fetching channels for routing hints provider of ${this.name} every ${RoutingHintsLnd.channelFetchInterval} minutes`);
            this.interval = setInterval(async () => {
                await this.update();
            }, (0, Utils_1.minutesToMilliseconds)(RoutingHintsLnd.channelFetchInterval));
        };
        this.stop = () => {
            if (this.interval !== undefined) {
                clearInterval(this.interval);
                this.interval = undefined;
            }
        };
        this.routingHints = async (nodeId) => {
            return this.channelInfos.get(nodeId) || [];
        };
        this.update = async () => {
            this.logger.silly(`Updating channel list for ${this.name} routing hint provider`);
            const channelInfos = new Map();
            const channels = await this.lnd.listChannels(true, true);
            for (const channel of channels) {
                const routingInfo = await this.lnd.getChannelInfo(channel.chanId);
                const remotePolicy = routingInfo.node1Pub === this.ourPubkey
                    ? routingInfo.node2Policy
                    : routingInfo.node1Policy;
                if (remotePolicy === undefined) {
                    continue;
                }
                if (!channelInfos.has(channel.remotePubkey)) {
                    channelInfos.set(channel.remotePubkey, []);
                }
                channelInfos.get(channel.remotePubkey).push([
                    {
                        nodeId: channel.remotePubkey,
                        chanId: channel.chanId,
                        feeBaseMsat: remotePolicy.feeBaseMsat,
                        cltvExpiryDelta: remotePolicy.timeLockDelta,
                        feeProportionalMillionths: remotePolicy.feeRateMilliMsat,
                    },
                ]);
            }
            this.logger.silly(`Found ${channelInfos.size} private ${this.name} channels`);
            this.channelInfos = channelInfos;
        };
        this.name = `${this.lnd.symbol} ${LndClient_1.default.serviceName}`;
    }
}
// How often the channel lists should be updated in minutes
RoutingHintsLnd.channelFetchInterval = 15;
exports.default = RoutingHintsLnd;
//# sourceMappingURL=RoutingHintsLnd.js.map