"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../Utils");
const Enums_1 = require("../consts/Enums");
exports.default = {
    NOT_INITIALIZED: () => ({
        message: 'wallet not initialized',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 0),
    }),
    INVALID_MNEMONIC: (mnemonic) => ({
        message: `mnemonic "${mnemonic}" is invalid`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 1),
    }),
    INVALID_DEPTH_INDEX: (depth) => ({
        message: `depth index "${depth}" is invalide`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 2),
    }),
    NOT_ENOUGH_FUNDS: (amount) => ({
        message: `not enough funds to send ${amount}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 3),
    }),
    INVALID_SIGNATURE: () => ({
        message: 'could not verify signatures of constructed transaction',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 4),
    }),
    CURRENCY_NOT_SUPPORTED: (symbol) => ({
        message: `${symbol} is not supported`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 5),
    }),
    OUTPUTTYPE_NOT_SUPPORTED: (symbol, type) => ({
        message: `${symbol} wallet does not supports outputs of type: ${type}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 6),
    }),
    NO_WALLET_SUPPORT: (symbol) => ({
        message: `${symbol} core has no wallet support compiled in`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 7),
    }),
    MISSING_SWAP_CONTRACTS: () => ({
        message: 'missing swap contracts',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 8),
    }),
    INVALID_ETHEREUM_CONFIGURATION: (reason) => ({
        message: `invalid Ethereum configuration: ${reason}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 9),
    }),
    NOT_SUPPORTED_BY_WALLET: (symbol, method) => ({
        message: `"${method}" not supported by ${symbol}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 9),
    }),
    UNSUPPORTED_CONTRACT_VERSION: (name, address, actualVersion, supportedVersion) => ({
        message: `unsupported ${name} (${address}) contract version ${Number(actualVersion)}; supported version: ${Number(supportedVersion)}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 10),
    }),
    TAPROOT_BLINDING_NOT_SUPPORTED: () => ({
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Wallet, 11),
        message: 'blinding of Taproot addresses not supported',
    }),
};
//# sourceMappingURL=Errors.js.map