/// <reference types="node" />
import { BIP32Interface } from 'bip32';
import { Network } from 'bitcoinjs-lib';
import Logger from '../Logger';
import { CurrencyType } from '../consts/Enums';
import WalletProviderInterface, { BalancerFetcher, SentTransaction, WalletBalance } from './providers/WalletProviderInterface';
declare class Wallet implements BalancerFetcher {
    private logger;
    type: CurrencyType;
    walletProvider: WalletProviderInterface;
    readonly symbol: string;
    network: Network;
    private derivationPath?;
    private highestUsedIndex?;
    private masterNode?;
    /**
     * Wallet is a hierarchical deterministic wallet for a currency
     */
    constructor(logger: Logger, type: CurrencyType, walletProvider: WalletProviderInterface);
    serviceName: () => string;
    /**
     * In case the Wallet should also provide keys and de/encode addresses
     * This is only required (and possible) for UTXO based chains like Bitcoin
     */
    initKeyProvider: (network: Network, derivationPath: string, highestUsedIndex: number, masterNode: BIP32Interface) => void;
    /**
     * Gets a specific pair of keys
     *
     * @param index index of the keys to get
     */
    getKeysByIndex: (index: number) => BIP32Interface;
    /**
     * Gets a new pair of keys
     */
    getNewKeys: () => {
        keys: BIP32Interface;
        index: number;
    };
    /**
     * Encodes an address
     *
     * @param outputScript the output script to encode
     */
    encodeAddress: (outputScript: Buffer) => string;
    /**
     * Decodes an address
     */
    decodeAddress: (toDecode: string) => Buffer;
    getAddress: () => Promise<string>;
    getBalance: () => Promise<WalletBalance>;
    sendToAddress: (address: string, amount: number, satPerVbyte?: number) => Promise<SentTransaction>;
    sweepWallet: (address: string, satPerVbyte?: number) => Promise<SentTransaction>;
}
export default Wallet;
