"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const liquidjs_lib_1 = require("liquidjs-lib");
const Enums_1 = require("../consts/Enums");
const Wallet_1 = __importDefault(require("./Wallet"));
class WalletLiquid extends Wallet_1.default {
    constructor(logger, walletProvider, slip77) {
        super(logger, Enums_1.CurrencyType.Liquid, walletProvider);
        this.slip77 = slip77;
        this.deriveBlindingKeyFromScript = (outputScript) => {
            return this.slip77.derive(outputScript);
        };
        this.encodeAddress = (outputScript, shouldBlind = true) => {
            try {
                // Fee output of Liquid
                if (outputScript.length == 0) {
                    return '';
                }
                const res = this.getPaymentFunc(outputScript)({
                    output: outputScript,
                    network: this.network,
                    blindkey: shouldBlind
                        ? this.deriveBlindingKeyFromScript(outputScript).publicKey
                        : undefined,
                });
                return shouldBlind ? res.confidentialAddress : res.address;
            }
            catch (error) {
                // Ignore invalid addresses
                return '';
            }
        };
        this.getPaymentFunc = (outputScript) => {
            switch (liquidjs_lib_1.address.getScriptType(outputScript)) {
                case liquidjs_lib_1.address.ScriptType.P2Pkh:
                    return liquidjs_lib_1.payments.p2pkh;
                case liquidjs_lib_1.address.ScriptType.P2Sh:
                    return liquidjs_lib_1.payments.p2sh;
                case liquidjs_lib_1.address.ScriptType.P2Wpkh:
                    return liquidjs_lib_1.payments.p2wpkh;
                case liquidjs_lib_1.address.ScriptType.P2Wsh:
                    return liquidjs_lib_1.payments.p2wsh;
                default:
                    return WalletLiquid.blindP2tr;
            }
        };
    }
}
WalletLiquid.blindP2tr = (payment) => {
    const addr = liquidjs_lib_1.address.fromOutputScript(payment.output, payment.network);
    const dec = liquidjs_lib_1.address.fromBech32(addr);
    let confidentialAddress;
    if (payment.blindkey) {
        confidentialAddress = liquidjs_lib_1.address.toBlech32(Buffer.concat([Buffer.from([dec.version, dec.data.length]), dec.data]), payment.blindkey, payment.network.blech32, dec.version);
    }
    return {
        confidentialAddress,
        address: addr,
    };
};
exports.default = WalletLiquid;
//# sourceMappingURL=WalletLiquid.js.map