import { Network } from 'bitcoinjs-lib';
import { Provider } from 'ethers';
import { CurrencyConfig } from '../Config';
import Logger from '../Logger';
import { IChainClient } from '../chain/ChainClient';
import { CurrencyType } from '../consts/Enums';
import LndClient from '../lightning/LndClient';
import ClnClient from '../lightning/cln/ClnClient';
import Wallet from './Wallet';
import EthereumManager from './ethereum/EthereumManager';
type CurrencyLimits = {
    minWalletBalance: number;
    minLocalBalance?: number;
    minRemoteBalance?: number;
    maxZeroConfAmount?: number;
};
type Currency = {
    symbol: string;
    type: CurrencyType;
    limits: CurrencyLimits;
    network?: Network;
    lndClient?: LndClient;
    clnClient?: ClnClient;
    chainClient?: IChainClient;
    provider?: Provider;
};
/**
 * WalletManager creates wallets instances that generate keys derived from the seed and
 * interact with the wallet of LND to send and receive onchain coins
 */
declare class WalletManager {
    private logger;
    private currencies;
    ethereumManagers: EthereumManager[];
    wallets: Map<string, Wallet>;
    private readonly mnemonic;
    private readonly slip77;
    private readonly masterNode;
    private readonly derivationPath;
    constructor(logger: Logger, mnemonicPath: string, currencies: Currency[], ethereumManagers: EthereumManager[]);
    init: (configCurrencies: CurrencyConfig[]) => Promise<void>;
    private loadMnemonic;
    private getKeyProviderMap;
    private getHighestDepthIndex;
}
export default WalletManager;
export { Currency };
