/// <reference types="node" />
import { ERC20Swap } from 'boltz-core/typechain/ERC20Swap';
import { EtherSwap } from 'boltz-core/typechain/EtherSwap';
import { Transaction, TransactionResponse } from 'ethers';
import Logger from '../../Logger';
import TypedEventEmitter from '../../consts/TypedEventEmitter';
import { ERC20SwapValues, EtherSwapValues } from '../../consts/Types';
import { NetworkDetails } from './EvmNetworks';
type Events = {
    'eth.lockup': {
        transaction: Transaction | TransactionResponse;
        etherSwapValues: EtherSwapValues;
    };
    'eth.claim': {
        transactionHash: string;
        preimageHash: Buffer;
        preimage: Buffer;
    };
    'eth.refund': {
        transactionHash: string;
        preimageHash: Buffer;
    };
    'erc20.lockup': {
        transaction: Transaction | TransactionResponse;
        erc20SwapValues: ERC20SwapValues;
    };
    'erc20.claim': {
        transactionHash: string;
        preimageHash: Buffer;
        preimage: Buffer;
    };
    'erc20.refund': {
        transactionHash: string;
        preimageHash: Buffer;
    };
};
declare class ContractEventHandler extends TypedEventEmitter<Events> {
    private readonly logger;
    private etherSwap;
    private erc20Swap;
    constructor(logger: Logger);
    init: (networkDetails: NetworkDetails, etherSwap: EtherSwap, erc20Swap: ERC20Swap) => Promise<void>;
    rescan: (startHeight: number) => Promise<void>;
    private subscribeContractEvents;
}
export default ContractEventHandler;
