"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatERC20SwapValues = exports.formatEtherSwapValues = exports.queryERC20SwapValues = exports.queryEtherSwapValues = exports.queryERC20SwapValuesFromLock = exports.queryEtherSwapValuesFromLock = void 0;
const Errors_1 = __importDefault(require("./Errors"));
const EthereumUtils_1 = require("./EthereumUtils");
// TODO: what happens if the hash doesn't exist or the transaction isn't confirmed yet?
const queryEtherSwapValuesFromLock = async (provider, etherSwap, lockTransactionHash) => {
    const lockTransactionReceipt = await provider.getTransactionReceipt(lockTransactionHash);
    const topicHash = etherSwap.filters.Lockup().fragment.topicHash;
    if (lockTransactionReceipt) {
        for (const log of lockTransactionReceipt.logs) {
            if (log.topics[0] === topicHash) {
                const event = etherSwap.interface.parseLog(log);
                return (0, exports.formatEtherSwapValues)(event.args);
            }
        }
    }
    throw Errors_1.default.INVALID_LOCKUP_TRANSACTION(lockTransactionHash);
};
exports.queryEtherSwapValuesFromLock = queryEtherSwapValuesFromLock;
const queryERC20SwapValuesFromLock = async (provider, erc20Swap, lockTransactionHash) => {
    const lockTransactionReceipt = await provider.getTransactionReceipt(lockTransactionHash);
    const topicHash = erc20Swap.filters.Lockup().fragment.topicHash;
    if (lockTransactionReceipt) {
        for (const log of lockTransactionReceipt.logs) {
            if (log.topics[0] === topicHash) {
                const event = erc20Swap.interface.parseLog(log);
                return (0, exports.formatERC20SwapValues)(event.args);
            }
        }
    }
    throw Errors_1.default.INVALID_LOCKUP_TRANSACTION(lockTransactionHash);
};
exports.queryERC20SwapValuesFromLock = queryERC20SwapValuesFromLock;
const queryEtherSwapValues = async (etherSwap, preimageHash, startHeight) => {
    const events = await etherSwap.queryFilter(etherSwap.filters.Lockup(preimageHash), startHeight);
    if (events.length === 0) {
        throw Errors_1.default.NO_LOCKUP_FOUND();
    }
    const event = events[0];
    return (0, exports.formatEtherSwapValues)(event.args);
};
exports.queryEtherSwapValues = queryEtherSwapValues;
const queryERC20SwapValues = async (erc20Swap, preimageHash, startHeight) => {
    const events = await erc20Swap.queryFilter(erc20Swap.filters.Lockup(preimageHash), startHeight);
    if (events.length === 0) {
        throw Errors_1.default.NO_LOCKUP_FOUND();
    }
    const event = events[0];
    return (0, exports.formatERC20SwapValues)(event.args);
};
exports.queryERC20SwapValues = queryERC20SwapValues;
const formatEtherSwapValues = (args) => {
    return {
        amount: args.amount,
        claimAddress: args.claimAddress,
        refundAddress: args.refundAddress,
        timelock: Number(args.timelock),
        preimageHash: (0, EthereumUtils_1.parseBuffer)(args.preimageHash),
    };
};
exports.formatEtherSwapValues = formatEtherSwapValues;
const formatERC20SwapValues = (args) => {
    return {
        amount: args.amount,
        claimAddress: args.claimAddress,
        tokenAddress: args.tokenAddress,
        refundAddress: args.refundAddress,
        timelock: Number(args.timelock),
        preimageHash: (0, EthereumUtils_1.parseBuffer)(args.preimageHash),
    };
};
exports.formatERC20SwapValues = formatERC20SwapValues;
//# sourceMappingURL=ContractUtils.js.map