"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../../Utils");
const Enums_1 = require("../../consts/Enums");
exports.default = {
    NO_PROVIDER_SPECIFIED: () => ({
        message: 'no RPC provider was specified',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Ethereum, 0),
    }),
    NO_LOCKUP_FOUND: () => ({
        message: 'no lockup transaction found',
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Ethereum, 1),
    }),
    INVALID_LOCKUP_TRANSACTION: (transactionHash) => ({
        message: `lockup transaction is invalid: ${transactionHash}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Ethereum, 2),
    }),
    UNEQUAL_PROVIDER_NETWORKS: (networks) => {
        const networkStrings = [];
        networks.forEach((network) => networkStrings.push(Number(network.chainId)));
        return {
            message: `not all RPC provider networks are equal: ${networkStrings.join(', ')}`,
            code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Ethereum, 3),
        };
    },
    REQUESTS_TO_PROVIDERS_FAILED: (errors) => ({
        message: `requests to all providers failed:\n - ${errors.join('\n - ')}`,
        code: (0, Utils_1.concatErrorCode)(Enums_1.ErrorCodePrefix.Ethereum, 4),
    }),
};
//# sourceMappingURL=Errors.js.map