import { ERC20Swap } from 'boltz-core/typechain/ERC20Swap';
import { EtherSwap } from 'boltz-core/typechain/EtherSwap';
import { Signer } from 'ethers';
import { EthereumConfig, RskConfig } from '../../Config';
import Logger from '../../Logger';
import Wallet from '../Wallet';
import ContractEventHandler from './ContractEventHandler';
import ContractHandler from './ContractHandler';
import { NetworkDetails } from './EvmNetworks';
import InjectedProvider from './InjectedProvider';
type Network = {
    chainId: bigint;
    name?: string;
};
declare class EthereumManager {
    private readonly logger;
    private static supportedContractVersions;
    readonly provider: InjectedProvider;
    readonly contractHandler: ContractHandler;
    readonly contractEventHandler: ContractEventHandler;
    etherSwap: EtherSwap;
    erc20Swap: ERC20Swap;
    signer: Signer;
    address: string;
    network: Network;
    readonly networkDetails: NetworkDetails;
    readonly tokenAddresses: Map<string, string>;
    private readonly config;
    constructor(logger: Logger, isRsk: boolean, config?: RskConfig | EthereumConfig);
    init: (mnemonic: string) => Promise<Map<string, Wallet>>;
    destroy: () => Promise<void>;
    getContractDetails: () => Promise<{
        network: {
            chainId: number;
            name: string | undefined;
        };
        tokens: Map<string, string>;
        swapContracts: Map<string, string>;
    }>;
    hasSymbol: (symbol: string) => boolean;
    private checkERC20Allowance;
    private checkContractVersion;
}
export default EthereumManager;
