"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const PendingEthereumTransactionRepository_1 = __importDefault(require("../../db/repositories/PendingEthereumTransactionRepository"));
class EthereumTransactionTracker {
    constructor(logger, networkDetails, provider, wallet) {
        this.logger = logger;
        this.networkDetails = networkDetails;
        this.provider = provider;
        this.wallet = wallet;
        this.init = async () => {
            this.logger.info(`Starting ${this.networkDetails.name} transaction tracker for address: ${await this.wallet.getAddress()}`);
            await this.scanPendingTransactions();
        };
        /**
         * Scans a block and removes pending transactions from the database in case they were confirmed
         * This method is public and gets called from "EthereumManager" because there is a block subscription
         * in that class already
         */
        this.scanPendingTransactions = async () => {
            for (const transaction of await PendingEthereumTransactionRepository_1.default.getTransactions()) {
                const receipt = await this.provider.getTransactionReceipt(transaction.hash);
                if (receipt && (await receipt.confirmations()) > 0) {
                    this.logger.silly(`Removing confirmed ${this.networkDetails.name} transaction: ${transaction.hash}`);
                    await transaction.destroy();
                }
            }
        };
    }
}
exports.default = EthereumTransactionTracker;
//# sourceMappingURL=EthereumTransactionTracker.js.map