import { AddressLike, BigNumberish, Block, BlockTag, FeeData, Filter, Listener, Log, Network, Provider, ProviderEvent, TransactionReceipt, TransactionRequest, TransactionResponse } from 'ethers';
import { EthereumConfig, RskConfig } from '../../Config';
import Logger from '../../Logger';
import { NetworkDetails } from './EvmNetworks';
declare enum EthProviderService {
    Node = "Node",
    Infura = "Infura",
    Alchemy = "Alchemy"
}
/**
 * This provider is a wrapper for the JsonRpcProvider of ethers, but it writes sent transactions to the database
 * and, depending on the configuration, falls back to Alchemy and Infura as Web3 provider
 */
declare class InjectedProvider implements Provider {
    private readonly logger;
    private readonly networkDetails;
    readonly provider: this;
    private providers;
    private network;
    private static readonly requestTimeout;
    constructor(logger: Logger, networkDetails: NetworkDetails, config: RskConfig | EthereumConfig);
    init: () => Promise<void>;
    private addEthProvider;
    destroy: () => void;
    call: (transaction: TransactionRequest, blockTag?: BlockTag) => Promise<string>;
    estimateGas: (transaction: TransactionRequest) => Promise<bigint>;
    getBalance: (addressOrName: string, blockTag?: BlockTag) => Promise<bigint>;
    getBlock: (blockHashOrBlockTag: BlockTag) => Promise<Block>;
    getBlockNumber: () => Promise<number>;
    getCode: (addressOrName: string, blockTag?: BlockTag) => Promise<string>;
    getStorage: (address: AddressLike, position: BigNumberish, blockTag?: BlockTag | undefined) => Promise<string>;
    getFeeData: () => Promise<FeeData>;
    getLogs: (filter: Filter) => Promise<Array<Log>>;
    getNetwork: () => Promise<Network>;
    getStorageAt: (addressOrName: string, position: BigNumberish, blockTag?: BlockTag) => Promise<string>;
    getTransaction: (transactionHash: string) => Promise<TransactionResponse | null>;
    getTransactionCount: (addressOrName: string, blockTag?: BlockTag) => Promise<number>;
    getTransactionReceipt: (transactionHash: string) => Promise<TransactionReceipt | null>;
    getTransactionResult: (transactionHash: string) => Promise<string | null>;
    lookupAddress: (address: string) => Promise<string>;
    resolveName: (name: string) => Promise<string>;
    broadcastTransaction: (signedTransaction: string) => Promise<TransactionResponse>;
    sendTransaction: (tx: TransactionRequest) => Promise<TransactionResponse>;
    waitForTransaction: (transactionHash: string, confirmations?: number, timeout?: number) => Promise<TransactionReceipt>;
    waitForBlock: (blockTag?: BlockTag) => Promise<Block>;
    emit: (eventName: ProviderEvent, ...args: Array<any>) => Promise<boolean>;
    addListener: (eventName: ProviderEvent, listener: Listener) => Promise<this>;
    listenerCount: (eventName?: ProviderEvent) => Promise<number>;
    listeners: (eventName?: ProviderEvent) => Promise<Array<Listener>>;
    off: (eventName: ProviderEvent, listener?: Listener) => Promise<this>;
    on: (eventName: ProviderEvent, listener: Listener) => Promise<this>;
    once: (eventName: ProviderEvent, listener: Listener) => Promise<this>;
    removeAllListeners: (eventName?: ProviderEvent) => Promise<this>;
    removeListener: (eventName: ProviderEvent, listener: Listener) => Promise<this>;
    private forwardMethod;
    private forwardMethodNullable;
    private promiseWithTimeout;
    private addToTransactionDatabase;
    private hashCode;
    private logAddedProvider;
    private logConnectedProvider;
    private logDisabledProvider;
}
export default InjectedProvider;
export { EthProviderService };
