"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bitcoinjs_lib_1 = require("bitcoinjs-lib");
const Utils_1 = require("../../Utils");
const ChainClient_1 = __importStar(require("../../chain/ChainClient"));
class CoreWalletProvider {
    constructor(logger, chainClient) {
        this.logger = logger;
        this.chainClient = chainClient;
        this.serviceName = () => {
            return 'Core';
        };
        this.getAddress = (type = ChainClient_1.AddressType.Taproot) => {
            return this.chainClient.getNewAddress(type);
        };
        this.getBalance = async () => {
            const utxos = await this.chainClient.listUnspent(0);
            let confirmed = BigInt(0);
            let unconfirmed = BigInt(0);
            utxos.forEach((utxo) => {
                const amount = BigInt(Math.round(utxo.amount * ChainClient_1.default.decimals));
                if (utxo.confirmations > 0) {
                    confirmed += amount;
                }
                else {
                    unconfirmed += amount;
                }
            });
            return {
                confirmedBalance: Number(confirmed),
                unconfirmedBalance: Number(unconfirmed),
            };
        };
        this.sendToAddress = async (address, amount, satPerVbyte) => {
            const transactionId = await this.chainClient.sendToAddress(address, amount, await this.getFeePerVbyte(satPerVbyte));
            return await this.handleCoreTransaction(transactionId, address);
        };
        this.sweepWallet = async (address, satPerVbyte) => {
            const { confirmedBalance } = await this.getBalance();
            const transactionId = await this.chainClient.sendToAddress(address, confirmedBalance, await this.getFeePerVbyte(satPerVbyte), true);
            return await this.handleCoreTransaction(transactionId, address);
        };
        this.handleCoreTransaction = async (transactionId, address) => {
            const rawTransactionVerbose = await this.chainClient.getRawTransactionVerbose(transactionId);
            const rawTransaction = bitcoinjs_lib_1.Transaction.fromHex(rawTransactionVerbose.hex);
            let vout = 0;
            let outputSum = BigInt(0);
            for (let i = 0; i < rawTransaction.outs.length; i += 1) {
                outputSum += BigInt(rawTransaction.outs[i].value);
                const scriptPubKey = rawTransactionVerbose.vout[i].scriptPubKey;
                if (scriptPubKey.address === address ||
                    (scriptPubKey.addresses && scriptPubKey.addresses.includes(address))) {
                    vout = i;
                }
            }
            // Fetch all input transactions before processing them to avoid fetching one twice
            const fetchedTransaction = new Map();
            for (const input of rawTransactionVerbose.vin) {
                if (!fetchedTransaction.has(input.txid)) {
                    fetchedTransaction.set(input.txid, await this.chainClient.getRawTransaction(input.txid));
                }
            }
            let inputSum = BigInt(0);
            for (const input of rawTransaction.ins) {
                const inputTransactionId = (0, Utils_1.transactionHashToId)(input.hash);
                const inputTransaction = bitcoinjs_lib_1.Transaction.fromHex(fetchedTransaction.get(inputTransactionId));
                const inputVout = inputTransaction.outs[input.index];
                inputSum += BigInt(inputVout.value);
            }
            return {
                vout,
                transactionId,
                transaction: rawTransaction,
                fee: Math.ceil(Number(inputSum - outputSum)),
            };
        };
        this.getFeePerVbyte = async (satPerVbyte) => {
            return satPerVbyte || (await this.chainClient.estimateFee());
        };
        this.symbol = chainClient.symbol;
        this.logger.info(`Initialized ${this.symbol} Core wallet`);
    }
}
exports.default = CoreWalletProvider;
//# sourceMappingURL=CoreWalletProvider.js.map