"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EthereumUtils_1 = require("../ethereum/EthereumUtils");
class EtherWalletProvider {
    constructor(logger, signer, networkDetails) {
        this.logger = logger;
        this.signer = signer;
        // The gas needed for sending Ether is 21000
        this.ethTransferGas = BigInt(21000);
        this.serviceName = () => {
            return 'Wallet';
        };
        this.getAddress = () => {
            return this.signer.getAddress();
        };
        this.getBalance = async () => {
            const balance = Number((await this.signer.provider.getBalance(await this.getAddress())) /
                this.decimals);
            return {
                confirmedBalance: balance,
                unconfirmedBalance: 0,
            };
        };
        this.sendToAddress = async (address, amount) => {
            const transaction = await this.signer.sendTransaction({
                to: address,
                value: BigInt(amount) * this.decimals,
                ...(await (0, EthereumUtils_1.getGasPrices)(this.signer.provider)),
            });
            return {
                transactionId: transaction.hash,
            };
        };
        this.sweepWallet = async (address) => {
            const balance = await this.signer.provider.getBalance(await this.getAddress());
            const { type, maxPriorityFeePerGas, maxFeePerGas } = await (0, EthereumUtils_1.getGasPrices)(this.signer.provider);
            const gasCost = this.ethTransferGas * BigInt(maxFeePerGas);
            const value = balance - gasCost;
            const transaction = await this.signer.sendTransaction({
                type,
                maxFeePerGas,
                maxPriorityFeePerGas,
                value,
                to: address,
                gasLimit: this.ethTransferGas,
            });
            return {
                transactionId: transaction.hash,
            };
        };
        this.symbol = networkDetails.symbol;
        this.decimals = networkDetails.decimals;
        this.logger.info(`Initialized ${this.symbol} wallet`);
    }
}
exports.default = EtherWalletProvider;
//# sourceMappingURL=EtherWalletProvider.js.map